/***************************************************************************
                          artistdialog.h  -  description
                             -------------------
    begin                : Tue Apr 10 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ARTISTDIALOG_H
#define ARTISTDIALOG_H

#include "include.h"
#include "artistwidget.h"
#include "kmusicdbvalidator.h"

/**This is the new/edit dialog for artists.
 * Use it whenever you need to get information from the user in a user friendly, safe way.
 *
 * This dialog IS non modal. Be careful!
 * @short New/Edit artist dialog
 * @author Will DeRousse
 *<pre>
 *artistDialog *ad=new artistDialog(parent, numtitles, okbuttontext, true, false);
 *ad->show();
 *</pre>
 */

class artistDialog : public KDialogBase
{
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 * @param QString Caption
		 * @param QString OK Button Text
		 * @param bool Demise Mirrors Conception
		 */
		artistDialog(QWidget *, const QString &, const QString &,
		const bool &);
		/**
		 * Destructor
		 */
		~artistDialog();
		/**
		 * Returns a pointer to artistWidget.
		 */
		artistWidget *widget() const;
	protected slots:
		/**
		 * Reimplemented. Closes dialog without doing anything.
		 */
		void slotCancel();
		/** Reimplementation of slotOK().
		 *
		 * Makes sure that the values were entered.
		 * Emits addArtist(artistInfo).
		 */
		void slotOk();
	signals:
		/**
		 * Emitted so the artist info can be passed to a different function.
		 * @param artistInfo Struct containing artist information
		 */
		void addArtist(artistInfo);
	private:
		artistWidget *artist;
		artistInfo test;
};


#endif

