/***************************************************************************
                          artistwidget.cpp  -  description
                             -------------------
    begin                : Tue Apr 10 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "artistwidget.h"

/**
 * Constructor
 *
 * @param QWidget Parent
 * @param int Spacing
 * @param bool Demise Mirrors Conception
 */
artistWidget::artistWidget(QWidget * parent, const int &spacing,
	const bool & b): QGridLayout(parent, 7, 4, 0, spacing)
{
	cerr << "artistWidget::initialized" << endl;
	label = new QLabel(i18n("Artist"), parent);
	addWidget(label, 0, 0);
	name = new KLineEdit("", parent);
	name->setMaxLength(40);
	QToolTip::add(name, i18n("Enter the name of the artist or band"));
	addMultiCellWidget(name, 0, 0, 1, 3);

	label = new QLabel(i18n("Conception"), parent);
	addWidget(label, 1, 0);
	begin = new KLineEdit("", parent);
	begin->setMaxLength(4);
	begin->setMinimumWidth(begin->fontMetrics().width("xxxx") + 20);
	begin->setMaximumWidth(begin->fontMetrics().width("xxxx") + 20);
	QToolTip::add(begin, i18n("Year they formed or first release"));
	addWidget(begin, 1, 1);
	label = new QLabel(i18n("Demise"), parent);
	label->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	addWidget(label, 1, 2);
	end = new KLineEdit("", parent);
	end->setMaxLength(4);
	end->setMinimumWidth(begin->fontMetrics().width("xxxx") + 20);
	end->setMaximumWidth(begin->fontMetrics().width("xxxx") + 20);
	QToolTip::add(end, i18n("Year they broke up or last release"));
	addWidget(end, 1, 3);

	label = new QLabel(i18n("Follow?"), parent);
	addWidget(label, 3, 0);
	follow = new QCheckBox("", parent);
	QToolTip::add(follow, i18n("Do you still collect this artist?"));
	addWidget(follow, 3, 1);
	label = new QLabel(i18n("No. of Releases: "), parent);
	addWidget(label, 3, 2);
	releases = new QLabel("", parent);
	addWidget(releases, 3, 3);

	label = new QLabel(i18n("Homepage"), parent);
	addWidget(label, 4, 0);
	homepage = new KLineEdit("", parent);
	homepage->setMaxLength(65);
	QToolTip::add(homepage,i18n("Put the web address here, and click the button to the right to open."));
	addMultiCellWidget(homepage, 4, 4, 1, 2);
	openHomepage = new QPushButton(parent);
	openHomepage->setPixmap(SmallIcon("konqueror"));
	QToolTip::add(openHomepage,i18n("Click to open homepage in your default browser"));
	addWidget(openHomepage, 4, 3);

	label = new QLabel(i18n("Notes"), parent);
	addWidget(label, 5, 0);
	notes = new QTextEdit(parent);
	notes->setWordWrap(QTextEdit::WidgetWidth);
	addMultiCellWidget(notes, 6, 6, 0, 3);

	if (b)
		connect(begin, SIGNAL(textChanged(const QString &)), end, SLOT(setText(const QString &)));
	connect(openHomepage, SIGNAL(clicked()), this, SLOT(openBrowser()));
	name->setFocus();
}

/**
 * Destructor
 */
artistWidget::~artistWidget()
{
}

/**
 * Returns the table aid.
 */
QString
artistWidget::getAid() const
{
	return aid;
}

/**
 * Retrieve value of year of conception.
 */
QString
artistWidget::getFirst() const
{
	return begin->text();
}

/** Retrieve value of follow. */
bool
artistWidget::getFollow() const
{
	return follow->isChecked();
}

/** Retrieves homepage. */
QString
artistWidget::getHomepage() const
{
	return homepage->text();
}

/**
 * Retrieve value of year of demise.
 */
QString
artistWidget::getLast() const
{
	return end->text();
}

/** Retrieve value of name. */
QString
artistWidget::getName() const
{
	return name->text();
}

/** Retrieve value of notes. */
QString
artistWidget::getNotes() const
{
	return notes->text();
}

/** Grays or ungrays the entire widget.
 * @param bool TRUE enables, FALSE disables
 */
void
artistWidget::enable(bool t) const
{
	begin->setEnabled(t);
	end->setEnabled(t);
	follow->setEnabled(t);
	name->setEnabled(t);
	notes->setEnabled(t);
	homepage->setEnabled(t);
	openHomepage->setEnabled(t);
}

/** This slot takes the homepage string and passes it to konqueror. */
void
artistWidget::openBrowser()
{
	cerr << "artistWidget::opening web browser: ";
	KProcess k;
	QString browser = getenv("BROWSER");
	if (browser.isNull())
		browser = "konqueror";
	cerr << browser << endl;
	k << browser;
//	k.setExecutable(browser);
	k << homepage->text();
	k.start(KProcess::DontCare);
}
/**
 *Passes artistInfo through to the widget.
 * @param artistInfo Pointer to artist info
 * @param bool Fill year conception with current year
 * @param bool Whether demise copies conception
 */
void artistWidget::updateInfo(artistInfo *z, const bool &fillyear, const bool &c3p0)
{
	name->setText(z->name);

	if (fillyear)
		begin->setText(z->first.setNum(QDate::currentDate().year()));
	else
		begin->setText(z->first);

	if (c3p0)
		end->setText(z->first);
	else
		end->setText(z->last);

	homepage->setText(z->homepage);

	follow->setChecked(z->follow);

	notes->setText(z->notes);
	aid = z->aid;
}
/**
 * Updates the number of titles.
 * @param QString Number Of Titles
 */
void artistWidget::updateTitles(const QString &number) const
{
	releases->setText(number);
}

#include "artistwidget.moc"
