/***************************************************************************
                          artistwidget.h  -  description
                             -------------------
    begin                : Tue Apr 10 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ARTISTWIDGET_H
#define ARTISTWIDGET_H

#include "include.h"

/**This is the main widget for artists. It is a QGridLayout containing the artistInfo values.
 *
 *Embed it anywhere you need it (dialog, preferences box).
 * @author Will DeRousse
 * @short Add/Edit Widget for artist information.
 *<pre>
 *QFrame *page=new QFrame();
 *artistWidget *artist=new artistWidget(page, spacing(), true, true);
 *</pre>
 */

class artistWidget : public QGridLayout  {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 *
		 * @param QWidget Parent
		 * @param int Spacing
		 * @param bool Demise Mirrors Conception
		 */
		artistWidget(QWidget *, const int &, const bool &);
		/**
		 * Destructor
		 */
		~artistWidget();
		/**
		 * Returns the table aid.
		 */
		QString getAid() const;
		/**
		 * Retrieve value of year of conception.
		 */
		QString getFirst() const;
		/**
		 *Retrieve value of follow.
		 */
		bool getFollow() const;
		/**
		 *Retrieves homepage.
		 */
		QString getHomepage() const;
		/**
		 * Retrieve value of year of demise.
		 */
		QString getLast() const;
		/**
		 * Retrieve value of name.
		 */
		QString getName() const;
		/**
		 * Retrieve value of notes.
		 */
		QString getNotes() const;
		/**
		 * Passes artistInfo through to the widget.
		 * @param artistInfo Pointer to artist info
		 * @param bool Fill year conception with current year
		 * @param bool Whether demise mirrors conception
		 */
		void updateInfo(artistInfo *, const bool &, const bool &);
		/**
		 * Updates the number of titles.
		 * @param QString Number Of Titles
		 */
		void updateTitles(const QString &) const;
	public slots:
		/** Grays or ungrays the entire widget.
		 * @param bool TRUE enables, FALSE disables
		 */
		void enable(bool=false) const;
	private slots:
		/**
		 *This slot takes the homepage string and passes it to konqueror.
		 */
		void openBrowser();
	private:
		QLabel *label, *releases;
		KLineEdit *name,*begin,*end, *homepage;
		QCheckBox *follow;
		QTextEdit *notes;
		QString aid;
		QPushButton *openHomepage;
};


#endif

