/***************************************************************************
                          categorydialog.cpp  -  description
                             -------------------
    begin                : Tue May 22 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "categorydialog.h"

/**
 * Constructor
 *
 * @param QWidget Parent
 * @param int Category Type
 * @param QString Caption
 * @param bool Default Sort Order
 */
categoryDialog::categoryDialog(QWidget * parent, const int &type,
	const QString &caption, const bool &s):
KDialogBase(Plain, caption, Ok | Close | User1 | User2 | User3, Ok, parent,
			 "category", false, false)
{
	cerr << "categoryDialog::initialized" << endl;
	cati.ctype = type;
	QFrame *page = plainPage();
	setButtonBoxOrientation(Vertical);
	setButtonText(Ok, i18n("&Add"));
	setButtonText(User1, i18n("&Update"));
	setButtonText(User2, i18n("&Delete"));
	setButtonTip(Ok, i18n("Add item to the list"));
	setButtonTip(User1, i18n("Update The Item"));
	setButtonTip(User2, i18n("Remove item from the list"));
	cat = new categoryWidget(page, spacingHint(), s);

	enableButton(Ok, FALSE);
	enableButtons(FALSE);

	connect(cat, SIGNAL(nameTextChanged(const QString &)), this, SLOT(enableAdd(const QString &)));
	connect(cat->list(), SIGNAL(dropped(QString)), this, SLOT(droppedText(QString)));
	connect(cat->list(),SIGNAL(imFocused(bool)),this,SLOT(enableButtons(bool)));

	if (type == KBTE)
	{
		showButton(User3, TRUE);
		setButtonText(User3, i18n("&Transfer"));
		setButtonTip(User3, i18n("Convert from explore to artist"));
	}
	else
		showButton(User3, FALSE);
}

/**
 * Destructor
 */
categoryDialog::~categoryDialog()
{
}

/**
 * Used for enabling the GUI buttons that are not "Add".
 * @param bool TRUE enables, FALSE disables
 */
void categoryDialog::enableButtons(bool z)
{
	enableButton(User1, z);
	enableButton(User2, z);
	enableButton(User3, z);
}
/** Handles drag and drop.
 * @param QString Text Dropped
 */
void categoryDialog::droppedText(QString d)
{
	cerr << "categoryDialog::dnd drop: " << d << endl;
	cati.name=d;
	emit addSignal(cati);
}

/** Used for enabling or disabling the "Add" button.
 * @param QString KLineEdit Text
 */
void
categoryDialog::enableAdd(const QString &t)
{
	enableButton(Ok, !t.isEmpty());
}

/** Returns a pointer to the listview of the widget. */
categoryWidget *categoryDialog::widget() const
{
	return cat;
}

/**
 * Re-implemented resize event makes sure the name is always displayed.
 * @param QResizeEvent Information pertaining to event
 */
void
categoryDialog::resizeEvent(QResizeEvent *)
{
	cerr << "categoryDialog::resize event" << endl;
	cat->list()->setColumnWidth(0, cat->list()->width()-9);
}

/**
 * Closes the dialog. Emits signalClose(int).
 */
void
categoryDialog::slotClose()
{
	emit signalClose(cati.ctype);
	delayedDestruct();
}

/**
 * Re-implemented for use as "Add". Emits addSignal(categoryInfo).
 */
void
categoryDialog::slotOk()
{
	cati.name=cat->getName();
	emit addSignal(cati);
	cat->clearName();
}

/**
 * Re-implemented for use as "Update". Emits updateSignal(categoryInfo, categoryInfo).
 */
void
categoryDialog::slotUser1()
{
	cati.xid=cat->getListXID();
	cati.name=cat->getName();
	emit updateSignal(cati);
}

/**
 * Re-implemented for use as Delete. Emits deleteSignal(categoryInfo).
 */
void
categoryDialog::slotUser2()
{
	cati.xid=cat->getListXID();
	emit deleteSignal(cati);
}

/**
 * Re-implemented for use as Transfer for Bands To Explore. Emits transferArtist(categoryInfo).
 */
void
categoryDialog::slotUser3()
{
	cati.name=cat->getName();
	cati.xid=cat->getListXID();
	emit transferArtist(cati);
}

#include "categorydialog.moc"
