/***************************************************************************
                          categorydialog.h  -  description
                             -------------------
    begin                : Tue May 22 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CATEGORYDIALOG_H
#define CATEGORYDIALOG_H

#include "include.h"
#include "categorywidget.h"

/** This is the category dialog for all categories and Bands To Explore. It contains a Close, Add, Remove and Delete button (right aligned).
 * Transfer only appears if KBTE is set as dialog type.
 * This is NON MODAL, so be careful with your implementation.
 * @short Category Dialog
 * @author Will DeRousse
 *<pre>
 *categoryDialog *w=new categoryDialog(this , KTYPE, "caption", true);
 *w->show();
 *</pre>
 */

class categoryDialog:public KDialogBase
{
	Q_OBJECT
	public:
		/**
		 * Constructor
		 *
		 * @param QWidget Parent
		 * @param int Category Type
		 * @param QString Caption
		 * @param bool Default Sort Order
		 */
		categoryDialog(QWidget *, const int &, const QString &, const bool &);
		/**
		 * Destructor
		 */
		~categoryDialog();
		/**
		 * Returns a pointer to the categoryWidget.
		 */
		categoryWidget *widget() const;
	protected slots:
		/**
		 * Re-implemented resize event makes sure the name is always displayed.
		 * @param QResizeEvent Information pertaining to event
		 */
		void resizeEvent(QResizeEvent *);
		/**
		 * Closes the dialog. Emits signalClose(int).
		 */
		void slotClose();
		/**
		 * Re-implemented for use as "Add". Emits addSignal(categoryInfo).
		 */
		void slotOk();
		/**
		 * Re-implemented for use as Update. Emits updateSignal(categoryInfo, categoryInfo).
		 */
		void slotUser1();
		/**
		 * Re-implemented for use as Delete. Emits deleteSignal(categoryInfo).
		 */
		void slotUser2();
		/**
		 * Re-implemented for use as Transfer for Bands To Explore. Emits transferArtist(categoryInfo).
		 */
		void slotUser3();
	private slots:
		/** Handles drag and drop.
		 * @param QString Text Dropped
		 */
		void droppedText(QString);
		/** Used for enabling or disabling the "Add" button.
		 * @param QString KLineEdit Text
		 */
		void enableAdd(const QString &);
		/** Used for enabling the GUI buttons that are not "Add".
		 * @param bool TRUE enables, FALSE disables
		 */
		void enableButtons(bool);
	signals:
		/** Emitted when add button is clicked.
		 * @param categoryInfo Struct filled with category information
		 */
		void addSignal(categoryInfo);
		/** Emitted when delete button is clicked.
		 * @param categoryInfo Struct filled with category information
		*/
		void deleteSignal(categoryInfo);
		/** Emitted when close button is clicked.
		 * @param int Category Type
		 */
		void signalClose(int);
		/** Emitted when transfer button is clicked.
		 * @param categoryInfo Struct filled with category information
		*/
		void transferArtist(categoryInfo);
		/** Emitted when update button is clicked.
		 * @param categoryInfo Struct with XID and new name.
		 */
		void updateSignal(categoryInfo);
	private:
		categoryWidget * cat;
		categoryInfo cati;
};


#endif

