/***************************************************************************
                          categorywidget.cpp  -  description
                             -------------------
    begin                : Tue May 22 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "categorywidget.h"

/**
 * Constructor
 *
 * @param QWidget Parent
 * @param int Category Type
 * @param bool Default Sort Order
 */
categoryWidget::categoryWidget(QWidget *parent, const int &spacing, const bool &sort):
QGridLayout(parent, 2, 1, 0, spacing)
{
	cerr << "categoryWidget::initialized" << endl;
	name = new KLineEdit("", parent);
	name->setMaxLength(40);
	list2 = new musicQListView(parent, 1);
	list2->addColumn(i18n("Name"));
	list2->setColumnWidthMode(0, QListView::Maximum);
	list2->setColumnWidth(0, 180);
	list2->setMinimumWidth(180);
	list2->setMinimumHeight(260);
	addWidget(name, 0, 0);
	addWidget(list2, 1, 0);
	QToolTip::add(name, i18n("Edit Item Here"));
	QToolTip::add(list2, i18n("List Of Items"));

	list2->setSorting(0, sort);

	name->setFocus();

	connect(list2, SIGNAL(selectionChanged(QListViewItem *)), this, SLOT(updateName(QListViewItem *)));
	connect(name, SIGNAL(textChanged(const QString &)), this, SIGNAL(nameTextChanged(const QString &)));
}

/**
 * Destructor
 */
categoryWidget::~categoryWidget()
{
}
/**
 * Clears the QListView.
 */
void
categoryWidget::clearList() const
{
	list2->clear();
}
/**
 * Clears the KLineEdit.
 */
void
categoryWidget::clearName() const
{
	name->setText("");
}

/**
 * Returns a pointer to the QListView.
 */
musicQListView *categoryWidget::list() const
{
	return list2;
}
/**
 * Returns name from current selection in the QListView.
 */
QString
categoryWidget::getListName() const
{
	if (list2->selectedItem() == 0)
		return "";
	else
		return list2->selectedItem()->text(0);
}

/**
 * Returns XID.
 */
QString categoryWidget::getListXID() const
{
	if (list2->selectedItem() == 0)
		return "";
	else
		return list2->selectedItem()->text(1);
}

/**
 * Returns name.
 */
QString
categoryWidget::getName() const
{
	return name->text();
}

/** Updates the name information in musicQListViewItem.
 * @param musicQListViewItem Pointer to name to update
 */
void
categoryWidget::updateName(QListViewItem *t)
{
	if (t != NULL)
		name->setText(t->text(0));
	else
		name->setText("");
	name->setFocus();
}

#include "categorywidget.moc"
