/***************************************************************************
                          categorywidget.h  -  description
                             -------------------
    begin                : Tue May 22 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CATEGORYWIDGET_H
#define CATEGORYWIDGET_H

#include "include.h"
#include "musicqlistview.h"

/** This is the primary widget used for all category (and similar) boxes.
 *
 * There is a KLineEdit box on the top and a QListView on the bottom.
 * @short Category Widget
 * @author Will DeRousse
 *<pre>
 *categoryWidget *w=new categoryWidget(this, KTYPE, true);
 *w->show();
 *</pre>
 *The above snippet would implement a category widget for Type's and set the default sorting order to ascending.
 */

class categoryWidget : public QGridLayout  {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 *
		 * @param QWidget Parent
		 * @param int Category Type
		 * @param bool Default Sort Order
		 */
		categoryWidget(QWidget *, const int &, const bool &);
		/**
		 * Destructor
		 */
		~categoryWidget();
		/**
		 * Clears the QListView.
		 */
		void clearList() const;
		/**
		 * Clears the KLineEdit.
		 */
		void clearName() const;
		/**
		 * Returns name from current selection in the QListView.
		 */
		QString getListName() const;
		/**
		 * Returns XID.
		 */
		QString getListXID() const;
		/**
		 * Returns name.
		 */
		QString getName() const;
		/**
		 * Returns a pointer to the QListView.
		 */
		musicQListView *list() const;
	private slots:
		/** Updates the name information in QListViewItem.
		 * @param QListView Pointer to highlighted QListViewItem
		 */
		void updateName(QListViewItem *);
	signals:
		/** Emitted when the KLineEdit text changes.
		 * @param QString Name
		 */
		void nameTextChanged(const QString &);
	private:
		KLineEdit *name;
		musicQListView *list2;
};


#endif

