/***************************************************************************
                          cddbsearchdialog.h  -  description
                             -------------------
    begin                : Wed Dec 11 2002
    copyright            : (C) 2002 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CDDBSEARCHDIALOG_H
#define CDDBSEARCHDIALOG_H

#include "include.h"
#include "cddbsearchwidget.h"

/**
 * The CDDB Search Dialog. Displays a dialog with a Search Text LineEdit and two check boxes, one for artist and one
 * for title. There is a Search and Cancel Button.
 *
 *<pre>
 *CDDBSearchDialog *sd=new CDDBSearchDialog(parent);
 *sd->show();
 *</pre>
 * @short CDDB Search Dialog
 * @author Will DeRousse
 */

class CDDBSearchDialog : public KDialogBase
{
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 */
		CDDBSearchDialog(QWidget *);
		/**
		 * Destructor
		 */
		~CDDBSearchDialog();
	protected slots:
		/**
		 * Reimplemented. Closes dialog without doing anything.
		 * Emits cancelled(bool);
		 */
		void slotCancel();
		/** Reimplementation of slotOk().
		 *
		 * Button enables when values are filled in.
		 * Emits search(QString, bool, bool).
		 */
		void slotOk();
	signals:
		/**
		 * Emitted when cancelled is clicked.
		 */
		void cancelled();
		/**
		 * Emits search parameters.
		 * @param QString Search Text
		 * @param bool Artist Enabled Or Disabled
		 * @param bool Title Enabled Or Disabled
		 */
		void search(QString, bool, bool);
	private slots:
		/**
		 * Used to enable "Search" button.
		 * @param QString Text From Search LineEdit
		 */
		void enableButtons(const QString &);
	private:
		CDDBSearchWidget *sw;

};


#endif

