/***************************************************************************
                          cddbsearchresultsdialog.cpp  -  description
                             -------------------
    begin                : Wed Dec 11 2002
    copyright            : (C) 2002 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cddbsearchresultsdialog.h"

/**
 * Constructor
 * @param QWidget Parent
 */
CDDBSearchResultsDialog::CDDBSearchResultsDialog(QWidget *parent) : KDialogBase(Plain, i18n("CDDB Search"), Ok | Cancel, Ok, parent, "cddbsearchresults", false, false)
{
	cerr << "CDDBSearchDialog initialized" << endl;
	QFrame *page = plainPage();
	QGridLayout *layout=new QGridLayout(page, 1, 1, 0, spacingHint());
	v=new musicQListView(page, 0);
	v->addColumn(i18n("Artist"));
	v->addColumn(i18n("Title"));
	v->setMinimumWidth(300);
	v->setMinimumHeight(150);
	layout->addWidget(v,0,0);
	QToolTip::add(v, i18n("Search Results"));
	enableButtonOK(FALSE);
	setButtonText(Ok,i18n("Add"));
	setButtonTip(Ok, i18n("Add title to database"));

	connect(v,SIGNAL(imFocused(bool)),this,SLOT(enableButtons(bool)));
	connect(v,SIGNAL(doubleClicked(QListViewItem *)),this,SLOT(slotOk()));
}
/**
 * QWidget Parent
 */
CDDBSearchResultsDialog::~CDDBSearchResultsDialog(){
}
/**
 * Used to enable the "Search" button.
 * @param QString Text From Search LineEdit
 */
void CDDBSearchResultsDialog::enableButtons(bool t)
{
	enableButtonOK(t);
}
/**
 * Reimplemented. Closes dialog without doing anything.
 * Emits cancelled(bool).
 */
void CDDBSearchResultsDialog::slotCancel()
{
	emit cancelled();
	delayedDestruct();
}
/** Reimplementation of slotOk().
 *
 * Button enables when values are filled in.
 * Emits addTitle(QString, QString).
 */
void CDDBSearchResultsDialog::slotOk()
{
	hide();
	emit addTitle(v->selectedItem()->text(2),v->selectedItem()->text(3));
	delayedDestruct();
}
/**
 * Populates the dialog with the search results.
 * @param QStringList Artist
 * @param QStringList Title
 * @param QStringList CDDB Category
 * @param QStringList CDDB Disc ID
 */
void CDDBSearchResultsDialog::populateQListView(const QStringList &a, const QStringList &t, const QStringList &cat, const QStringList &cdid)
{
	int items=a.count();
	QListViewItem *item;

	v->clear();
	for (int i=0;i<items;i++)
	{
		item=new QListViewItem(v,a[i],t[i],cat[i],cdid[i]);
		if (!item)
			exit(1);
	}
}

#include "cddbsearchresultsdialog.moc"
