/***************************************************************************
                          cddbsearchresultsdialog.h  -  description
                             -------------------
    begin                : Wed Dec 11 2002
    copyright            : (C) 2002 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CDDBSEARCHRESULTSDIALOG_H
#define CDDBSEARCHRESULTSDIALOG_H

#include "include.h"
#include "musicqlistview.h"

/** Displays the results of a CDDB search in a 3 column QListView.
 *
 *<pre>
 *CDDBSearchResultsDialog *sd=new CDDBSearchResultsDialog(parent);
 *sd->show();
 *@author Will DeRousse
 */

class CDDBSearchResultsDialog : public KDialogBase  {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 */
		CDDBSearchResultsDialog(QWidget *);
		/**
		 * Destructor
		 */
		~CDDBSearchResultsDialog();
		/**
		 * Populates the dialog with the search results.
		 * @param QStringList Artist
		 * @param QStringList Title
		 * @param QStringList CDDB Category
		 * @param QStringList CDDB Disc ID
		 */
		void populateQListView(const QStringList &, const QStringList &, const QStringList &, const QStringList &);
	protected slots:
		/**
		 * Reimplemented. Closes dialog without doing anything.
		 * Emits cancelled(bool).
		 */
		void slotCancel();
		/** Reimplementation of slotOk().
		 *
		 * Button enables when values are filled in.
		 * Emits addTitle(QString, QString).
		 */
		void slotOk();
	signals:
		/**
		 * Emits CDDB information.
		 * @param QString CDDB Category
		 * @param QString CDDB Disc ID
		 */
		void addTitle(QString, QString);
		/**
		 * Emitted when cancelled is clicked.
		 */
		void cancelled();
	private slots:
		/**
		 * Used to enable the "Search" button.
		 * @param QString Text From Search LineEdit
		 */
		void enableButtons(bool);
	private:
		musicQListView *v;
};


#endif

