/***************************************************************************
                          cddbsearchwidget.cpp  -  description
                             -------------------
    begin                : Wed Dec 11 2002
    copyright            : (C) 2002 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cddbsearchwidget.h"

/**
 * Constructor
 *
 * @param QWidget Parent
 * @param int Spacing
 */
CDDBSearchWidget::CDDBSearchWidget(QWidget *parent, const int &spacing) : QGridLayout(parent, 2, 4, spacing)
{
	cerr << "CDDBSearchWidget::initialized" << endl;
	QLabel *label=new QLabel(i18n("Search: "),parent);
	search=new QLineEdit("",parent);
	addWidget(label, 0,0);
	addMultiCellWidget(search, 0,0,1,3);
	label=new QLabel(i18n("Artist"),parent);
	artist=new QCheckBox("",parent);
	addWidget(label,1,0);
	addWidget(artist,1,1);
	label=new QLabel(i18n("Title"),parent);
	title=new QCheckBox("",parent);
	addWidget(label,1,2);
	addWidget(title,1,3);

	artist->setChecked(TRUE);

	QToolTip::add(search, i18n("Enter search text"));
	QToolTip::add(artist, i18n("Check to search artists"));
	QToolTip::add(title, i18n("Check to search titles"));

	search->setFocus();

	connect(search,SIGNAL(textChanged(const QString &)),this,SIGNAL(textChanged(const QString &)));
}

/**
 * Destructor
 */
CDDBSearchWidget::~CDDBSearchWidget()
{
}

/**
 * Returns "Artist" checked or not.
 */
bool CDDBSearchWidget::getArtistChecked()
{
	return artist->isChecked();
}

/**
 * Returns "Title" checked or not.
 */
bool CDDBSearchWidget::getTitleChecked()
{
	return title->isChecked();
}

/**
 * Returns search text.
 */
QString CDDBSearchWidget::getSearchText()
{
	return search->text();
}

/**
 * Sets "Artist".
 * @param bool TRUE Checked, FALSE Unchecked
 */
void CDDBSearchWidget::setArtistChecked(const bool &t)
{
	artist->setChecked(t);
}

/**
 * Sets "Title".
 * @param bool TRUE Checked, FALSE Unchecked
 */
void CDDBSearchWidget::setTitleChecked(const bool &t)
{
	title->setChecked(t);
}

/**
 * Sets search text.
 * @param QString Search Text
 */
void CDDBSearchWidget::setSearchText(const QString &t)
{
	search->setText(t);
}

#include "cddbsearchwidget.moc"
