/***************************************************************************
                          dbconnectwidget.cpp  -  description
                             -------------------
    begin                : Tue Mar 27 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : root@localhost.localdomain
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dbconnectwidget.h"

/**
 * Constructor
 *
 * @param QWidget Parent
 * @param int Spacing
 */
dbConnectWidget::dbConnectWidget(QWidget *parent, const int &spacing):
QGridLayout(parent, 4, 4, 0, spacing)
{
	cerr << "dbConnectedWidget::initialized" << endl;
	label = new QLabel(i18n("Database"), parent);
	name = new KLineEdit("", parent);
	name->setMinimumWidth(name->fontMetrics().width("xxxxxxxxxxxx"));
	addWidget(label, 0, 0);
	addWidget(name, 0, 1);
	QToolTip::add(name, i18n("Enter the name of the database"));

	label = new QLabel(i18n("Port"), parent);
	addWidget(label, 0, 2);
	port = new KLineEdit("", parent);
	port->setMaxLength(4);
	port->setMinimumWidth(port->fontMetrics().width("xxxx") + 15);
	port->setMaximumWidth(port->fontMetrics().width("xxxx") + 15);
	addWidget(port, 0, 3);
	QToolTip::add(port, i18n("Enter the port number. Default is 5432."));

	label = new QLabel(i18n("Location"), parent);
	addWidget(label, 1, 0);
	location = new KLineEdit("", parent);
	addMultiCellWidget(location, 1, 1, 1, 3);
	QToolTip::add(location,i18n("Enter the location (i.e. localhost, 192.168.0.1)"));

	label = new QLabel(i18n("Username"), parent);
	addWidget(label, 2, 0);
	user = new KLineEdit("", parent);
	addMultiCellWidget(user, 2, 2, 1, 3);
	QToolTip::add(user,i18n("Enter your database username (i.e. root, blank)"));

	label = new QLabel(i18n("Password"), parent);
	addWidget(label, 3, 0);
	passwd = new KLineEdit("", parent);
	passwd->setEchoMode(QLineEdit::Password);
	addMultiCellWidget(passwd, 3, 3, 1, 3);
	QToolTip::add(passwd, i18n("Enter your password if you have one"));

	name->setFocus();
}

/**
 * Destructor
 */
dbConnectWidget::~dbConnectWidget()
{
}

/** Returns the location. */
QString
dbConnectWidget::getLocation() const
{
	return location->text();
}

/** Returns the name. */
QString
dbConnectWidget::getName() const
{
	return name->text();
}

/** Returns the password. */
QString
dbConnectWidget::getPasswd() const
{
	return passwd->text();
}

/** Returns the port. */
QString
dbConnectWidget::getPort() const
{
	return port->text();
}

/** Returns the username. */
QString
dbConnectWidget::getUser() const
{
	return user->text();
}

/**
 * Sets all the information in the widget.
 * @param dbInfo Pointer to dbInfo struct
 */
void dbConnectWidget::updateInfo(const dbInfo *i) const
{
	location->setText(i->db_location);
	name->setText(i->db_name);
	port->setText(i->db_port);
	user->setText(i->db_user);
	passwd->setText(i->db_passwd);
}

/** Grays or ungrays the widget.
 * @param QString TRUE enable, FALSE disables
 */
void
dbConnectWidget::enable(bool t) const
{
	location->setEnabled(t);
	name->setEnabled(t);
	port->setEnabled(t);
	user->setEnabled(t);
	passwd->setEnabled(t);
}

#include "dbconnectwidget.moc"
