/***************************************************************************
                          dbConnectWidget.h  -  description
                             -------------------
    begin                : Tue Mar 27 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DBCONNECTWIDGET_H
#define DBCONNECTWIDGET_H

#include "include.h"

/**This class is a QGridLayout containing the widgets for getting database connection info.
 *
 * Embed it anywhere you need to.
 * @author Will DeRousse
 * @short Database connection widget.
 *<pre>
 *QFrame *page=new QFrame();
 *dbConnectWidget *dbconnect=new dbConnectWidget(this,spacing);
 *</pre>
 */

class dbConnectWidget : public QGridLayout  {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 *
		 * @param QWidget Parent
		 * @param int Spacing
		 */
		dbConnectWidget(QWidget *, const int &);
		/**
		 * Destructor
		 */
		~dbConnectWidget();
		/**
		 * Returns the location.
		 */
		QString getLocation() const;
		/**
		 * Returns the name.
		 */
		QString getName() const;
		/**
		 * Returns the password.
		 */
		QString getPasswd() const;
		/**
		 * Returns the port.
		 */
		QString getPort() const;
		/**
		 * Returns the username.
		 */
		QString getUser() const;
		/**
		 * Sets all the information in the widget.
		 * @param dbInfo Pointer to dbInfo struct
		 */
		void updateInfo(const dbInfo *) const;
	public slots:
		/** Grays or ungrays the widget.
		 * @param QString TRUE enable, FALSE disables
		 */
		void enable(bool=false) const;
	private:
		QLabel *label;
		KLineEdit *location, *name, *passwd, *port, *user;
};


#endif

