/***************************************************************************
                          dbinitgui.cpp  -  description
                             -------------------
    begin                : Fri Jan 25 2002
    copyright            : (C) 2002 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmusicdb.h"

/**
 * Runs the initial checks on the db to insure that it's up and working properly.
 *
 * @param QString Connect string (see @ref returnConnectInfo).
 */
void
kmusicdb::initDb(const QString &dirt)
{
	if (dirt.isEmpty())
		return;
	cerr << "kmusicdb::initdb: Connecting to template1" << endl;

	updateStatusBar(i18n("Connecting to server..."));
	updateProgress(20);
	init = new dbInit(dirt);
	if (!init)
		exit(1);
	updateStatusBar(i18n("Checking connection to server..."));
	if (init->checkConnect())
	{
		updateProgress(40);
	}
	else
	{
		errBox(DB_CONNECTION_BAD);
		goto cleanup;
	}
	updateStatusBar(i18n("Verifying quality of connection to server..."));
	if (init->testConnect())
		updateProgress(60);
	else
	{
		errBox(DB_QUALITY_BAD);
		goto cleanup;
	}
	updateStatusBar(i18n("Retrieving Version Information..."));
	stat = init->dbVersion(dbconnectinfo->pg_version);
	if (stat < 0)
	{
		errBox(stat);
		goto cleanup;
	}
	updateProgress(100);
	updateStatusBar(i18n("Server check complete."));
	return;
cleanup:
	delete init;
	init = 0;
	return;
}



/* *************************************************************************** */
/* *************************** DATABASE FUNCTIONS************************ */
/* ************************************************************************** */
/**
 *Checks the version of the open database against the program . If they differ, a warning message is issued.
 */
void
kmusicdb::checkVersion()
{
	QString s, t;
	stat=music->dbVersionLive(t);
	if (stat < 0)
		goto cleanup;
	stat=music->dbVersionCompat(s);
	if (stat < 0)
		goto cleanup;
	cerr << "kmusicdb::checkVersion: database: " << t << " program: " << s << endl;
	if (s.toFloat() > t.toFloat())
		errBox(DB_OLD_RELEASE);
	if (t.toFloat() > s.toFloat())
		errBox(DB_NEWER_RELEASE);
	return;
cleanup:
	errBox(stat);
	return;

}

/** This slot is for closing the current database. Dialog.*/
void
kmusicdb::closeDatabase()
{
	int z = KMessageBox::questionYesNo(this, i18n("Do you really want to close the database?"),
				i18n("Close Database"));
	if (z == KMessageBox::Yes)
		close_db();
	cerr << "kmusicdb::closeDatabase:close" << endl;

}

/**
 * Closes the db by deleting the object. Emits signalDbConnect(FALSE).
 */
void
kmusicdb::close_db()
{
	if (music)
	{
		delete music;
		music = 0;
		dbconnectinfo->pg_version = "";
		catresult->clear();
	}
	emit signalDbConnect(FALSE);
}

/** This is the slot for File->New. Opens the new db popup. Dialog.*/
void
kmusicdb::newDatabase()
{
	updateProgress(0);
	opener = new dbConnectDialog(this, i18n("Create New Database..."), i18n("C&reate"));
	if (!opener)
		exit(1);
	opener->widget()->updateInfo(dbconnectinfo);
	dialog_result = opener->exec();

	if (dialog_result)
	{
		dbInfo *tmp = new dbInfo();
		if (!tmp)
			exit(1);
		tmp->db_name=dbconnectinfo->db_name;
		tmp->db_location=dbconnectinfo->db_location;
		tmp->db_port=dbconnectinfo->db_port;
		tmp->db_user=dbconnectinfo->db_user;
		tmp->db_passwd=dbconnectinfo->db_passwd;
		dbconnectinfo->db_name = opener->widget()->getName();
		dbconnectinfo->db_location = opener->widget()->getLocation();
		dbconnectinfo->db_port = opener->widget()->getPort();
		dbconnectinfo->db_user = opener->widget()->getUser();
		if (dbconnectinfo->db_user == "")
			dbconnectinfo->db_user = getenv("USER");
		dbconnectinfo->db_passwd = opener->widget()->getPasswd();
		updateProgress(10);
		new_db();
		updateProgress(60);
		if (music)
		{
			updateArtistTitleView();
			updateProgress(70);
			checkVersion();
			updateProgress(90);
			emit signalDbConnect(TRUE);
		}
		if (def->dbdata != LAST_VALUE)
		{
			dbconnectinfo->db_name = tmp->db_name;
			dbconnectinfo->db_location=tmp->db_location;
			dbconnectinfo->db_port=tmp->db_port;
			dbconnectinfo->db_user=tmp->db_user;
			dbconnectinfo->db_passwd=tmp->db_passwd;
		}
		delete tmp;
	}
	delete opener;
	opener = 0;
	updateGUI(100, i18n("Ready..."));
}

/** Creates a new db in music. */
void
kmusicdb::new_db()
{
	cerr << "kmusicdb::new_db:" << endl;
	int test = 0;

	if (music)
		closeDatabase();
	if (music != 0)
	{
		errBox(DB_ALREADY_OPEN);
		return;
	}

	if (init)
		delete init;
	initDb(returnConnectInfo(dbconnectinfo->db_location, dbconnectinfo->db_port, "template1", dbconnectinfo->db_user, dbconnectinfo->db_passwd));
	if (!init)
		return;

	updateGUI(20, i18n("Finding database..."));
	cerr << "kmusicdb::new_db: Looking for database...";
	if (init->findDb(dbconnectinfo->db_name) == TRUE)
	{
		cerr << "Found" << endl;
		errBox(DB_EXISTS);
		delete init;
		init = 0;
		return;
	}

	cerr << "Not Found" << endl;
	updateGUI(30, i18n("Creating Database..."));
	cerr << "kmusicdb::new_db: Creating database...";
	test=init->createDb(dbconnectinfo->db_name);
	delete init;
	init=0;
	if (test)
	{
		cerr << "Successful." << endl;
		updateProgress(40);
		music =	new dbMusic(returnConnectInfo(dbconnectinfo->db_location, dbconnectinfo->db_port, dbconnectinfo->db_name,dbconnectinfo->db_user, dbconnectinfo->db_passwd));
		if (!music)
			exit(1);
		updateGUI(50, i18n("Formatting database..."));
		if ((test=music->formatDb()) < 0)
		{
			errBox(test);
			return;
		}
	}
	else
	{
		cerr << "Failed." << endl;
		errBox(DB_CREATE_FAIL);
		return;
	}
}

/** This slot is for opening an existing database. Dialog.*/
void
kmusicdb::openDatabase()
{
	updateProgress(0);
	opener = new dbConnectDialog(this, i18n("Open Database..."), i18n("&Open"));
	if (!opener)
		exit(1);
	opener->widget()->updateInfo(dbconnectinfo);
	dialog_result = opener->exec();

	if (dialog_result)
	{
		dbInfo *tmp = new dbInfo();
		if (!tmp)
			exit(1);
		tmp->db_name=dbconnectinfo->db_name;
		tmp->db_location=dbconnectinfo->db_location;
		tmp->db_port=dbconnectinfo->db_port;
		tmp->db_user=dbconnectinfo->db_user;
		tmp->db_passwd=dbconnectinfo->db_passwd;
		dbconnectinfo->db_name = opener->widget()->getName();
		dbconnectinfo->db_location = opener->widget()->getLocation();
		dbconnectinfo->db_port = opener->widget()->getPort();
		dbconnectinfo->db_user = opener->widget()->getUser();
		if (dbconnectinfo->db_user == "")
			dbconnectinfo->db_user = getenv("USER");
		dbconnectinfo->db_passwd = opener->widget()->getPasswd();
		updateProgress(10);
		int stat=0;
		stat = open_db(dbconnectinfo);
		cerr << "stat: " << stat << endl;
		if (stat < 0)
			return;
		updateProgress(60);
		if (music)
		{
			updateArtistTitleView();
			updateProgress(70);
			checkVersion();
			updateProgress(90);
			emit signalDbConnect(TRUE);
		}
		if (def->dbdata != LAST_VALUE)
		{
			dbconnectinfo->db_name = tmp->db_name;
			dbconnectinfo->db_location=tmp->db_location;
			dbconnectinfo->db_port=tmp->db_port;
			dbconnectinfo->db_user=tmp->db_user;
			dbconnectinfo->db_passwd=tmp->db_passwd;
		}
		delete tmp;
	}
	delete opener;
	opener = 0;
	updateGUI(100, i18n("Ready..."));
}

/**
 * Opens a database on music.
 * Returns negative if unsuccessful
 *
 * @param dbInfo Struct with database connect info.
 */
int
kmusicdb::open_db(const dbInfo *const dbinfo)
{
	cerr << "kmusicdb::open_db:" << endl;
	QString dirt;
	int test=0;
	if (music)
		closeDatabase();
	if (music != 0)
	{
		errBox(DB_ALREADY_OPEN);
		return -1;
	}
	updateProgress(20);
	if (init) {
		delete init;
		init=0;
	}
	cerr << "dbinfo: location: " << dbinfo->db_location << " port: " << dbinfo->db_port << " user: " << dbinfo->db_user << " pw: " << dbinfo->db_passwd << endl;
	initDb(returnConnectInfo(dbinfo->db_location, dbinfo->db_port, "template1",dbinfo->db_user, dbinfo->db_passwd));
	if (!init)
		return -1;
	cerr << "kmusicdb::open_db: Looking for database...";
	updateGUI(30, i18n("Finding database"));
	test=init->findDb(dbinfo->db_name);
	delete init;
	init=0;
	if (test)
	{
		cerr << "Found." << endl;
		updateProgress(40);
		music =	new dbMusic(returnConnectInfo(dbinfo->db_location, dbinfo->db_port, dbinfo->db_name,dbinfo->db_user, dbinfo->db_passwd));
		if (!music)
			exit(1);
	}
	else
	{
		cerr << "Failed." << endl;
		errBox(DB_NOT_FOUND);
		return -1;
	}
	updateProgress(50);

	return 1;
}

/** This slot is for backuping up the current database to a dump file. File Dialog.*/
void
kmusicdb::slotBackup()
{
	updateGUI(0, i18n("Backing up database..."));
	KURL burl = KFileDialog::getSaveURL(0,i18n("*.dump|PostgreSQL Dump File(*.dump)\n*|All Files"),
		 this, i18n("Backup Database"));
	if (burl.path() == NULL)
		return;
	windowInc();
	updateProgress(10);

	enableItems(FALSE);
	if (burl.isLocalFile())
	{
		QString localfilename = burl.path();
		stat = music->backup(localfilename);
		if (stat < 0)
			goto cleanup;
		updateProgress(60);
	}
	else
	{
		updateStatusBar(i18n("Backing up to temp file..."));
		KTempFile *tfile = new KTempFile("upload", "tmp");
		if (!tfile)
			exit(1);
		tfile->setAutoDelete(TRUE);
		if (tfile->status() == 0)
		{
			KMessageBox::error(this, i18n("Could not open file."),i18n("Save File..."));
			tfile->close();
			goto cleanup;
		}

		stat = music->backup(tfile->name());
		if (stat < 0)
		{
			tfile->close();
			goto cleanup;
		}
		updateGUI(30, i18n("Uploading..."));
		KIO::NetAccess::upload(tfile->name(), burl, this);
		updateGUI(50, i18n("Removing temp file..."));
		tfile->close();
		updateProgress(60);
	}
cleanup:
	if (stat < 0)
		errBox(stat);
	windowDec();
	updateArtistTitleView();
	updateProgress(70);
	updateTitleView();
	updateProgress(80);
	enableItems(TRUE);
	updateGUI(100, i18n("Ready..."));

}

/** This will perform a vacuum and an analyze on the database. */
void
kmusicdb::slotOpti()
{
	enableItems(FALSE);
	windowInc();
	updateGUI(0, i18n("Optimizing Database..."));
	kapp->processEvents();
	if ((stat = music->opti()) < 0)
		errBox(stat);
	updateProgress(60);
	updateArtistTitleView();
	updateProgress(80);
	updateTitleView();
	enableItems(TRUE);
	updateProgress(100);
	windowDec();
	updateStatusBar(i18n("Ready..."));
}

/** This slot is for restoring a database from a dump file. File Dialog.*/
void
kmusicdb::slotRestore()
{
	updateGUI(0, i18n("Restoring database..."));
	KURL rurl = KFileDialog::getOpenURL(0,i18n("*.dump|PostgreSQL Dump File(*.dump)\n*|All Files"),
		this, i18n("Restore Database"));
	if (rurl.path() == NULL)
		return;
	windowInc();
	updateProgress(10);

	enableItems(FALSE);
	if (rurl.isLocalFile())
	{
		QString localfilename = rurl.path();
		stat = music->restore(localfilename);
		if (stat < 0)
			goto cleanup;
		updateProgress(60);
	}
	else
	{
		QString tmpFile;
		updateStatusBar(i18n("Downloading to temp file..."));
		if (KIO::NetAccess::download(rurl, tmpFile, this))
		{
			updateGUI(30, i18n("Restoring..."));
			stat = music->restore(tmpFile);
			if (stat < 0)
				goto cleanup;
			updateGUI(50, i18n("Removing temp file..."));
			KIO::NetAccess::removeTempFile(tmpFile);
			updateProgress(60);
		}
	}
cleanup:
	if (stat < 0)
		errBox(stat);
	updateProgress(70);
	checkVersion();
	updateArtistTitleView();
	updateProgress(80);
	updateTitleView();
	updateProgress(90);
	enableItems(TRUE);
	windowDec();
	updateGUI(100, i18n("Ready..."));

}

