/***************************************************************************
                          errbox.cpp  -  description
                             -------------------
    begin                : Thu Jan 17 2002
    copyright            : (C) 2002 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmusicdb.h"

/** Displays a standard ErrorBox if a bad return value is received.
 * @param int Pass error code (see error.h) to display the appropriate error message box
 */
void
kmusicdb::errBox(int x)
{
	QString errors = "", maintext = "";

	// init is always connected first. Check if init is open, if not go for music. If neither, maintext stays "", see bottom of function.
	if (init && init->checkConnect())
		init->errorMessage(maintext);
	else
	{
		if (music)
			music->errorMessage(maintext);
	}
	switch(x)
	{
		case DB_EMPTY_QUERY:
			errors = i18n("The Query is empty.");
			break;
		case DB_FATAL_ERROR:
			errors = i18n("A Fatal Error was received by the server.");
			break;
		case DB_UNKNOWN_ERROR:
			errors = i18n("An Unknown Error was received by the server.");
			break;
		case DB_NULL_QUERY:
			errors = i18n("The Query is NULL.");
			break;
		case DB_CONNECTION_BAD:
			errors = i18n("The connection to the server is bad.");
			break;
		case FILE_OPEN_FAIL:
			errors = i18n("Could not open file.");
			break;
		case VACUUM_FAIL:
			errors = i18n("Could not optimize database.");
			break;
		case TABLE_FORMAT_FAIL:
			errors = i18n("Could not format table in database.");
			break;
		case PROCESS_ERROR:
			errors = i18n("Error running child process.");
			break;
		case NO_MEMORY:
			errors = i18n("Could not initialize object. There was a problem or you may have low memory.");
			break;
		case NO_TUPLES:
			errors = i18n("No tuples were returned when there should have been.");
			break;
		case ARTIST_DELETE_FAIL:
			errors = i18n("Could not delete artist.");
			break;
		case TITLE_DELETE_FAIL:
			errors = i18n("Could not delete title.");
			break;
		case ARTIST_ADD_FAIL:
			errors = i18n("Could not add artist.");
			break;
		case ARTIST_UPDATE_FAIL:
			errors = i18n("Could not update artist.");
			break;
		case ARTIST_GET_FAIL:
			errors = i18n("Could not retrieve artist information.");
			break;
		case LYRIC_GET_FAIL:
			errors = i18n("Could not retrive lyric information.");
			break;
		case LYRIC_UPDATE_FAIL:
			errors = i18n("Could not update lyric.");
			break;
		case TRACK_DELETE_FAIL:
			errors = i18n("Could not delete track.");
			break;
		case WORK_FAIL:
			errors = i18n("Could not perform the transaction.");
			break;
		case TITLE_ADD_FAIL:
			errors = i18n("Could not add title.");
			break;
		case TITLE_UPDATE_FAIL:
			errors = i18n("Could not update title.");
			break;
		case CATEGORY_ADD_FAIL:
			errors = i18n("Could not add to category.");
			break;
		case CATEGORY_DELETE_FAIL:
			errors = i18n("Could not delete from category.");
			break;
		case CATEGORY_UPDATE_FAIL:
			errors = i18n("Could not update category.");
			break;
		case TITLE_GET_FAIL:
			errors = i18n("Could not retrieve title information.");
			break;
		case TRACK_ADD_FAIL:
			errors = i18n("Could not add track.");
			break;
		case TRACK_GET_FAIL:
			errors = i18n("Could not retrieve track information.");
			break;
		case TRACK_UPDATE_FAIL:
			errors = i18n("Could not update track.");
			break;
		case VERSION_GET_FAIL:
			errors = i18n("Could not retrieve version number.");
			break;
		case TITLE_SUM_FAIL:
			errors = i18n("Could not sum titles.");
			break;
		case ARTIST_SUM_FAIL:
			errors = i18n("Could not sum artists.");
			break;
		case CATEGORY_SUM_FAIL:
			errors = i18n("Could not sum category.");
			break;
		case TRACK_SUM_FAIL:
			errors = i18n("Could not sum tracks.");
			break;
		case CATEGORY_GET_FAIL:
			errors = i18n("Could not get category.");
			break;
		case CATEGORY_UNKDEL_FAIL:
			errors = i18n("Cannot delete or modify UNKNOWN from the list!\n\nIt is a system entry.");
			break;
		case CDPARA_WRONG_VERSION:
			errors = i18n("Wrong version of cdparanoia. Please upgrade to the latest release.");
			break;
		case CDPARA_BAD_DATA:
			errors = i18n("cdparanoia returned bad data. Try again.");
			break;
			// now these are kmusicdb defined CDDB error codes.
		case SOCKET_WRITE_FAIL:
			errors = i18n("A failure occured writing to the socket.");
			break;
		case SOCKET_READ_FAIL:
			errors = i18n("A failure occured reading from the socket.");
			break;
		case CDDB_SERVER_BAD_RESPONSE:
			errors = i18n("The CDDB server returned a bad response.");
			break;
		case CDDB_NO_MATCH:
			errors = i18n("There was no match found in the CDDB database. Try again.");
			break;
		case CDDB_UNKNOWN_ERROR:
			errors = i18n("An unknown error occurred with the CDDB database.");
			break;
		case SOCKET_TIMEOUT:
			errors = i18n("Socket timeout exceeded.");
			break;
		case SMTP_SERVER_BAD_RESPONSE:
			errors = i18n("The SMTP server returned a bad response.");
			break;
		case SMTP_EHLO_FAIL:
			errors = i18n("SMTP HELO failed.\n\nThere is most likely a problem with your domain name.");
			break;
		case SMTP_MAIL_FAIL:
			errors = i18n("SMTP MAIL failed.\n\nThere is a problem with the senders address.");
			break;
		case SMTP_RCPT_FAIL:
			errors = i18n("SMTP RCPT failed.\n\nThere is a problem with the receivers address.");
			break;
		case SMTP_BODY_FAIL:
			errors = i18n("SMTP BODY failed.\n\nThere is a problem with the body of the message.");
			break;
		case SOCKET_HOST_NOT_FOUND:
			errors = i18n("Host not found, or look up failed.\n\nMake sure you have access to the internet.");
			break;
		case SOCKET_CREATE_FAILED:
			errors = i18n("Could not create socket.");
			break;
		case SOCKET_CONNECT_FAILED:
			errors = i18n("Could not connect to host.");
			break;
		case PROCESS_TIMEOUT:
			errors = i18n("Timed out waiting for cdparanoia.");
			break;
		case BOXSET_NO_ACTION:
			errors = i18n("Cannot perform this operation on a boxset.");
			break;
		case SEARCH_STRING_EMPTY:
			errors = i18n("You must enter a name or string.");
			break;
		case COMBO_VALUE_EMPTY:
			errors = i18n("You must select a value from the combo box.");
			break;
		case UPDATE_SAME_VALUE:
			errors = i18n("The values you are trying to update are equal.");
			break;
		case DB_OLD_RELEASE:
			maintext = i18n("This database is from an older release.\n\nYou may encounter errors during use.");
			errors = i18n("Database/kmusicdb version mismatch.");
			break;
		case DB_NEWER_RELEASE:
			maintext = i18n("This database is newer than the software you are running it on.\n\nYou may encounter errors. Try upgrading the software.");
			errors = i18n("Database/kmusicdb version mismatch.");
			break;
		case DB_CREATE_FAIL:
			errors = i18n("Could not create database: %1.").arg(dbconnectinfo->db_name);
			break;
		case DB_EXISTS:
			errors = i18n("Database %1 already exists.").arg(dbconnectinfo->db_name);
			break;
		case DB_NOT_FOUND:
			errors = i18n("The database %1 was not found.").arg(dbconnectinfo->db_name);
			break;
		case DB_ALREADY_OPEN:
			errors = i18n("Database is already open.");
			break;
		case KSCD_MISSING:
			maintext = i18n("Local CDDB files missing or not found.\nIf local CDDB is not installed uncheck 'Check for Local CDDB First'\nfrom Preferences->CDDB.\nNote that KSCD is the default local database.");
			errors= i18n("Local CDDB Lookup Fail.");
			break;
		case DB_QUALITY_BAD:
			maintext = i18n("Bad connection to database: %1").arg(dbconnectinfo->db_name);
			errors=i18n("Database transfer fail.");
			break;
		case CDPARA_NOT_FOUND:
			maintext = i18n("cdparanoia not found. Install it or set the path in preferences.");
			errors=i18n("cdparanoia Not Found");
			break;
		default:
			errors = i18n("Unknown error message caught! Please submit this bug.");
	};

	if (maintext.isEmpty())
		maintext = errors;
	KMessageBox::error(this, maintext, errors);

}

