/***************************************************************************
                          htmlreport.cpp  -  description
                             -------------------
    begin                : Fri Jul 6 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "htmlreport.h"
#include <khtmlview.h>

/**
 * Constructor
 * @param QWidget Parent
 * @param QString Dialog Caption
 * @param QString HTML String
 */
HTMLReport::HTMLReport(QWidget *parent, const QString &caption, const QString &html):
KDialogBase(Plain, caption, Ok | User1 | Close, Ok, parent, "HTMLReport",
			 false, false)
{
	cerr << "HTMLReport::initialized" << endl;
	// setup variables, page
	localHTML = html;
	QFrame *page = plainPage();
	htmlpart = new KHTMLPart(page);
	htmlwidget = htmlpart->view();

	// write html document to the screen
	htmlpart->begin();
	kapp->processEvents();
	htmlpart->write(localHTML);
	htmlpart->end();

	// set window size
	int width=750+htmlwidget->verticalScrollBar()->width();
	int height=600+75;
	QSize s(width,height);
//	htmlwidget->setFixedSize(s);
//	htmlwidget->resize(s);
//	htmlwidget->setFixedSize(width, height);
	setInitialSize(s);

	// start setting up sizes and buttons
	setButtonText(Ok, i18n("&Print"));
	setButtonText(User1, i18n("&Save"));

}

/**
 * Destructor
 */
HTMLReport::~HTMLReport()
{
}

void HTMLReport::resizeEvent ( QResizeEvent * )
{
  htmlwidget->setFixedSize(size().width()-2*htmlwidget->verticalScrollBar()->width(), size().height()-40);
}
void HTMLReport::manualResize() {
  htmlwidget->setFixedSize(size().width()-2*htmlwidget->verticalScrollBar()->width(), size().height()-40);
}

/**
 * Opens Printing dialog.
 */
void
HTMLReport::slotOk()
{
	hide();
	// Open print dialog and print html.
	htmlwidget->print();
}

/**
 * Reimplementation of close button.
 */
void HTMLReport::slotClose()
{
	delayedDestruct();
}

/**
 * Opens Save dialog.
 */
void
HTMLReport::slotUser1()
{
	// Open File dialog, get url and save file, network transparently
	KURL burl = KFileDialog::getSaveURL(0, i18n("*.html|HTML Document(*.html)\n*|All Files"),
					 this, i18n("Save Report"));
	if (burl.path() == NULL)
		return;

	if (burl.isLocalFile())
	{
		QFile f(burl.path());
		if (f.open(IO_WriteOnly))
		{
			QTextStream t(&f);
			t << localHTML;
			f.close();
		}
		else
		{
			KMessageBox::error(this, i18n("Could not save file."), i18n("Save File..."));
			return;
		}
	}
	else
	{
		KTempFile *tfile = new KTempFile("upload", "tmp");
		if (!tfile)
			exit(1);
		tfile->setAutoDelete(TRUE);
		if (tfile->status() == 0)
		{
			KMessageBox::error(this, i18n("Could not open file."), i18n("Save File..."));
			tfile->close();
			return;
		}
		*tfile->textStream() << localHTML;

		KIO::NetAccess::upload(tfile->name(), burl,this);
		tfile->close();
	}
}

