/***************************************************************************
                          include.h  -  description
                             -------------------
    begin                : Thu Mar 22 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/**
 * This file is the central location for all includes, global structs and variables,
 * and for processing the compiling environment.
 * Put includes in this file instead of individual headers to maintain consistency.
 * @short Central location for includes and global decs.
 * @author Will DeRousse <badhack@users.sourceforge.net>
 */

#ifndef INCLUDE_H
#define INCLUDE_H

/**
 * Error code file.
 */
#include "errorcode.h"

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

/**
 * Default values
 */
#define AUTO_CONNECT_DB 0
#define DEFAULT_COLUMN 0
#define DEFAULT_SORT_ORDER 0
#define TOOLBAR_TITLE 0
#define TOOLBAR_NAME 0
#define TOOLBAR_DEFAULT 1
#define STATUSBAR_DEFAULT 1

#define TITLE_LABEL1 0
#define TITLE_CAT_COMBO 1
#define TITLE_LIST_COMBO 2
#define TITLE_BUTTON 3

#define NAME_LABEL1 0
#define NAME_LINED 1
#define NAME_LABEL2 2
#define NAME_COMBO 3
#define NAME_BUTTON 4

#define DEFAULT_FOLLOW 1
#define DEFAULT_COLLECT 1
#define DEFAULT_AUTO_FILL 0
#define DEFAULT_YEAR 0

#define DEFAULT_COLUMN_SORT_ORDER 1
#define DEFAULT_CAPITALIZE 0
#define DEFAULT_PROCESS_TEXT 0

#define DIALOG_MARGIN 15

#define PICWIDTH 250
#define PICHEIGHT 230

#define DEFAULT_CDDBP_SERVER "freedb.freedb.org"
#define DEFAULT_CDDBP_PORT "888"
#define DEFAULT_HTTP_SERVER "freedb.freedb.org"
#define DEFAULT_HTTP_PORT "80"
#define DEFAULT_HTTP_CGI "/~cddb/cddb.cgi"
#define DEFAULT_PROXY_SERVER "192.168.0.1"
#define DEFAULT_PROXY_PORT "3128"
#define DEFAULT_SMTP_PORT "25"
#define DEFAULT_SMTP_SERVER "smtp.yourmailserver.net"
#define DEFAULT_CDDB_EMAIL "freedb-submit@freedb.org"
#define DEFAULT_EMAIL_PROFILE "Default"
#define DEFAULT_CDPARANOIA_LOCATION "/usr/bin"
#define DEFAULT_READ_DEVICE "/dev/cdrom"
#define DEFAULT_SOCKET_TIMEOUT 30
#define DEFAULT_CDDB_TITLE_OPEN 1
#define DEFAULT_CDDB_CHECK_LOCAL 1

#define CDDB_PROTO_CDDBP 0
#define CDDB_PROTO_HTTP 1
#define CDDB_PROTO_PROXY 2
#define CDDB_PROTO_EMAIL 3
#define CDDB_PROTO_SEARCH 4
#define CDDB_PROTO_SEARCHPROXY 5

#define CDDB_STATE_CDDBHELLO 0
#define CDDB_STATE_CDDBQUERY 1
#define CDDB_STATE_CDDBQUIT 2
#define CDDB_STATE_CDDBREAD 3
#define CDDB_STATE_CDDBLOCAL 4
#define CDDB_STATE_SMTPHELLO 5
#define CDDB_STATE_SMTPSENDHEADER 6
#define CDDB_STATE_SMTPSENDBODY 7
#define CDDB_STATE_SMTPQUIT 8
#define CDDB_STATE_SHUTDOWN 9
#define CDDB_STATE_SEARCHQUERY 10
#define CDDB_STATE_SEARCHREAD 11
#define CDDB_STATE_END -1
#define CDDB_STATE_CONNECTING 99

#define DEFAULT_DB "music"
#define DEFAULT_LOCATION "localhost"
#define DEFAULT_SERVER_PORT "5432"
#define DEFAULT_SERVER_USER ""
#define DEFAULT_SERVER_PASSWD ""

#define STATIC_DEFAULT 0
#define BLANK 1
#define LAST_VALUE 2

#define DEFAULT_FAMILY "Bitstream Charter"
#define DEFAULT_WEIGHT 0
#define DEFAULT_STYLE 0
#define DEFAULT_SIZE 10

#define DEFAULT_MISC_COLOR "#97ACEC"
#define DEFAULT_ARTIST_COLOR "#FF8548"
#define DEFAULT_TITLE_COLOR "#90C6FF"
#define DEFAULT_TRACK_COLOR "#AF9CE3"
#define DEFAULT_TOTALS_COLOR "#7888BB"
#define DEFAULT_BG_COLOR "#FFFFFF"
#define DEFAULT_TEXT_COLOR "#000000"

#define COLOR_ARTIST 0
#define COLOR_BACKGROUND 1
#define COLOR_MISC 2
#define COLOR_TITLE 3
#define COLOR_TOTAL 4
#define COLOR_TRACK 5
#define COLOR_TEXT 6

/*
 *SUSE 7.2 fix (maybe others who knows)
 */
#define HAVE_CXX_STRING_HEADER 1

/**
 * C/C++ library headers
 */
#include <cstdlib>
#include <errno.h>
#include <iostream>
#include <netdb.h>
#include <signal.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <sys/time.h>

/**
 * QT library headers #include <qdragobject.h>
 */
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcolor.h>
#include <qdatetime.h>
#include <qdragobject.h>
#include <qfont.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qmap.h>
#include <qobject.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qprogressbar.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qstring.h>
#include <qtextedit.h>
#include <qtimer.h>
#include <qtooltip.h>
#include <qvaluelist.h>
#include <qvbox.h>
#include <qwidget.h>

/**
 * KDE library headers
 */
#include <kaboutdata.h>
#include <kaction.h>
#include <kaccel.h>
#include <kapplication.h>
#include <kcmdlineargs.h>
#include <kcolordialog.h>
#include <kconfig.h>
#include <kcombobox.h>
#include <kdialogbase.h>
#include <kemailsettings.h>
#include <kfiledialog.h>
#include <kfontdialog.h>
#include <khtml_part.h>
#include <khtmlview.h>
#include <kiconloader.h>
#include <kimageio.h>
#include <kinstance.h>
#include <klineedit.h>
#include <klocale.h>
#include <kparts/mainwindow.h>
#include <kmessagebox.h>
#include <kio/netaccess.h>
#include <kparts/plugin.h>
#include <kprocess.h>
#include <kstatusbar.h>
#include <kstdaction.h>
#include <kstatusbar.h>
#include <kstddirs.h>
#include <ktempfile.h>
#include <kurl.h>

/**
 * libdbmusic headers
 */
#include <dbmusicglobals.h>
#include <dbmusicwidget.h>

/**
 * Global struct prototypes
 */
using namespace std;

/**
 * @short Centralized location containing all default variables.
 * @param artist_year Fill in current year in conception or not
 * @param artist_autofill Demise Mirrors Conception
 * @param artist_follow Whether or not the default is to check Follow in artist box
 * @param title_year Fill in current year in yearstart or not
 * @param title_autofill Yearend Mirrors Yearstart
 * @param title_collect Whether or not the default is to check Collect in title box
 * @param cddb_popup Whether or not the title dialog opens after a CDDB title has been added
 * @param default_sort_order Ascending Or Descending
 * @param category_sort_order Ascending Or Descending
 * @param auto_capitalize Auto capitalize data or not
 * @param auto_format Format the data or not
 * @param artistdata New artist style (1 values, 2 blank, 3 last)
 * @param titledata New title style (1 values, 2 blank, 3 last)
 * @param dbdata New database style (1 values, 2 blank, 3 last)
 * @param auto_connect Auto connect or not to default auto connect db on startup.
 * @param default_column_sort Which column to sort
 */
struct Defaults {
	bool artist_year, artist_autofill, artist_follow, title_year, title_autofill, title_collect, cddb_popup;
	bool default_sort_order, category_sort_order, auto_capitalize, auto_format;
	int artistdata, titledata, dbdata, auto_connect, default_column_sort;
};

/**
 * @short CDDBtitleInfo is used to hold all necessary information needed to add a title to the database via CDDB.
 * @param disc_id The calculated CDDB disc id
 * @param cddb_genre The CDDB genre name
 * @param totalLength The total length of the album in seconds
 * @param titleInfo The titleInfo struct used to store title information (like the name)
 * @param trackSectorLength The QValueList<int> which holds the length of each track in sectors
 * @param trackSectorBegin The QValueList<int> which holds the beginning sector of each track
 * @param trackData The QValueList<trackInfo> which holds the track information
 * @param device CD-Rom device (/dev/cdrom, /dev/dvd, etc.)
 */
struct CDDBtitleInfo {
	QString disc_id, cddb_genre, totalLength;
	titleInfo title;
	QValueList<int> trackSectorLength;
	QValueList<int> trackSectorBegin;
	QValueList<trackInfo> trackData;
	QString device;
};

/**
 * @short CDDBserverInfo is used to hold all variables regarding connections to the freedb database.
 * This includes CDDB servers and ports (HTTP and CDDBP), SMTP servers and ports (for
 * submitions), email profiles (using the KDE email profile standard), and proxy support
 * (authentication not supported).
 * @param get_protocol Protocol used for retrieving CDDB data.
 * @param put_protocol Protocol used for submitting CDDB data.
 * @param cddbp_server Holds the CDDB server host
 * @param cddbp_port Holds the CDDB port
 * @param http_server Holds the CDDB server host
 * @param http_port Holds the CDDB port
 * @param http_cgi Path to CGI on server
 * @param proxy_server Holds the CDDB server host
 * @param proxy_port Holds the CDDB port
 * @param smtp_server The SMTP server host name
 * @param smtp_port The SMTP Port
 * @param kde_email_profile The default profile used to retrieve email information from
 * @param cddb_email Holds the CDDB submit address
 * @param check_cddb_local Local KSCD CDDB before network access
 * @param local_cddb_path Path to local CDDB files
 * @param cdparanoia_location An alt location for cdparanoia
 * @param read_device The read device (i.e. /dev/cdrom)
 * @param host Host Information
 */
struct CDDBserverInfo {
	int get_protocol, put_protocol;
	QString cddbp_server, cddbp_port, http_server, http_port, http_cgi, proxy_server, proxy_port, smtp_server, smtp_port;
	QString kde_email_profile, cddb_email, cdparanoia_location, read_device, local_cddb_path;
	bool check_cddb_local;
	char host[255];
};

///**
// * @short completeArtist can be used to hold all information about an artist, including titles and track information.
// *
// * @param artist artistInfo struct containing artist information
// * @param boxset QStringList comtaining all the boxsets that the titles belong to (not including none)
// * @param title QValueList holding title (and track) information
// */
//struct completeArtist {
//  artistInfo artist;
//  QStringList boxset;
//  QValueList<CDDBtitleInfo> title;
//};


#endif

