/***************************************************************************
                          kmusicdb.cpp  -  description
                             -------------------
    begin                : Thu Mar 22 14:01:50 /etc/localtime 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmusicdb.h"

/* *************************************************************************** */
/* *********************** CONSTRUCTOR AND DESTRUCTOR ************************ */
/* *************************************************************************** */

/**
 * Constructor
 * @param name Name Of Program
 * @param KCmdLineArgs Command line arguments passed to KmusicdB
 */
kmusicdb::kmusicdb(const char *name, KCmdLineArgs *args):
KParts::MainWindow(0, name)
{
	// connect signals, disable items
	connect(this, SIGNAL(signalDbConnect(bool)), this, SLOT(enableItems(bool)));
	connect(this, SIGNAL(signalDbConnect(bool)), this, SLOT(connectGUI(bool)));

	setMinimumWidth(450);
	// set xml file
	setXMLFile("kmusicdbui.rc");
	// initialize the variables
	initDefault();
	// setup the central widget
	initCenterWidget();
	// build menus
	initMenu();
	// build gui interface
	KInstance * instancekm = new KInstance("kmusicdb");
	KParts::Plugin::loadPlugins(this, instancekm);
	createGUI(0L);
	// build toolbars
	initToolBars();
	// setup status bar
	initStatusBar();
	// show()
	show();
	emit signalDbConnect(FALSE);
	// read config file
	updateGUI(70, i18n("Reading Settings..."));
	readSettings();
	// parse command line args
	updateGUI(80, i18n("Processing Command Line Arguments..."));
	processArgs(args);
	// autoconnect if enabled
	updateGUI(90, i18n("Auto Connecting..."));
	initAutoConnect();
	// all done...
	updateGUI(100, i18n("Ready..."));
}

/**
 * Destructor
 */
kmusicdb::~kmusicdb()
{
	if (music)
		close_db();
}



/* *************************************************************************** */
/* ******************************* CDDB FUNCTIONS **************************** */
/* *************************************************************************** */
/**
 * Add new CD from CDDB.
 */
void kmusicdb::slotCDDBAdd()
{
	// see if artist exists/variable
	if (mid->getArtistText().isEmpty())
		return;
	CDDBtitleInfo t;

	//take care of GUI
	windowInc();
	enableCDDB(FALSE);
	updateGUI(0, i18n("Retrieving CDDB..."));

	// initiate cdparanoia class
	t.device=si->read_device;
	cdp = new cdparanoia(si->cdparanoia_location, t);
	if (!cdp)
		exit(1);
	connect(cdp, SIGNAL(updateGUI(int, QString)), this, SLOT(updateGUI(int, QString)));
	connect(cdp, SIGNAL(cdpDone(CDDBtitleInfo)), this, SLOT(slotCDDBAdd2(CDDBtitleInfo)));
	connect(cdp, SIGNAL(error(int)), this, SLOT(cdpError(int)));
	connect(cdp, SIGNAL(cdpFinish()), this, SLOT(cdpCleanup()));

	cdp->get();
}
/**
 * Part 2 of the CDDB add process.
 * @param CDDBtitleInfo Struct Containing CDDB Info
 */
void
kmusicdb::slotCDDBAdd2(CDDBtitleInfo t)
{
	windowInc();
	enableCDDB(FALSE);
	// now we are done with cdparanoia. time to query CDDB
	cdnet = new cddb(this, t, si);
	if (!cdnet)
		exit(1);
	// connect signals to local slots
	connect(cdnet, SIGNAL(updateGUI(int, QString)), this, SLOT(updateGUI(int, QString)));
	connect(cdnet, SIGNAL(error(int)), this, SLOT(errBox(int)));
	connect(cdnet, SIGNAL(cddbFinished()), this, SLOT(cddbCleanup()));
	// this slot that actually does the work after cddb returns the query data.
	connect(cdnet, SIGNAL(titleReceived(CDDBtitleInfo)), this, SLOT(slotCDDBAddFinish(CDDBtitleInfo)));

	// get() the data now.
	cdnet->get();

}
/**
 * Finish Adding CD.
 * @param CDDBtitleInfo Struct Containing CDDB Info
 */
void
kmusicdb::slotCDDBAddFinish(CDDBtitleInfo t)
{
	QValueList < trackInfo >::Iterator it2=0;

	windowInc();
	enableCDDB(FALSE);
	// setup a generic titleInfo struct, and fill with default info
	// which is either blank or with data. any "blank" combo boxes will be
	// filled with useful data from CDDB(basically the title name)
	updateGUI(60, i18n("Formatting Title..."));
	titleInfo *tempTitle = new titleInfo();
	if (!tempTitle)
		exit(1);
	tempTitle->cdid=title->cdid;
	cerr << "formatname: " << title->cat[KFORMAT].name << endl;
	tempTitle->cat[KFORMAT].name=title->cat[KFORMAT].name;
	tempTitle->cat[KGENRE].name=title->cat[KGENRE].name;
	tempTitle->cat[KBOXSET].name=title->cat[KBOXSET].name;
	tempTitle->cat[KLABEL].name=title->cat[KLABEL].name;
	tempTitle->cat[KMETHOD].name=title->cat[KMETHOD].name;
	tempTitle->cat[KTYPE].name=title->cat[KTYPE].name;
	tempTitle->length=title->length;
	tempTitle->nod=title->nod;
	tempTitle->notes=title->notes;
	tempTitle->notracks=title->notracks;
	tempTitle->rating=title->rating;
	tempTitle->quality=title->quality;
	tempTitle->name = t.title.name;
	tempTitle->code = t.title.code;
	tempTitle->loan = t.title.loan;
	tempTitle->collected=TRUE;
	if (def->title_year)
		tempTitle->yearstart.setNum( QDate::currentDate().year() );
	if (def->title_autofill)
		tempTitle->yearend=tempTitle->yearstart;
	cerr << "kmusicdb::CDDB retrieval complete. Adding titles and tracks." << endl;
	updateGUI(65, i18n("Adding Title to Database..."));
	// check for text formatting
	if (def->auto_format)
		tempTitle->name = formatText(tempTitle->name);
	else if (def->auto_capitalize)
		tempTitle->name = capitalizeWords(tempTitle->name);
	tempTitle->name.truncate(40);
	// add the title to the database.. use default settings or unknown/one if default is blank.
	cerr << "kmusicdb::slotCDDBAddFinish:name: " << tempTitle->name << endl;
	stat = music->addTitle(tempTitle, aid);
	if (stat < 0)
		goto cleanup;

	updateGUI(80, i18n("Finding CDID..."));
	cerr << "kmusicdb::CDID: " << tempTitle->cdid << endl;
	// add tracks to db
	updateGUI(85, i18n("Adding Tracks to Database..."));
	kapp->processEvents();

	for (it2 = t.trackData.begin(); it2 != t.trackData.end(); ++it2)
	{
		// check for text formatting
		if (def->auto_format)
			(*it2).name = formatText((*it2).name);
		else if (def->auto_capitalize)
			(*it2).name = capitalizeWords((*it2).name);
		(*it2).side="1";

		stat = music->addTrack(&(*it2), tempTitle->cdid);
		if (stat < 0)
			goto cleanup;
	}

	// display if enabled
	updateGUI(90, i18n("Ready..."));
	if (def->cddb_popup)
		openEditTitle(tempTitle->cdid);
cleanup:
	if (stat < 0)
		errBox(stat);
	delete tempTitle;
	kapp->processEvents();
	windowDec();
	updateProgress(90);
	updateTitleView();
	updateGUI(100, i18n("Ready..."));
	return;

}



/**
 * Append to CD CDDB slot.
 */
void kmusicdb::slotCDDBAppend()
{
	// see if artist exists/variable
	if (mid->getArtistText().isEmpty())
		return;
	CDDBtitleInfo t;

	//take care of GUI
	windowInc();
	enableCDDB(FALSE);
	updateGUI(0, i18n("Retrieving CDDB..."));

	// initiate cdparanoia class
	t.device=si->read_device;
	cdp = new cdparanoia(si->cdparanoia_location, t);
	if (!cdp)
		exit(1);
	connect(cdp, SIGNAL(updateGUI(int, QString)), this, SLOT(updateGUI(int, QString)));
	connect(cdp, SIGNAL(cdpDone(CDDBtitleInfo)), this, SLOT(slotCDDBAppend2(CDDBtitleInfo)));
	connect(cdp, SIGNAL(error(int)), this, SLOT(cdpError(int)));
	connect(cdp, SIGNAL(cdpFinish()), this, SLOT(cddbCleanup()));

	cdp->get();
}
/**
 * Part 2 of CDDB append process.
 * @param CDDBtitleInfo Struct Containing CDDB Info
 */
void kmusicdb::slotCDDBAppend2(CDDBtitleInfo t)
{
	windowInc();
	enableCDDB(FALSE);
	// now we are done with cdparanoia. time to query CDDB
	cdnet = new cddb(this, t, si);
	if (!cdnet)
		exit(1);
	// connect signals to local slots
	connect(cdnet, SIGNAL(updateGUI(int, QString)), this, SLOT(updateGUI(int, QString)));
	connect(cdnet, SIGNAL(error(int)), this, SLOT(errBox(int)));
	connect(cdnet, SIGNAL(cddbFinished()), this, SLOT(cddbCleanup()));
	// this slot that actually does the work after cddb returns the query data.
	connect(cdnet, SIGNAL(titleReceived(CDDBtitleInfo)), this, SLOT(slotCDDBAppendFinish(CDDBtitleInfo)));

	// get() the data now.
	cdnet->get();

}
/**
 * Finish Appending Data.
 * @param CDDBtitleInfo Struct Containing CDDB Info
 */
void kmusicdb::slotCDDBAppendFinish(CDDBtitleInfo t)
{
	QValueList < trackInfo >::Iterator it2;
	enabledTrackColumns *tr=new enabledTrackColumns();
	if (!tr)
		exit(1);
	QListView *v=new QListView();
	if (!v)
		exit(1);
	QListViewItem *ti;
	QString maxside;

	windowInc();
	enableCDDB(FALSE);
	updateGUI(60, i18n("Finding CDID..."));
	QString scdid=mid->listview()->selectedItem()->text(mid->listview()->columns());
	cerr << "kmusicdb::CDID: " << scdid << endl;
	// add tracks to db
	kapp->processEvents();

	tr->num=FALSE;
	tr->name=FALSE;
	tr->length=FALSE;
	tr->lyric=FALSE;
	tr->side=TRUE;
	tr->trackid=FALSE;
	updateGUI(50, i18n("Retrieving Track List..."));
	stat=music->populateTrackQListView(v, scdid, tr);
	if (stat < 0)
		goto cleanup;
	ti=v->firstChild();
	maxside=ti->text(0);
	if (ti)
	{
		do
		{
			if (ti->text(0) > maxside)
				maxside=ti->text(0);
			ti=ti->nextSibling();
		} while (ti);
	}
	maxside=QString::number(maxside.toInt()+1);

	for (it2 = t.trackData.begin(); it2 != t.trackData.end(); ++it2)
	{
		// check for text formatting
		if (def->auto_format)
			(*it2).name = formatText((*it2).name);
		else if (def->auto_capitalize)
			(*it2).name = capitalizeWords((*it2).name);
		(*it2).side=maxside;

		stat = music->addTrack(&(*it2), scdid);
		if (stat < 0)
			goto cleanup;
	}
cleanup:
	if (stat < 0)
		errBox(stat);
	windowDec();
	// display if enabled
	updateProgress(90);
	updateTitleView();
	updateGUI(90, i18n("Ready..."));
	if (def->cddb_popup)
		openEditTitle(scdid);

	// reenable toolbar button
	updateGUI(100, i18n("Ready..."));
}



/**
 * Search for title in CDDB.
 */
void kmusicdb::slotCDDBSearch()
{
	// see if artist exists/variable
	if (mid->getArtistText().isEmpty())
		return;

	//take care of GUI
	windowInc();
	enableCDDB(FALSE);
	updateGUI(0, i18n("Searching CDDB..."));

	CDDBSearchDialog *sd=new CDDBSearchDialog(this);
	if (!sd)
		exit(1);
	connect(sd,SIGNAL(search(QString,bool,bool)),this,SLOT(slotCDDBSearch2(QString,bool,bool)));
	connect(sd,SIGNAL(cancelled()),this,SLOT(cdpCleanup()));
	sd->show();

}
/**
 * Part 2 of CDDB search process.
 * @param QString Search Text
 * @param bool Artist Enabled Or Disabled
 * @param bool Title Enabled Or Disabled
 */
void kmusicdb::slotCDDBSearch2(QString ssearch,bool aartist,bool ttitle)
{
	// now we are done with cdparanoia. time to query CDDB
	CDDBtitleInfo temp;
	cdnet = new cddb(this, temp, si);
	if (!cdnet)
		exit(1);
	// connect signals to local slots
	connect(cdnet, SIGNAL(updateGUI(int, QString)), this, SLOT(updateGUI(int, QString)));
	connect(cdnet, SIGNAL(error(int)), this, SLOT(errBox(int)));
	connect(cdnet, SIGNAL(cddbFinished()), this, SLOT(cddbCleanup()));
	// this slot that actually does the work after cddb returns the query data.
	connect(cdnet, SIGNAL(searchReceived(QStringList, QStringList, QStringList, QStringList)), this, SLOT(slotCDDBSearch3(QStringList, QStringList, QStringList, QStringList)));

	// get() the data now.
	cdnet->search(ssearch,aartist,ttitle);
}
/**
 * Part 3 of CDDB search process.
 * @param QStringList Artist
 * @param QStringList Title
 * @param QStringList CDDB Category
 * @param QStringList CDDB Disc ID
 */
void kmusicdb::slotCDDBSearch3(QStringList lartist, QStringList ltitle, QStringList lcat, QStringList lcdid)
{
	updateGUI(100,i18n("Ready..."));
	windowInc();
	enableCDDB(FALSE);
	CDDBSearchResultsDialog *d=new CDDBSearchResultsDialog(this);
	if (!d)
		exit(1);
	d->populateQListView(lartist,ltitle,lcat,lcdid);
	connect(d,SIGNAL(addTitle(QString, QString)),this,SLOT(slotCDDBSearch4(QString, QString)));
	connect(d,SIGNAL(cancelled()),this,SLOT(cddbCleanup()));
	d->show();
}
/**
 * Finish CDDB search process.
 * @param QString CDDB Category
 * @param QString CDDB Disc ID
 */
void kmusicdb::slotCDDBSearch4(QString cat, QString discid)
{
	CDDBtitleInfo t;

	// time to query CDDB
	cdnet = new cddb(this, t, si);
	if (!cdnet)
		exit(1);
	// connect signals to local slots
	connect(cdnet, SIGNAL(updateGUI(int, QString)), this, SLOT(updateGUI(int, QString)));
	connect(cdnet, SIGNAL(error(int)), this, SLOT(errBox(int)));
	connect(cdnet, SIGNAL(cddbFinished()), this, SLOT(cddbCleanup()));
	// this slot that actually does the work after cddb returns the query data.
	connect(cdnet, SIGNAL(titleReceived(CDDBtitleInfo)), this, SLOT(slotCDDBAddFinish(CDDBtitleInfo)));

	// get() the data now.
	cdnet->simpleGet(cat,discid);
}



/**
 * Submits a CD to CDDB.
 */
void
kmusicdb::slotCDDBSubmit()
{
	// no signal, but lets make sure it is selected.
	QListViewItem *item = mid->listview()->selectedItem();
	if (item == 0)
		return;
	if (item->childCount())
	{
		errBox(BOXSET_NO_ACTION);
		return;
	}

	CDDBtitleInfo t;

	// take care of GUI
	windowInc();
	enableCDDB(FALSE);
	updateGUI(0, i18n("Submitting CD to CDDB..."));

	// initiate cdparanoia class
	t.device=si->read_device;
	cdp = new cdparanoia(si->cdparanoia_location,t);
	if (!cdp)
		exit(1);
	connect(cdp, SIGNAL(updateGUI(int, QString)), this, SLOT(updateGUI(int, QString)));
	connect(cdp, SIGNAL(cdpDone(CDDBtitleInfo)), this, SLOT(slotCDDBSubmit2(CDDBtitleInfo)));
	connect(cdp, SIGNAL(error(int)), this, SLOT(cdpError(int)));
	connect(cdp, SIGNAL(cdpFinish()), this, SLOT(cddbCleanup()));

	cdp->get();
}
/**
 * Part 2 of the CDDB submit process.
 * @param CDDBtitleInfo Struct Containing CDDB Info
 */
void
kmusicdb::slotCDDBSubmit2(CDDBtitleInfo t)
{
	QValueList < trackInfo >::Iterator it2;
	QListViewItem *current=0;
	windowInc();
	enableCDDB(FALSE);
	QListView *temp = new QListView();
	if (!temp)
		exit(1);
	// no signal, but lets make sure it is selected.
	QListViewItem *item = mid->listview()->selectedItem();
	if (item == 0)
		goto cleanup;
	if (item->childCount())
	{
		errBox(BOXSET_NO_ACTION);
		goto cleanup;
	}

	// now we must fill cddbtitle up with the title name and the track names
	t.title.name = item->text(0);

	t.title.cdid = item->text(mid->listview()->columns());

	if (!temp)
		exit(1);
	stat=music->populateTrackQListView(temp, t.title.cdid, tr_op);
	if (stat < 0)
	{
		errBox(stat);
		goto cleanup;
	}
	kapp->processEvents();
	current = temp->firstChild();
	if (!current)
		goto cleanup;
	for (it2 = t.trackData.begin(); it2 != t.trackData.end(); it2++)
	{
		(*it2).name = current->text(1);
		if (current->nextSibling())
			current = current->nextSibling();
	}

	current = 0;

	// now we are done with cdparanoia. time to query CDDB
	cdnet = new cddb(this, t, si);
	if (!cdnet)
		exit(1);
	// connect signals to local slots
	connect(cdnet, SIGNAL(updateGUI(int, QString)), this, SLOT(updateGUI(int, QString)));
	connect(cdnet, SIGNAL(error(int)), this, SLOT(errBox(int)));
	connect(cdnet, SIGNAL(cddbFinished()), this, SLOT(cddbCleanup()));
	connect(cdnet, SIGNAL(submitSuccess(CDDBtitleInfo)), this, SLOT(slotCDDBSubmitFinish(CDDBtitleInfo)));

	// put() the data and we're done
	cdnet->put(mid->getArtistText());

cleanup:
	delete temp;
	enableCDDB(TRUE);
}
/**
 * Called when the submit to CDDB is finished.
 * @param CDDBtitleInfo Struct Containing CDDB Info
 */
void
kmusicdb::slotCDDBSubmitFinish(CDDBtitleInfo)
{
	updateGUI(100, i18n("Submit Successful."));
}




/**
 * Update(replace tracks) CD from CDDB.
 */
void kmusicdb::slotCDDBUpdate()
{
	// see if artist exists/variable
	if (mid->getArtistText().isEmpty())
		return;
	CDDBtitleInfo t;

	//take care of GUI
	windowInc();
	enableCDDB(FALSE);
	updateGUI(0, i18n("Retrieving CDDB..."));

	// initiate cdparanoia class
	t.device=si->read_device;
	cdp = new cdparanoia(si->cdparanoia_location, t);
	if (!cdp)
		exit(1);
	connect(cdp, SIGNAL(updateGUI(int, QString)), this, SLOT(updateGUI(int, QString)));

	connect(cdp, SIGNAL(cdpDone(CDDBtitleInfo)), this, SLOT(slotCDDBUpdate2(CDDBtitleInfo)));
	connect(cdp, SIGNAL(error(int)), this, SLOT(cdpError(int)));
	connect(cdp, SIGNAL(cdpFinish()), this, SLOT(cddbCleanup()));

	cdp->get();
}
/**
 * Part 2 of CDDB update process.
 * @param CDDBtitleInfo Struct Containing CDDB Info
 */
void kmusicdb::slotCDDBUpdate2(CDDBtitleInfo t)
{
	windowInc();
	enableCDDB(FALSE);
	// now we are done with cdparanoia. time to query CDDB
	cdnet = new cddb(this, t, si);
	if (!cdnet)
		exit(1);
	// connect signals to local slots
	connect(cdnet, SIGNAL(updateGUI(int, QString)), this, SLOT(updateGUI(int, QString)));
	connect(cdnet, SIGNAL(error(int)), this, SLOT(errBox(int)));
	connect(cdnet, SIGNAL(cddbFinished()), this, SLOT(cddbCleanup()));
	// this slot that actually does the work after cddb returns the query data.
	connect(cdnet, SIGNAL(titleReceived(CDDBtitleInfo)), this, SLOT(slotCDDBUpdateFinish(CDDBtitleInfo)));

	// get() the data now.
	cdnet->get();

}
/**
 * Finish CDDB update process.
 * @param CDDBtitleInfo Struct Containing CDDB Info
 */
void kmusicdb::slotCDDBUpdateFinish(CDDBtitleInfo t)
{
	QValueList < trackInfo >::Iterator it2;
	QListViewItem *ti;
	windowInc();
	enableCDDB(FALSE);
	// we need to drop all the tracks on the current title.
	// then add all the titles from CDDB.

	QString scdid=mid->listview()->selectedItem()->text(mid->listview()->columns());
	enabledTrackColumns *tr=new enabledTrackColumns();
	if (!tr)
		exit(1);
	tr->num=FALSE;
	tr->name=FALSE;
	tr->length=FALSE;
	tr->lyric=FALSE;
	tr->side=FALSE;
	tr->trackid=TRUE;
	QListView *v=new QListView();
	if (!v)
		exit(1);
	updateGUI(50, i18n("Retrieving Track List..."));
	stat=music->populateTrackQListView(v, scdid, tr);
	if (stat < 0)
		goto cleanup;

	ti=v->firstChild();
	if (ti)
	{
		do
		{
			stat=music->deleteTrack(ti->text(0),scdid);
			if (stat < 0)
				goto cleanup;
			ti=ti->nextSibling();
		} while (ti);
	}

	kapp->processEvents();

	for (it2 = t.trackData.begin(); it2 != t.trackData.end(); ++it2)
	{
		// check for text formatting
		if (def->auto_format)
			(*it2).name = formatText((*it2).name);
		else if (def->auto_capitalize)
			(*it2).name = capitalizeWords((*it2).name);
		(*it2).side="1";

		stat = music->addTrack(&(*it2), scdid);
		if (stat < 0)
			goto cleanup;
	}

	// display if enabled
	updateProgress(90);
	updateTitleView();
	updateGUI(90, i18n("Ready..."));
	if (def->cddb_popup)
		openEditTitle(scdid);

	// reenable toolbar button
	updateGUI(100, i18n("Ready..."));

cleanup:
	delete tr;
	delete v;
	windowDec();
	if (stat < 0)
	{
		errBox(stat);
		updateTitleView();
	}
}


/**
 * Slot wrapper for cddb cleanup.
 */
void kmusicdb::cddbCleanup()
{
	if (cdnet)
	{
		delete cdnet;
		cdnet=0;
	}

	// reenable toolbar button
	enableCDDB(TRUE);
	windowDec();
}
/**
 * Slot wrapper for cdp cleanup.
 */
void kmusicdb::cdpCleanup()
{
	if (cdp)
	{
		delete cdp;
		cdnet=0;
	}

	// reenable toolbar button
	enableCDDB(TRUE);
	windowDec();
}
/**
 * Slot wrapper for cddb errors.
 * @param int Error Value
 */
void kmusicdb::cdpError(int e)
{
	// error handling slot for cdparanoia errors.
	// we handle cd not found specially because we want the option of retrying.
	if (e==CDPARA_NO_CD)
	{
		if (KMessageBox::warningYesNo(this, i18n("No CD found in drive"), i18n("No CD found in drive"), i18n("Retry"), i18n("Cancel")) == KMessageBox::Yes)
			cdp->get();
		else
			cdpCleanup();
	}
	else
		errBox(e);
}



/* *************************************************************************** */
/* ***************************** SEARCH FUNCTIONS **************************** */
/* *************************************************************************** */
/**
 * Updates second combobox in title search when the first one is changed.
 * @param int Sets the value of the KComboBox
 */
void
kmusicdb::update_toolbar_combobox(int t)
{
	if (!music)
		return;
	int z = t + 1;
	if (z==KBTE || z==KBOXSET)
		return;
	// the value coming off the combobox is usually 1 less than we need it to be so we add one to compensate.
	QStringList *temp = new QStringList();
	if (!temp)
		exit(1);
	stat=music->populateCategoryQStringList(temp, z);
	if (stat < 0)
		goto cleanup;

	toolBar("titleSearch")-> getCombo(TITLE_LIST_COMBO)->clear();
	toolBar("titleSearch")-> getCombo(TITLE_LIST_COMBO)->insertStringList(*temp);
	return;

cleanup:
	if (stat < 0)
		errBox(stat);
	delete temp;
}

/**
 * Dummy slot for disposing of run time error messages.
 * @param int Integer
 */
void
kmusicdb::dummy_slot(int) const
{
}

/**
 * Opens dialog for generating report.
 */
void
kmusicdb::slotCategorySearch()
{
	if (!music)
		return;

	if (toolBar("titleSearch")->getCombo(TITLE_LIST_COMBO)-> currentText().isEmpty())
	{
		errBox(COMBO_VALUE_EMPTY);
		return;
	}

	windowInc();
	report = new reportDialog(0, i18n("Category Search"), REPORT_CATEGORYSEARCH);
	if (!report)
		exit(1);
	reportProvides r=dbReport::dbreportProvides(REPORT_CATEGORYSEARCH);
	report->widget()->updateInfo(reportOptions, color, &r);
	connect(report, SIGNAL(generateReport(reportInfo, colorInfo,int)), this, SLOT(generate_report(reportInfo, colorInfo,int)));
	connect(report,SIGNAL(destroyed()),this,SLOT(windowDec()));
	updateGUI(100, i18n("Ready..."));
	report->show();
}

/**
 * Opens dialog for generating report based on info from name search toolbar.
 */
void
kmusicdb::slotNameSearch()
{
	if (!music)
		return;

	QString x = toolBar("nameSearch")->getLined(NAME_LINED)->text();
	if (x == "")
	{
		errBox(SEARCH_STRING_EMPTY);
		return;
	}

	windowInc();
	report = new reportDialog(0, i18n("Name Search"), REPORT_NAMESEARCH);
	if (!report)
		exit(1);
	reportProvides r=dbReport::dbreportProvides(REPORT_NAMESEARCH);
	report->widget()->updateInfo(reportOptions, color, &r);
	connect(report, SIGNAL(generateReport(reportInfo, colorInfo,int)), this,
		 SLOT(generate_report(reportInfo, colorInfo,int)));
	connect(report,SIGNAL(destroyed()),this,SLOT(windowDec()));
	updateGUI(100, i18n("Ready..."));
	report->show();
}




/* *************************************************************************** */
/* ****************************** REPORT FUNCTIONS *************************** */
/* *************************************************************************** */
/** Artist report. */
void
kmusicdb::slotArtistRep()
{
	if (!music)
		return;

	windowInc();
	report = new reportDialog(0, i18n("Artist"), REPORT_ARTISTS);
	if (!report)
		exit(1);
	reportProvides r=dbReport::dbreportProvides(REPORT_ARTISTS);
	report->widget()->updateInfo(reportOptions, color, &r);
	connect(report, SIGNAL(generateReport(reportInfo, colorInfo,int)), this,
		 SLOT(generate_report(reportInfo, colorInfo,int)));
	connect(report,SIGNAL(destroyed()),this,SLOT(windowDec()));
	updateGUI(100, i18n("Ready..."));
	report->show();
}

/** Bands To Explore report. */
void
kmusicdb::slotBteRep()
{
	if (!music)
		return;

	windowInc();
	report = new reportDialog(0, i18n("Bands To Explore"), REPORT_BTE);
	if (!report)
		exit(1);
	reportProvides r=dbReport::dbreportProvides(REPORT_BTE);
	report->widget()->updateInfo(reportOptions, color, &r);
	connect(report, SIGNAL(generateReport(reportInfo, colorInfo,int)), this,	SLOT(generate_report(reportInfo, colorInfo,int)));
	connect(report,SIGNAL(destroyed()),this,SLOT(windowDec()));
	updateGUI(100, i18n("Ready..."));
	report->show();
}

/** Collection report. */
void
kmusicdb::slotCollectionRep()
{
	if (!music)
		return;

	windowInc();
	report = new reportDialog(0, i18n("Collection"), REPORT_COLLECTION);
	if (!report)
		exit(1);
	reportProvides r=dbReport::dbreportProvides(REPORT_COLLECTION);
	report->widget()->updateInfo(reportOptions, color, &r);
	connect(report, SIGNAL(generateReport(reportInfo, colorInfo,int)), this, SLOT(generate_report(reportInfo, colorInfo,int)));
	connect(report,SIGNAL(destroyed()),this,SLOT(windowDec()));
	updateGUI(100, i18n("Ready..."));
	report->show();
}

/**
 * Custom Report.
 */
void
kmusicdb::slotCustomRep()
{
	if (!music)
		return;

	windowInc();
	report = new reportDialog(0, i18n("Custom"), REPORT_CUSTOM);
	if (!report)
		exit(1);
	reportProvides r=dbReport::dbreportProvides(REPORT_CUSTOM);
	report->widget()->updateInfo(reportOptions, color, &r);
	connect(report, SIGNAL(generateReport(reportInfo, colorInfo,int)), this, SLOT(generate_report(reportInfo, colorInfo,int)));
	connect(report,SIGNAL(destroyed()),this,SLOT(windowDec()));
	updateGUI(100, i18n("Ready..."));
	report->show();
}

/** Date Range report. */
void
kmusicdb::slotDateRangeRep()
{
	if (!music)
		return;

	windowInc();
	report = new reportDialog(0, i18n("Date Range"), REPORT_DATERANGE);
	if (!report)
		exit(1);
	reportProvides r=dbReport::dbreportProvides(REPORT_DATERANGE);
	report->widget()->updateInfo(reportOptions, color, &r);
	connect(report, SIGNAL(generateReport(reportInfo, colorInfo,int)), this, SLOT(generate_report(reportInfo, colorInfo,int)));
	connect(report,SIGNAL(destroyed()),this,SLOT(windowDec()));
	updateGUI(100, i18n("Ready..."));
	report->show();
}

/** Statistics report. */
void
kmusicdb::slotStatistics()
{
	if (!music)
		return;

	windowInc();
	report = new reportDialog(0, i18n("Statistics"), REPORT_STATS);
	if (!report)
		exit(1);
	reportProvides r=dbReport::dbreportProvides(REPORT_STATS);
	report->widget()->updateInfo(reportOptions, color, &r);
	connect(report, SIGNAL(generateReport(reportInfo, colorInfo,int)), this, SLOT(generate_report(reportInfo, colorInfo,int)));
	connect(report,SIGNAL(destroyed()),this,SLOT(windowDec()));
	updateGUI(100, i18n("Ready..."));
	report->show();
}

/**
 * Top titles Report.
 */
void
kmusicdb::slotTopCDRep()
{
	if (!music)
		return;

	windowInc();
	report = new reportDialog(0, i18n("Top Titles"), REPORT_TOPTITLES);
	if (!report)
		exit(1);
	reportProvides r=dbReport::dbreportProvides(REPORT_TOPTITLES);
	report->widget()->updateInfo(reportOptions, color, &r);
	connect(report, SIGNAL(generateReport(reportInfo, colorInfo,int)), this, SLOT(generate_report(reportInfo, colorInfo,int)));
	connect(report,SIGNAL(destroyed()),this,SLOT(windowDec()));
	updateGUI(100, i18n("Ready..."));
	report->show();
}

/**
 * Generates and displays report.
 * @param reportInfo Struct with report options
 * @param colorInfo Struct with color options
 * @param int Report Type
 */
void
kmusicdb::generate_report(reportInfo i, colorInfo color, int rt)
{
	if (!music)
		return;

	updateGUI(0, i18n("Generating Report..."));
	// name is used for the name of the report window.
	HTMLReport *report=0;
	QString filestring, x, name;
	int y;
	QListViewItem *item;
	KURL rurl;
	QFile f;
	dbReport *rep=new dbReport(music);
	if (!rep)
		exit(1);

	switch(rt)
	{
		case REPORT_CATEGORYSEARCH:
			// get the value of the first and set the k type.
			// K____ + 1 is the general rule here for matching combobox items to a Ksomething.
			y = toolBar("titleSearch")->getCombo(TITLE_CAT_COMBO)->currentItem() + 1;
			// get the value of the second
			x = toolBar("titleSearch")->getCombo(TITLE_LIST_COMBO)->currentText();
			cerr << "kmusicdb::titlesearch:	x: " << x << " y: " << y << endl;
			stat = rep->categorySearchReport(filestring, &i, &color, y, x);
			name = i18n("Category Search Report");
			break;
		case REPORT_NAMESEARCH:
			// get the value of the second
			x = toolBar("nameSearch")->getLined(NAME_LINED)->text();
			// get the value of the first and set the k type.
			y = toolBar("nameSearch")->getCombo(NAME_COMBO)->currentItem();
			if (y==0)
				y=ARTIST;
			else if (y==1)
				y=TITLE;
			else if (y==2)
				y=TRACK;
			else
				y=LYRIC;
			cerr << "kmusicdb::namesearch:	x:" << x << " y: " << y << endl;
			updateProgress(20);
			stat = rep->nameSearchReport(filestring, &i, &color, x, y);
			name = i18n("Name Search Report");
			break;
		case REPORT_TBYA:
			stat = rep->titlesByArtist(filestring, &i, &color, aid);
			name = i18n("Titles By Artist Report");
			break;
		case REPORT_TBYT:
			item = mid->listview()->selectedItem();
			stat = rep->tracksByTitle(filestring, &i, &color, item->text(mid->listview()->columns()));
			name = i18n("Tracks By Title Report");
			break;
		case REPORT_ARTISTS:
			stat = rep->artistReport(filestring, &i, &color);
			name = i18n("Artist Report");
			break;
		case REPORT_COLLECTION:
			stat = rep->collectionReport(filestring, &i, &color);
			name = i18n("Collection Report");
			break;
		case REPORT_DATERANGE:
			stat = rep->dateRangeReport(filestring, &i, &color);
			name = i18n("Date Range Report");
			break;
		case REPORT_BTE:
			stat = rep->bteReport(filestring, &i, &color);
			name = i18n("Bands To Explore Report");
			break;
		case REPORT_STATS:
			stat = rep->statisticsReport(filestring, &i, &color);
			name = i18n("Statistics Report");
			break;
		case REPORT_CUSTOM:
			rurl = KFileDialog::getOpenURL(0, i18n("*.sql|SQL Files"), this, i18n("SQL Query"));
			if (rurl.path() == NULL)
				goto cleanup;
			if (!rurl.isLocalFile())
			{
				updateStatusBar(i18n("Downloading to temp file..."));
				if (KIO::NetAccess::download(rurl, name, this))
					rurl.setPath(name);
			}
			f.setName(rurl.path());
			if (f.open(IO_ReadOnly))
			{
				QTextStream t(&f);
				x.append(t.readLine());
				f.close();
			}
			else
			{
				KMessageBox::error(this, i18n("Could not read from file."), i18n("Read File..."));
			}
			cerr << "kmusicdb::Custom Report: " << x << endl;
			if (!rurl.isLocalFile())
				KIO::NetAccess::removeTempFile(name);
			stat = rep->customReport(filestring, &i, &color, x);
			name = i18n("Custom Report");
			break;
		case REPORT_TOPTITLES:
			stat = rep->topTitlesReport(filestring, &i, &color);
			name = i18n("Top Titles Report");
			break;
		default:
			goto cleanup;
	};
	delete rep;
	if (stat < 0)
	{
		errBox(stat);
		return;
	}
	updateProgress(50);

	windowInc();
	report = new HTMLReport(0, name, filestring);
	if (!report)
		exit(1);
	connect(report,SIGNAL(destroyed()),this,SLOT(windowDec()));
	updateGUI(100, i18n("Ready..."));
	report->show();
	report->manualResize();
	return;

cleanup:
	delete rep;

}

/**
 * Titles By Artist Report.
 */
void
kmusicdb::slotTbya()
{
	if (!music)
		return;

	windowInc();
	report = new reportDialog(0, i18n("Titles By Artist"), REPORT_TBYA);
	if (!report)
		exit(1);
	reportProvides r=dbReport::dbreportProvides(REPORT_TBYA);
	report->widget()->updateInfo(reportOptions, color, &r);
	connect(report, SIGNAL(generateReport(reportInfo, colorInfo,int)), this, SLOT(generate_report(reportInfo, colorInfo,int)));
	connect(report,SIGNAL(destroyed()),this,SLOT(windowDec()));
	updateGUI(100, i18n("Ready..."));
	report->show();
}

/**
 * Tracks By Title Report.
 */
void
kmusicdb::slotTrbyt()
{
	if (!music)
		return;

	if (mid->listview()->selectedItem() == 0)
		return;
	if (mid->listview()->selectedItem()->childCount())
	{
		errBox(BOXSET_NO_ACTION);
		return;
	}

	windowInc();
	report = new reportDialog(0, i18n("Tracks By Title"), REPORT_TBYT);
	if (!report)
		exit(1);
	reportProvides r=dbReport::dbreportProvides(REPORT_TBYT);
	report->widget()->updateInfo(reportOptions, color, &r);
	connect(report, SIGNAL(generateReport(reportInfo, colorInfo,int)), this, SLOT(generate_report(reportInfo, colorInfo,int)));
	connect(report,SIGNAL(destroyed()),this,SLOT(windowDec()));
	updateGUI(100, i18n("Ready..."));
	report->show();
}




/* *************************************************************************** */
/* ****************************** TRACK FUNCTIONS **************************** */
/* *************************************************************************** */
/** Add track slot.
 * @param QString CD ID
 * @param trackInfo Track Information
 * @param QListView Pointer to list view for updating
 */
void
kmusicdb::addTrack(QString cdid, trackInfo t, QListView *z)
{
	updateGUI(0, i18n("Adding Track..."));
	if (cdid.isEmpty() || t.name.isEmpty())
		return;
	// check for text formatting
	if (def->auto_format)
		t.name = formatText(t.name);
	else if (def->auto_capitalize)
		t.name = capitalizeWords(t.name);

	stat = music->addTrack(&t, cdid);
	if (stat < 0)
	{
		errBox(stat);
		music->populateTrackQListView(z, cdid, tr_op);
		z->topLevelWidget()->raise();
		if (z->firstChild())
			z->setSelected(z->firstChild(), TRUE);
		return;
	}

	if (!z)
		return;
	updateProgress(50);

	musicQListViewItem *item = new musicQListViewItem(z, t.side, "999", t.name, t.length, t.notes, t.trackid);
	if (!item)
		exit(1);
	z->ensureItemVisible(item);

//	z->sort();
	int tn=1;
	QString side="0";
	QListViewItemIterator it(z);
	for (; it.current(); ++it)
	{
		if (side < it.current()->text(0))
			tn=1;
		it.current()->setText(1, QString::number(tn));
		side=it.current()->text(0);
		tn++;
	}


	updateGUI(100, i18n("Ready..."));

}

/**
 * Delete track slot.
 * @param QString Track ID
 * @param QString CD ID
 * @param QListView Pointer to QListView for updating
 */
void
kmusicdb::deleteTrack(QString trackid, QString cdid, QListView *z)
{
	updateGUI(0, i18n("Deleting Track..."));
	if (trackid.isEmpty() || cdid.isEmpty())
		return;

	stat = music->deleteTrack(trackid, cdid);
	if (stat < 0)
	{
		errBox(stat);
		music->populateTrackQListView(z, cdid, tr_op);
		z->topLevelWidget()->raise();
		if (z->firstChild())
			z->setSelected(z->firstChild(), TRUE);
		return;
	}

	updateProgress(60);
	if (!z)
		return;
	delete z->currentItem();

	int tn=1;
	QString side="0";
	QListViewItemIterator it(z);
	for (; it.current(); ++it)
	{
		if (side < it.current()->text(0))
			tn=1;
		it.current()->setText(1, QString::number(tn));
		side=it.current()->text(0);
		tn++;
	}
	updateGUI(100, i18n("Ready..."));
}

/** Move track down slot
 * @param QString CDID
 * @param trackInfo top track
 * @param trackInfo bottom track
 * @param QListView Pointer to listview for updating
 */
void
kmusicdb::trackDown(QString cdid, trackInfo top, trackInfo bottom, QListView *z)
{
	updateGUI(0, i18n("Moving Track Down..."));
	if (top.trackid.isEmpty() || bottom.trackid.isEmpty() || cdid.isEmpty())
		return;

	if (!z)
		return;
	QListViewItem *item = z->selectedItem();
	if (!item)
		return;

	stat = music->trackDown(&top, &bottom);
	if (stat < 0)
	{
		errBox(stat);
		music->populateTrackQListView(z, cdid, tr_op);
		z->topLevelWidget()->raise();
		item=z->firstChild();
		if (item)
			z->setSelected(item,TRUE);
		return;
	}

	updateProgress(60);
	QString name, length, trackno, notes;
	QListViewItemIterator it(item);
	it++;
	name = it.current()->text(2);
	length = it.current()->text(3);
	notes = it.current()->text(4);
	it.current()->setText(2, item->text(2));
	it.current()->setText(3, item->text(3));
	it.current()->setText(4, item->text(4));
	item->setText(2, name);
	item->setText(3, length);
	item->setText(4, notes);
	z->setSelected(it.current(), TRUE);

	updateGUI(100, i18n("Ready..."));
}

/** Move track down slot
 * @param QString CDID
 * @param trackInfo top track
 * @param trackInfo bottom track
 * @param QListView Pointer to listview for updating
 */
void
kmusicdb::trackUp(QString cdid, trackInfo top, trackInfo bottom, QListView *z)
{
	updateGUI(0, i18n("Moving Track Up..."));
	if (top.trackid.isEmpty() || bottom.trackid.isEmpty() || cdid.isEmpty())
		return;

	if (!z)
		return;
	QListViewItem *item = z->selectedItem();
	if (!item)
		return;

	stat = music->trackUp(&top,&bottom);
	if (stat < 0)
	{
		errBox(stat);
		music->populateTrackQListView(z, cdid, tr_op);
		z->topLevelWidget()->raise();
		item = z->firstChild();
		if (item)
			z->setSelected(item,TRUE);
		return;
	}

	updateProgress(60);
	QString name, length, notes;

	QListViewItemIterator it(item);
	it--;
	if (it.current() == 0)
		return;
	name = it.current()->text(2);
	length = it.current()->text(3);
	notes = it.current()->text(4);
	it.current()->setText(2, item->text(2));
	it.current()->setText(3, item->text(3));
	it.current()->setText(4, item->text(4));
	item->setText(2, name);
	item->setText(3, length);
	item->setText(4, notes);
	z->setSelected(it.current(), TRUE);

	updateGUI(100, i18n("Ready..."));
}

/** Update track slot.
 * @param QString CDID
 * @param trackInfo Track information
 * @param QListView Pointer to listview for updating
 */
void
kmusicdb::updateTrack(QString cdid, trackInfo t, QListView *z)
{
	updateGUI(0, i18n("Updating Track..."));
	if (t.name.isEmpty() || cdid.isEmpty())
		return;
	// check for text formatting
	if (def->auto_format)
		t.name = formatText(t.name);
	else if (def->auto_capitalize)
		t.name = capitalizeWords(t.name);

	updateProgress(20);
	stat = music->updateTrack(&t);
	if (stat < 0)
	{
		errBox(stat);
		music->populateTrackQListView(z, cdid, tr_op);
		z->topLevelWidget()->raise();
		if (z->firstChild())
			z->setSelected(z->firstChild(), TRUE);
		return;
	}
	updateProgress(80);
	if (!z)
		return;
	z->currentItem()->setText(0, t.side);
	z->currentItem()->setText(2, t.name);
	z->currentItem()->setText(3, t.length);
	z->currentItem()->setText(4, t.notes);
	z->sort();
/*	int tn=1;
	QString side="0";
	QListViewItemIterator it(z);
	for (; it.current(); ++it)
	{
		if (side < it.current()->text(0))
			tn=1;
		it.current()->setText(1, QString::number(tn));
		side=it.current()->text(0);
		tn++;
	}*/
	stat=music->populateTrackQListView(z, cdid, tr_op);//had to resort to this due to lack of multicolumn sorting
	if (stat < 0)
		errBox(stat);
	updateGUI(100, i18n("Ready..."));

}
/**
 * Repopulates the track information in a QListView.
 * @param QListView Pointer to QListVIew for updating
 * @param QString CDID
 */
void kmusicdb::updateTrackInfo(QListView *t, QString cdid)
{
	stat=music->populateTrackQListView(t, cdid, tr_op);
	if (stat<0)
		errBox(stat);
}
/**
 * Updates lyrics for a track.
 * @param QString Lyric
 * @param QString Track ID
 * @param bool Set TRUE on successful completion, FALSE otherwise.
 */
void kmusicdb::updateLyric(QString lyric, QString trackid, bool *success)
{
	updateGUI(0, i18n("Updating Lyric..."));
	// set the lyric in track id to QString lyric
	if (trackid.isEmpty())
		return;
	trackInfo track;
	track.trackid=trackid;
	track.lyric=lyric;
	stat=music->updateLyric(&track);
	if (stat < 0)
	{
		errBox(stat);
		*success=FALSE;
		return;
	} else
		*success=TRUE;
	updateGUI(100, i18n("Ready..."));
}
/**
 * Retrieves lyric and populates QTextEdit.
 * @param QTextEdit Item To Populate
 * @param QString Lyric
 * @param bool Used to check if there was a successful retrieval of lyrics.
 */
void kmusicdb::getLyric(QTextEdit *t, QString trackid, bool *goodretrieve)
{
	updateGUI(0, i18n("Retrieving Lyric..."));
	// retrieve the lyric from cdid.
	// then populate the qtextedit with it.
	if (trackid.isEmpty())
		return;
	trackInfo track;
	track.trackid=trackid;
	stat=music->getLyric(&track);
	if (stat < 0)
	{
		errBox(stat);
		*goodretrieve=FALSE;
		return;
	} else
		*goodretrieve=TRUE;
	t->setText(track.lyric);
	updateGUI(100, i18n("Ready..."));
}
/**
 * Called when a track is dragged from a track window.
 * @param QListView Pointer To Track QListView
 */
void kmusicdb::trackDragged(QListView *i)
{
	updateGUI(0, i18n("Dragging Track..."));
	QListViewItem *item=i->selectedItem();
	if (!item)
		return;
	trackInfo dd;
	cerr << "kmusicdb::trackDragged: " << item->text(0) << " " << item->text(2) << " " << item->text(3) << " " << item->text(4) << " " << item->text(5) << endl;
	dd.side=item->text(0);
	dd.name=item->text(2);
	dd.length=item->text(3);
	dd.notes=item->text(4);
	dd.trackid=item->text(5);
	stat=music->getLyric(&dd);
	if (stat < 0)
	{
		errBox(stat);
		i->setSelected(i->currentItem(),TRUE);
		return;
	}
	QDragObject *d = new trackInfoDrag( i, dd );
	d->dragCopy();
	updateGUI(100, i18n("Ready..."));
}
/**
 * Called when a track is dropped onto a track window.
 * @param QString Track Name
 * @param trackInfo Track Information
 * @param Pointer to Track QListView
 */
void kmusicdb::trackDropped(QString cdid, trackInfo i, QListView *z)
{
	updateGUI(0, i18n("Dropping Track..."));
	if (cdid.isEmpty() || i.name.isEmpty())
		return;
	// check for text formatting
	if (def->auto_format)
		i.name = formatText(i.name);
	else if (def->auto_capitalize)
		i.name = capitalizeWords(i.name);

	stat = music->addTrack(&i, cdid);
	if (stat < 0)
	{
		errBox(stat);
		music->populateTrackQListView(z, cdid, tr_op);
		if (z->firstChild())
			z->setSelected(z->firstChild(), TRUE);
		return;
	}

	updateProgress(40);
	stat = music->updateLyric(&i);
	if (stat < 0)
	{
		errBox(stat);
		return;
	}
	updateProgress(80);

	if (!z)
		return;
	updateProgress(50);

	musicQListViewItem *item = new musicQListViewItem(z, i.side, QString::number(z->childCount()+1), i.name, i.length, i.notes, i.trackid);
	if (!item)
		exit(1);

	updateGUI(100, i18n("Ready..."));
}




/* *************************************************************************** */
/* **************************** CATEGORY FUNCTIONS *************************** */
/* *************************************************************************** */
/**
 * Slot to sort open category dialogs.
 * @param bool TRUE is ascending, FALSE is descending
 */
void
kmusicdb::sortCategorySlot(bool a) const
{
	cerr << "kmusicdb::Sorting Live Category: " << a << endl;
	for (int i = 0; i < 7; i++)
	{
		if (catdialog[i])
			catdialog[i]->widget()->list()->setSorting(0, a);
	}
}

/**
 * Delete pointer from QMap.
 * @param titleDialog Pointer to titleDialog to remove
 */
void
kmusicdb::deleteFromQMap(titleDialog *z) const
{
	cerr << "kmusicdb::Delete from QMap: " << z << endl;
	twm->remove(z);
}

/**
 * Delete pointer from QMap.
 * @param preferencesDialog Pointer to preferencesDialog to remove
 */
void
kmusicdb::deleteFromQMap(preferencesDialog *z) const
{
	cerr << "kmusicdb::Delete from QMap: " << z << endl;
	pwm->remove(z);
}

/** This slot will open the popup for browsing, adding, editing and removing bte names. */
void
kmusicdb::slotBteBrowse()
{
	updateGUI(0, i18n("Category Browse..."));
	if (catdialog[KBTE])
	{
		cerr << "kmusicdb::BTE Already open, raised to top." << endl;
		catdialog[KBTE]->raise();
		return;
	}
	windowInc();
	catdialog[KBTE] = new categoryDialog(0, KBTE, i18n("Bands To Explore"),def->category_sort_order);
	if (!catdialog[KBTE])
		exit(1);
	stat=music->populateCategoryQListView(catdialog[KBTE]->widget()->list(), KBTE, c_op);
	if (stat<0)
	{
		errBox(stat);
		return;
	}
	connect(catdialog[KBTE], SIGNAL(addSignal(categoryInfo)), this,SLOT(addCategory(categoryInfo)));
	connect(catdialog[KBTE], SIGNAL(deleteSignal(categoryInfo)), this,SLOT(deleteCategory(categoryInfo)));
	connect(catdialog[KBTE], SIGNAL(updateSignal(categoryInfo)),this, SLOT(updateCategory(categoryInfo)));
	connect(catdialog[KBTE], SIGNAL(signalClose(int)), this,SLOT(windowManager(int)));
	connect(catdialog[KBTE], SIGNAL(transferArtist(categoryInfo)), this,SLOT(transferArtist(categoryInfo)));
	updateGUI(100, i18n("Ready..."));
	catdialog[KBTE]->show();
}

/** Slot for editing boxsets. */
void
kmusicdb::slotEditBoxset()
{
	updateGUI(0, i18n("Category Browse..."));
	if (catdialog[KBOXSET])
	{
		cerr << "kmusicdb::Boxset Already open, raised to top." << endl;
		catdialog[KBOXSET]->raise();
		return;
	}
	windowInc();
	catdialog[KBOXSET] = new categoryDialog(0, KBOXSET, i18n("Boxset"),def->category_sort_order);
	if (!catdialog[KBOXSET])
		exit(1);
	stat=music->populateCategoryQListView(catdialog[KBOXSET]->widget()->list(), KBOXSET, c_op);
	if (stat<0)
	{
		errBox(stat);
		return;
	}
	connect(catdialog[KBOXSET], SIGNAL(addSignal(categoryInfo)), this, SLOT(addCategory(categoryInfo)));
	connect(catdialog[KBOXSET], SIGNAL(deleteSignal(categoryInfo)), this, SLOT(deleteCategory(categoryInfo)));
	connect(catdialog[KBOXSET], SIGNAL(updateSignal(categoryInfo)), this, SLOT(updateCategory(categoryInfo)));
	connect(catdialog[KBOXSET], SIGNAL(signalClose(int)), this, SLOT(windowManager(int)));
	updateGUI(100, i18n("Ready..."));
	catdialog[KBOXSET]->show();
}

/** Slot for editing the different formats. */
void
kmusicdb::slotEditFormat()
{
	updateGUI(0, i18n("Category Browse..."));
	if (catdialog[KFORMAT])
	{
		cerr << "kmusicdb::Format Already open, raised to top." << endl;
		catdialog[KFORMAT]->raise();
		return;
	}
	windowInc();
	catdialog[KFORMAT] = new categoryDialog(0, KFORMAT, i18n("Format"),def->category_sort_order);
	if (!catdialog[KFORMAT])
		exit(1);
	stat=music->populateCategoryQListView(catdialog[KFORMAT]->widget()->list(), KFORMAT, c_op);
	if (stat<0)
	{
		errBox(stat);
		return;
	}
	connect(catdialog[KFORMAT], SIGNAL(addSignal(categoryInfo)), this, SLOT(addCategory(categoryInfo)));
	connect(catdialog[KFORMAT], SIGNAL(deleteSignal(categoryInfo)), this, SLOT(deleteCategory(categoryInfo)));
	connect(catdialog[KFORMAT], SIGNAL(updateSignal(categoryInfo)), this, SLOT(updateCategory(categoryInfo)));
	connect(catdialog[KFORMAT], SIGNAL(signalClose(int)), this, SLOT(windowManager(int)));
	updateGUI(100, i18n("Ready..."));
	catdialog[KFORMAT]->show();
}

/** Slot for editing the different genres. */
void
kmusicdb::slotEditGenre()
{
	updateGUI(0, i18n("Category Browse..."));
	if (catdialog[KGENRE])
	{
		cerr << "kmusicdb::Genre Already open, raised to top." << endl;
		catdialog[KGENRE]->raise();
		return;
	}
	windowInc();
	catdialog[KGENRE] = new categoryDialog(0, KGENRE, i18n("Genre"), def->category_sort_order);
	if (!catdialog[KGENRE])
		exit(1);
	stat=music->populateCategoryQListView(catdialog[KGENRE]->widget()->list(), KGENRE, c_op);
	if (stat<0)
	{
		errBox(stat);
		return;
	}
	connect(catdialog[KGENRE], SIGNAL(addSignal(categoryInfo)), this, SLOT(addCategory(categoryInfo)));
	connect(catdialog[KGENRE], SIGNAL(deleteSignal(categoryInfo)), this, SLOT(deleteCategory(categoryInfo)));
	connect(catdialog[KGENRE], SIGNAL(updateSignal(categoryInfo)), this, SLOT(updateCategory(categoryInfo)));
	connect(catdialog[KGENRE], SIGNAL(signalClose(int)), this, SLOT(windowManager(int)));
	updateGUI(100, i18n("Ready..."));
	catdialog[KGENRE]->show();
}

/** Slot for editing the different labels. */
void
kmusicdb::slotEditLabel()
{
	updateGUI(0, i18n("Category Browse..."));
	if (catdialog[KLABEL])
	{
		cerr << "kmusicdb::Label Already open, raised to top." << endl;
		catdialog[KLABEL]->raise();
		return;
	}
	windowInc();
	catdialog[KLABEL] = new categoryDialog(0, KLABEL, i18n("Label"), def->category_sort_order);
	if (!catdialog[KLABEL])
		exit(1);
	stat=music->populateCategoryQListView(catdialog[KLABEL]->widget()->list(), KLABEL, c_op);
	if (stat<0)
	{
		errBox(stat);
		return;
	}
	connect(catdialog[KLABEL], SIGNAL(addSignal(categoryInfo)), this, SLOT(addCategory(categoryInfo)));
	connect(catdialog[KLABEL], SIGNAL(deleteSignal(categoryInfo)), this, SLOT(deleteCategory(categoryInfo)));
	connect(catdialog[KLABEL], SIGNAL(updateSignal(categoryInfo)), this, SLOT(updateCategory(categoryInfo)));
	connect(catdialog[KLABEL], SIGNAL(signalClose(int)), this, SLOT(windowManager(int)));
	updateGUI(100, i18n("Ready..."));
	catdialog[KLABEL]->show();
}

/** Slot for editing the methods (music ones anyways). */
void
kmusicdb::slotEditMethod()
{
	updateGUI(0, i18n("Category Browse..."));
	if (catdialog[KMETHOD])
	{
		cerr << "kmusicdb::Method Already open, raised to top." << endl;
		catdialog[KMETHOD]->raise();
		return;
	}
	windowInc();
	catdialog[KMETHOD] = new categoryDialog(0, KMETHOD, i18n("Method"),def->category_sort_order);
	if (!catdialog[KMETHOD])
		exit(1);
	stat=music->populateCategoryQListView(catdialog[KMETHOD]->widget()->list(), KMETHOD, c_op);
	if (stat<0)
	{
		errBox(stat);
		return;
	}
	connect(catdialog[KMETHOD], SIGNAL(addSignal(categoryInfo)), this, SLOT(addCategory(categoryInfo)));
	connect(catdialog[KMETHOD], SIGNAL(deleteSignal(categoryInfo)), this, SLOT(deleteCategory(categoryInfo)));
	connect(catdialog[KMETHOD], SIGNAL(updateSignal(categoryInfo)), this, SLOT(updateCategory(categoryInfo)));
	connect(catdialog[KMETHOD], SIGNAL(signalClose(int)), this, SLOT(windowManager(int)));
	updateGUI(100, i18n("Ready..."));
	catdialog[KMETHOD]->show();
}

/** Slot for editing the different types. */
void
kmusicdb::slotEditType()
{
	updateGUI(0, i18n("Category Browse..."));
	if (catdialog[KTYPE])
	{
		cerr << "kmusicdb::Type Already open, raised to top." << endl;
		catdialog[KTYPE]->raise();
		return;
	}
	windowInc();
	catdialog[KTYPE] = new categoryDialog(0, KTYPE, i18n("Type"), def->category_sort_order);
	if (!catdialog[KTYPE])
		exit(1);
	stat=music->populateCategoryQListView(catdialog[KTYPE]->widget()->list(), KTYPE, c_op);
	if (stat<0)
	{
		errBox(stat);
		return;
	}
	connect(catdialog[KTYPE], SIGNAL(addSignal(categoryInfo)), this, SLOT(addCategory(categoryInfo)));
	connect(catdialog[KTYPE], SIGNAL(deleteSignal(categoryInfo)), this, SLOT(deleteCategory(categoryInfo)));
	connect(catdialog[KTYPE], SIGNAL(updateSignal(categoryInfo)), this, SLOT(updateCategory(categoryInfo)));
	connect(catdialog[KTYPE], SIGNAL(signalClose(int)), this, SLOT(windowManager(int)));
	updateGUI(100, i18n("Ready..."));
	catdialog[KTYPE]->show();
}

/** Add category slot.
 * @param categoryInfo Struct containing category information
 */
void
kmusicdb::addCategory(categoryInfo s)
{
	updateGUI(0, i18n("Adding Category..."));

	if (s.name.isEmpty())
	{
		errBox(SEARCH_STRING_EMPTY);
		catdialog[s.ctype]->raise();
		return;
	}
	// check for text formatting
	if (def->auto_format)
		s.name = formatText(s.name);
	else if (def->auto_capitalize)
		s.name = capitalizeWords(s.name);

	stat = music->addCategory(&s);
	if (stat < 0)
	{
		errBox(stat);
		catdialog[s.ctype]->widget()->clearList();
		music->populateCategoryQListView(catdialog[s.ctype]->widget()->list(), s.ctype, c_op);
		catdialog[s.ctype]->raise();
		catdialog[s.ctype]->widget()->list()->setSelected(catdialog[s.ctype]->widget()->list()->currentItem(),TRUE);
		return;
	}
	updateProgress(40);

	musicQListViewItem *item=new musicQListViewItem(catdialog[s.ctype]->widget()->list(),s.name,s.xid);
	if (!item)
		exit(1);
	catdialog[s.ctype]->widget()->list()->ensureItemVisible(item);

	// now we must real time update the titledialog's that are open
	updateProgress(70);
	updateLists(s.ctype);
	if (t_op->fid || t_op->gid || t_op->lid || t_op->mid || t_op->tid)
		updateTitleView();
	updateGUI(100, i18n("Ready..."));
}

/** Delete category slot.
 * @param categoryInfo Struct containing category information
 */
void
kmusicdb::deleteCategory(categoryInfo s)
{
	updateGUI(0, i18n("Deleting Category..."));

	if (s.xid.isEmpty())
	{
		cerr << "kmusicdb::Delete Category: empty name" << endl;
		errBox(SEARCH_STRING_EMPTY);
		catdialog[s.ctype]->raise();
		return;
	}

	stat = music->deleteCategory(&s);
	if (stat < 0)
	{
		errBox(stat);
		catdialog[s.ctype]->widget()->clearList();
		music->populateCategoryQListView(catdialog[s.ctype]->widget()->list(), s.ctype, c_op);
		catdialog[s.ctype]->raise();
		catdialog[s.ctype]->widget()->list()->setSelected(catdialog[s.ctype]->widget()->list()->currentItem(),TRUE);
		return;
	}

	updateProgress(50);
	// value of t here matches Kwhatever
	delete catdialog[s.ctype]->widget()->list()->currentItem();

	// now we must real time update the titledialog's that are open
	updateProgress(80);
	updateLists(s.ctype);
	if (t_op->fid || t_op->gid || t_op->lid || t_op->mid || t_op->tid)
		updateTitleView();
	updateGUI(100, i18n("Ready..."));
}

/** Transfer artist slot.
 * @param categoryInfo Struct containing info to transfer
 */
void
kmusicdb::transferArtist(categoryInfo t)
{
	updateStatusBar(i18n("Transferring Artist..."));
	if (t.xid.isEmpty())
		return;
	//popup dialog, set defaults and name t. add artist, remove
	//bte name and update list.
	artistInfo *temp = new artistInfo(*artist);
	if (!temp)
		exit(1);
	temp->aid=artist->aid;
	temp->notes=artist->notes;
	temp->first=artist->first;
	temp->last=artist->last;
	temp->follow=artist->follow;
	temp->homepage=artist->homepage;
	temp->name = t.name;
	artistdialog = new artistDialog(0, i18n("New Artist..."), i18n("&Add"), def->artist_autofill);
	if (!artistdialog)
		exit(1);
	artistdialog->widget()->updateTitles("0");
	connect(artistdialog, SIGNAL(addArtist(artistInfo)), this, SLOT(addArtist(artistInfo)));
	temp->follow=def->artist_follow;
	artistdialog->widget()->updateInfo(temp, def->artist_year, def->artist_autofill);
	artistdialog->show();
	deleteCategory(t);
	delete temp;

	updateStatusBar(i18n("Ready..."));
}

/** Update category slot.
 * @param categoryInfo Struct holding new info
 */
void
kmusicdb::updateCategory(categoryInfo snew)
{
	updateGUI(0, i18n("Ready..."));

	if (snew.name.isEmpty() || snew.xid.isEmpty())
	{
		cerr << "kmusicdb::Update Category: New or Old Name Empty" << endl;
		errBox(SEARCH_STRING_EMPTY);
		catdialog[snew.ctype]->raise();
		return;
	}

	// check for text formatting
	if (def->auto_format)
		snew.name = formatText(snew.name);
	else if (def->auto_capitalize)
		snew.name = capitalizeWords(snew.name);

	stat = music->updateCategory(&snew);
	if (stat < 0)
	{
		errBox(stat);
		catdialog[snew.ctype]->widget()->clearList();
		music->populateCategoryQListView(catdialog[snew.ctype]->widget()->list(), snew.ctype, c_op);
		catdialog[snew.ctype]->raise();
		catdialog[snew.ctype]->widget()->list()->setSelected(catdialog[snew.ctype]->widget()->list()->currentItem(),TRUE);
		return;
	}

	updateProgress(50);
	// t is the value of Kwhatever
	catdialog[snew.ctype]->widget()->list()->currentItem()->setText(0,snew.name);

	// now we must real time update the titledialog's that are open
	updateProgress(80);
	updateLists(snew.ctype);
	if (t_op->fid || t_op->gid || t_op->lid || t_op->mid || t_op->tid)
		updateTitleView();
	updateGUI(100, i18n("Ready..."));
}

/** Updates the category lists in the currently open windows. This function uses the QMap's pointers to fill the combo boxes with new information.
 * @param int Which Category
 */
void
kmusicdb::updateLists(int z)
{
	QStringList *cat;
	PreferencesDialogManager::Iterator pt;
	TitleDialogManager::Iterator it;
	cerr << "kmusicdb::Updating Lists" << endl;
	for (it = twm->begin(); it != twm->end(); it++)
	{
		cat = new QStringList();
		if (!cat)
			exit(1);
		stat=music->populateCategoryQStringList(cat, z);
		if (stat<0)
			goto cleanup;

		it.key()->widget()->populateCategoryBox(z, cat);

		delete cat;
	}
	for (pt = pwm->begin(); pt != pwm->end(); pt++)
	{
		cat = new QStringList();
		if (!cat)
			exit(1);
		stat=music->populateCategoryQStringList(cat, z);
		if (stat<0)
			goto cleanup;

		pt.key()->populateCategoryBoxPT(z, cat);

		delete cat;
	}
	return;
cleanup:
	delete cat;
	errBox(stat);
}

/** Manages the category menues to ensure category memory gets freed.
 * @param int Which Category
 */
void
kmusicdb::windowManager(int t)
{
	windowDec();
	cerr << "kmusicdb::windowManager: " << t << endl;
	// t is the value of Kwhatever
	if (catdialog[t])
		catdialog[t] = 0;

}



/* *************************************************************************** */
/* *********************** UPDATE INTERFACE FUNCTIONS ************************ */
/* *************************************************************************** */
/** Updates the Caption.
 * @param bool TRUE if connecting, FALSE if disconnecting
 */
void
kmusicdb::connectGUI(bool t)
{
	if (t)
		setCaption(i18n("Connected: %1 on %2 (%3)").arg(dbconnectinfo->db_name).arg(dbconnectinfo->db_location).arg(VERSION));
	else
		setCaption(i18n("Disconnected (%1)").arg(VERSION));
}

/** Sets new artist button, grays the delete option in the artist menu.
 * @param bool TRUE enables, FALSE disables
 */
void
kmusicdb::enableArtist(bool t)
{
	cerr << "kmusicdb::enableArtist: " << t << endl;
	// should ungray new artist menu's and toolbar
	listartist->setEnabled(t);
	toolBar()->setItemEnabled(3, t);
	newartist->setEnabled(t);

	// if blank gray both edit and delete, if not blank, then ungray only edit
	if (mid->getArtistText().isEmpty())
	{
		eartist->setEnabled(FALSE);
		toolBar()->setItemEnabled(4, FALSE);
	}
	else
	{
		eartist->setEnabled(t);
		toolBar()->setItemEnabled(4, t);
	}
	delartist->setEnabled(FALSE);
	toolBar()->setItemEnabled(5, FALSE);

}
/**
 * Enables or disables the CDDB menu/toolbar buttons.
 * @param bool TRUE enables, FALSE disables
 */
void kmusicdb::enableCDDB(bool t)
{
	if (!t)
	{
		addcddb->setEnabled(FALSE);
		submitcddb->setEnabled(FALSE);
		appendcddb->setEnabled(FALSE);
		updatecddb->setEnabled(FALSE);
		searchcddb->setEnabled(FALSE);
		toolBar()->setItemEnabled(11, FALSE);
	}
	else
	{
		mid->listview()->clearSelection();
		addcddb->setEnabled(TRUE);
		toolBar()->setItemEnabled(11, TRUE);
		searchcddb->setEnabled(TRUE);
	}
}

/**
 * Enables or disables the new and open db buttons.
 * @param bool TRUE enables, FALSE disables
 */
void kmusicdb::enableDbGUI(bool t){
	newdb->setEnabled(t);
	opendb->setEnabled(t);
	closedb->setEnabled(t);
	toolBar()->setItemEnabled(0, t);
	toolBar()->setItemEnabled(1, t);
}

/** Grays the delete artist buttons.
 * @param bool TRUE enables, FALSE disables
 */
void
kmusicdb::enableDeleteArtist(bool t)
{
	cerr << "kmusicdb::enableDeleteArtist: " << t << endl;
	enableEditDeleteTitle(FALSE);
	mid->listview()->clearSelection();
	if (mid->getArtistText().isEmpty())
	{
		delartist->setEnabled(FALSE);
		toolBar()->setItemEnabled(5, FALSE);
	}
	else
	{
		delartist->setEnabled(t);
		toolBar()->setItemEnabled(5, t);
	}
}

/** Grays the edit/delete title buttons.
 * @param bool TRUE enables, FALSE disables
 */
void
kmusicdb::enableEditDeleteTitle(bool t)
{
	cerr << "kmusicdb::enableEditDeleteTitle: " << t << endl;

	etitle->setEnabled(t);
	trbyt->setEnabled(t);
	deltitle->setEnabled(t);
	toolBar()->setItemEnabled(8, t);
	toolBar()->setItemEnabled(9, t);
	submitcddb->setEnabled(t);
	updatecddb->setEnabled(t);
	appendcddb->setEnabled(t);
}

/** Grays the most items in the menu.
 * @param bool TRUE enables, FALSE disables
 */
void
kmusicdb::enableItems(bool t)
{
	cerr << "kmusicdb::enableItems: " << t << endl;
	// take care of the center widget
	mid->enable(t);
	enableTitle(t);
	enableArtist(t);

	// search tool bars
	toolBar("nameSearch")->setEnabled(t);
	toolBar("titleSearch")->setEnabled(t);

	// the rest of the items
	closedb->setEnabled(t);
	opti->setEnabled(t);
	backupdb->setEnabled(t);
	restoredb->setEnabled(t);
	eformat->setEnabled(t);
	egenre->setEnabled(t);
	etype->setEnabled(t);
	emethod->setEnabled(t);
	elabel->setEnabled(t);
	eboxset->setEnabled(t);
	statistics->setEnabled(t);
	topcdrep->setEnabled(t);
	customrep->setEnabled(t);
	tbya->setEnabled(t);
	collectionrep->setEnabled(t);
	artistrep->setEnabled(t);
	daterangerep->setEnabled(t);
	bterep->setEnabled(t);
	btebrowse->setEnabled(t);

	// must clear out status bar
	if (!t)
	{
		updateArtistText("0");
		updateTotalTitles("0", "0");
		updateLength(i18n("00:00:00"));
	}

}

/** Grays some title options and toolbar buttons.
 * @param bool TRUE enables, FALSE disables
 */
void
kmusicdb::enableTitle(bool t)
{
	cerr << "kmusicdb::enableTitle: " << t << endl;
	if (mid->getArtistText().isEmpty())
	{
		toolBar()->setItemEnabled(7, FALSE);
		newtitle->setEnabled(FALSE);
		toolBar()->setItemEnabled(11, FALSE);
		addcddb->setEnabled(FALSE);
		searchcddb->setEnabled(FALSE);
		listtitle->setEnabled(FALSE);
	}
	else
	{
		toolBar()->setItemEnabled(7, t);
		newtitle->setEnabled(t);
		toolBar()->setItemEnabled(11, t);
		addcddb->setEnabled(t);
		searchcddb->setEnabled(t);
		listtitle->setEnabled(t);
	}

	enableEditDeleteTitle(FALSE);

}

/** Updates the number of artists displayed on the statusbar.
 * @param QString Number Of Titles
 */
void
kmusicdb::updateArtistText(const QString &text)
{
	QString s=text;
	if (s.isEmpty())
		s="0";

	statusBar()->changeItem(i18n(" Total Artists: %1").arg(s), 1);
}

/**
 * Code to fill up artist and title widgets.
 *
 * @param QString Artist Name
 */
void
kmusicdb::updateArtistView()
{
	QString temp;
	cerr << "kmusicdb::updateArtistView" << endl;
	if (!music)
		return;
	QString s = mid->getArtistText();
	QString z="0";
	mid->clearItems();

	QStringList *list = new QStringList();
	if (!list)
		exit(1);
	stat=music->populateQStringList(list, ARTIST, z);
	if (stat<0)
		goto cleanup;
	mid->populateArtist(list);
	mid->setArtistText(s);

	stat = music->total(temp,ARTIST,REPORT_ALL);
	if (stat < 0)
		goto cleanup;
	updateArtistText(temp);
	mid->combobox()->setFocus();

cleanup:
	if (stat < 0)
		errBox(stat);
	delete list;
}

/**
 * Updates the GUI progress bar and text.
 * @param int Progressbar Value
 * @param QString Statusbar Text
 */
void
kmusicdb::updateGUI(int progressLevel, QString statusBarMessage)
{
	sbp->setProgress(progressLevel);
	statusBar()->changeItem(statusBarMessage, 0);
}

/** Updates total time displayed on the statusbar.
 * @param QString Total Length
 */
void
kmusicdb::updateLength(const QString &text)
{
	QString s=text;
	if (s.isEmpty())
		s="00:00:00";
	statusBar()->changeItem(i18n(" Total Time: %1 ").arg(s),3);
}

/** Set the value of the progress bar on the statusbar.
 * @param int Percentage to set the progress bar
 */
void
kmusicdb::updateProgress(const int &a) const
{
	sbp->setProgress(a);
}

/** Updates the text on the statusbar.
 * @param char Text to display on the statusbar
 */
void
kmusicdb::updateStatusBar(const char *text)
{
	statusBar()->changeItem(text, 0);
}

/** Slot for updating title browser window.
 */
void
kmusicdb::updateTitleView()
{
	QString temp, temp2;
	static int loop=0;
	cerr << "kmusicdb::updateTitleView" << endl;
	if (!music)
		return;

	QString internal=mid->getArtistText();
	if (internal.isEmpty())
		return;

	stat = music->getAid(aid, internal);
	if (stat < 0)
		goto cleanup;

	stat = music->populateTitleQListView(mid->listview(), aid, t_op);
	if (stat < 0)
		goto cleanup;
	stat = music->totalBy(temp, aid, ARTIST, REPORT_ALL);
	if (stat < 0)
		goto cleanup;
	stat = music->totalBy(temp2, aid, ARTIST, REPORT_YES);
	if (stat < 0)
		goto cleanup;
	updateTotalTitles(temp, temp2);
	stat = music->totalLength(temp, aid, ARTIST);
	if (stat < 0)
		goto cleanup;
	updateLength(temp);
	// new data, better resize the columns.
	mid->resizeColumns();
	mid->sortList(def->default_column_sort, def->default_sort_order);
	//small bug, this doesn't get set automatically for some reason.
	enableEditDeleteTitle(FALSE);
	loop=0;
	return;

cleanup:
	errBox(stat);
	if (loop==0) {
		loop=1;
		updateArtistTitleView();
	} else {
		loop=0;
	}
		
}

/** Updates the number of titles displayed on the statusbar.
 * @param char Total titles not collected
 * @param char Total titles all together
 */
void
kmusicdb::updateTotalTitles(const char *t, const char *c)
{
	QString s = i18n(" Titles: %1/%2 ").arg(c).arg(t);
	statusBar()->changeItem(s, 2);
}



/* *************************************************************************** */
/* ****************************** MISC FUNCTIONS ***************************** */
/* *************************************************************************** */
/**
 * Resizing to keep columns lined up.
 * @param QResizeEvent Resize Event
 */
void
kmusicdb::resizeEvent(QResizeEvent *)
{
	cerr << "kmusicdb::resizeEvent" << endl;
	if (mid)
		mid->resizeColumns();
}

/** Settings dialog for the program.*/
void
kmusicdb::slotPreferences()
{
	windowInc();
	pref = new preferencesDialog(0, si, def, dbauto, t_op, reportOptions, color);
	if (!pref)
		exit(1);
	pwm->insert(pref, pwm->count());

	QStringList *cat[6];
	for (int i = 0; i < 6; i++)
	{
		if (!music)
			break;
		cat[i] = new QStringList();
		if (!cat[i])
			exit(1);
		stat=music->populateCategoryQStringList(cat[i], i);
		if (stat<0)
		{
			errBox(stat);
			goto cleanup;
		}
		pref->populateCategoryBoxPT(i, cat[i]);
	}

	pref->updatetitleInfo();

	if (music)
		pref->setVersion(dbconnectinfo->pg_version);
	else
		pref->setVersion(i18n("Not Connected"));

	connect(pref, SIGNAL(okClicked()), this, SLOT(readSettings()));
	connect(pref, SIGNAL(okClicked()), this, SLOT(initAutoConnect()));
	connect(pref, SIGNAL(okClicked()), this, SLOT(updateTitleView()));
	connect(pref, SIGNAL(cancelled(preferencesDialog *)), this, SLOT(deleteFromQMap(preferencesDialog *)));
	connect(pref, SIGNAL(sortCategorySignal(bool)), this, SLOT(sortCategorySlot(bool)));
	connect(pref, SIGNAL(destroyed()), this, SLOT(windowDec()));
	pref->show();

cleanup:
	if (music)
	{
		for (int i = 0; i < 6; i++)
			delete cat[i];
	}
}

/** Toggles the mainToolbar on and off. */
void
kmusicdb::slotToolbar(bool a)
{
	if (a)
		toolBar()->show();
	else
		toolBar()->hide();
}

/** Toggles the status bar on and off. */
void
kmusicdb::slotStatusbar()
{
	if (showstatusbar->isChecked())
		statusBar()->show();
	else
		statusBar()->hide();
}

/** Slot for toggling the name search toolbar.
 * @param bool TRUE enables, FALSE disables
 */
void
kmusicdb::toggleName(bool a)
{
	if (a)
		toolBar("nameSearch")->show();
	else
		toolBar("nameSearch")->hide();
}

/** slot for toggling the title search toolbar.
 * @param bool TRUE enables, FALSE disables
 */
void
kmusicdb::toggleTitle(bool a)
{
	if (a)
		toolBar("titleSearch")->show();
	else
		toolBar("titleSearch")->hide();
}
/**
 * Increments the window count by one when called. If window count is greater than 0, the new and open db buttons are disabled. This makes it impossible to create or open a database while other windows are still open.
 */
void kmusicdb::windowInc(){
	window_count++;
	if (window_count > 0)
		enableDbGUI(FALSE);
	cerr << "kmusicdb::windowInc: " << window_count << endl;
}
/**
 * Decreases window count by one. If count reaches 0, new and open database buttons are reenabled.
 */
void kmusicdb::windowDec(){
	window_count--;
	if (window_count==0)
		enableDbGUI(TRUE);
	cerr << "kmusicdb::windowDec: " << window_count << endl;
}



/* *************************************************************************** */
/* ***************************** INIT FUNCTIONS ****************************** */
/* *************************************************************************** */
/** Connects to the default db if that option is enabled. */
void
kmusicdb::initAutoConnect()
{
	if (c)
		return;
	if (music)
		return;
	if (def->auto_connect)
	{
		cerr << "kmusicdb::initAutoConnect: Connecting to DB" << endl;
		// check if db/server is available

		updateStatusBar(i18n("Auto Connecting to default db..."));
		cerr << "dbauto values: name: " << dbauto->db_name << " location: " << dbauto->db_location;
		cerr << " pw: " << dbauto->db_passwd << " port: " << dbauto->db_port;
		cerr << " user: " << dbauto->db_user;
		cerr << endl;

		open_db(dbauto);

		dbInfo *temp=dbconnectinfo;
		dbconnectinfo=dbauto;
		updateProgress(80);
		if (music)
		{
			checkVersion();
			updateArtistTitleView();
			emit signalDbConnect(TRUE);
			updateProgress(100);
		}
		dbconnectinfo=temp;
	}
}

/** Sets up musicArtistTitle, the center widget. */
void
kmusicdb::initCenterWidget()
{
	cerr << "kmusicdb::Initializing Center Widget" << endl;
	mid = new musicArtistTitle(this);
	if (!mid)
		exit(1);
	mid->sortList(def->default_column_sort,def->default_sort_order);
	setCentralWidget(mid);
	connect(mid->combobox(), SIGNAL(activated(const QString &)), this, SLOT(updateTitleView()));
	connect(mid->combobox(), SIGNAL(rightClicked()), this, SLOT(slotEditArtist()));
	connect(mid->combobox(),SIGNAL(imFocused(bool)),this,SLOT(enableDeleteArtist(bool)));

	connect(mid->listview(),SIGNAL(imFocused(bool)),this,SLOT(enableEditDeleteTitle(bool)));
	connect(mid->listview(), SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(slotEditTitle()));
	connect(mid->listview(), SIGNAL(returnPressed(QListViewItem *)), this, SLOT(slotEditTitle()));

	connect(mid->listview(),SIGNAL(dragged(QListView *)),this,SLOT(titleDragged(QListView *)));
	connect(mid->listview(),SIGNAL(dropped(CDDBtitleInfo)),this,SLOT(titleDropped(CDDBtitleInfo)));

}

/** Initializes all the default values for the values. readSettings then loads the values from the config file. */
void
kmusicdb::initDefault()
{
	cerr << "kmusicdb::Initializing Defaults" << endl;
	// init the defaults struct
	def = new Defaults;
	t_op = new enabledTitleColumns();
	tr_op= new enabledTrackColumns();
	c_op= new enabledCategoryColumns();
	dbconnectinfo = new dbInfo();
	dbauto = new dbInfo();
	si = new CDDBserverInfo();
	title = new titleInfo();
	artist = new artistInfo();
	twm = new TitleDialogManager();
	pwm = new PreferencesDialogManager();
	color = new colorInfo();
	reportOptions = new reportInfo();
	if (!def || !t_op || !tr_op || !c_op || !dbconnectinfo || !dbauto || !si || !title || !artist || !twm || !pwm || !color || !reportOptions)
		exit(1);

	// initialize some default values
	def->default_sort_order=FALSE;
	def->default_column_sort=FALSE;

	t_op->notes=FALSE;
	t_op->name=TRUE;
	t_op->cdid=TRUE;

	tr_op->num=TRUE;
	tr_op->name=TRUE;
	tr_op->length=TRUE;
	tr_op->lyric=FALSE;
	tr_op->trackid=TRUE;
	tr_op->side=TRUE;
	tr_op->notes=TRUE;

	c_op->name=TRUE;
	c_op->xid=TRUE;

	gethostname(si->host,255);

	// tell our pointers and ints to goto 0.
	dialog_result=0;
	stat=0;
	c=0;
	init=0;
	music=0;
	artistdialog=0;
	titledialog=0;
	opener=0;
	window_count=0;
	cdp=0;
	cdnet=0;
	for (int i=0; i < 7; i++)
		catdialog[i]=0;

}

/** Initializes all the menu entries. */
void
kmusicdb::initMenu()
{
	cerr << "kmusicdb::Building Menues" << endl;
	// build menus
	// file menu
	newdb = new KAction(i18n("&New Database"), "dbnew.png", 0, this, SLOT(newDatabase()), actionCollection(), "newdb");
	opendb = new KAction(i18n("&Open Database"), "dbopen.png", CTRL + Key_O, this, SLOT(openDatabase()), actionCollection(), "opendb");
	closedb = new KAction(i18n("&Close Database"), "dbclose.png", 0, this, SLOT(closeDatabase()), actionCollection(), "closedb");
	KStdAction::quit(kapp, SLOT(closeAllWindows()), actionCollection());
	restoredb = new KAction(i18n("&Restore"), "dbrestore.png", 0, this, SLOT(slotRestore()), actionCollection(), "restore");
	backupdb = new KAction(i18n("&Backup"), "dbbackup.png", 0, this, SLOT(slotBackup()), actionCollection(), "backup");
	opti = new KAction(i18n("O&ptimize"), "dbopti.png", 0, this, SLOT(slotOpti()), actionCollection(), "opti");

	// edit menu
	newtitle = new KAction(i18n("New &Title"), "newtitle.png", CTRL + Key_N, this, SLOT(slotNewTitle()), actionCollection(), "newtitle");
	etitle = new KAction(i18n("&Edit Title"), "edittitle.png", CTRL + Key_E, this, SLOT(slotEditTitle()), actionCollection(), "title");
	deltitle = new KAction(i18n("&Delete Title"), "deltitle.png", CTRL + Key_D, this, SLOT(slotDelTitle()), actionCollection(), "deltitle");
	listtitle = new KAction(i18n("Re&list Titles"), "", CTRL + Key_R, this, SLOT(updateTitleView()), actionCollection(), "listtitle");

	newartist = new KAction(i18n("New &Artist"), "newartist.png", CTRL + SHIFT + Key_N, this, SLOT(slotNewArtist()), actionCollection(),
		 "newartist");
	eartist = new KAction(i18n("Edit A&rtist"), "editartist.png", CTRL + SHIFT + Key_E, this, SLOT(slotEditArtist()),
		 actionCollection(), "artist");
	delartist = new KAction(i18n("Delete Art&ist"), "delartist.png", CTRL + SHIFT + Key_D, this, SLOT(slotDelArtist()),
		 actionCollection(), "delartist");
	listartist = new KAction(i18n("Relist Artists"), "", CTRL + SHIFT + Key_R, this, SLOT(updateArtistTitleView()), actionCollection(), "listartist");

	eboxset = new KAction(i18n("&Boxset"), CTRL + Key_B, this, SLOT(slotEditBoxset()), actionCollection(), "boxset");
	eformat = new KAction(i18n("&Format"), CTRL + Key_F, this, SLOT(slotEditFormat()), actionCollection(), "format");
	egenre = new KAction(i18n("&Genre"), CTRL + Key_G, this, SLOT(slotEditGenre()), actionCollection(), "genre");
	elabel = new KAction(i18n("&Label"), CTRL + Key_L, this, SLOT(slotEditLabel()), actionCollection(), "label");
	emethod = new KAction(i18n("&Method"), CTRL + Key_M, this, SLOT(slotEditMethod()), actionCollection(), "method");
	etype = new KAction(i18n("&Type"), CTRL + Key_T, this, SLOT(slotEditType()), actionCollection(), "type");

	// cddb menu
	addcddb = new KAction(i18n("New From &CDDB"), "cddb.png", CTRL + Key_C, this, SLOT(slotCDDBAdd()), actionCollection(), "addcddb");
	updatecddb = new KAction(i18n("&Update From CDDB"), "cddb.png", 0, this, SLOT(slotCDDBUpdate()), actionCollection(), "updatecddb");
	appendcddb = new KAction(i18n("&Append From CDDB"), "cddb.png", 0, this, SLOT(slotCDDBAppend()), actionCollection(), "appendcddb");
	searchcddb = new KAction(i18n("&Search From CDDB"), "cddb.png", CTRL + Key_S, this, SLOT(slotCDDBSearch()), actionCollection(), "searchcddb");
	submitcddb = new KAction(i18n("Su&bmit To CDDB"), "cddbsubmit.png", 0, this, SLOT(slotCDDBSubmit()), actionCollection(), "submitcddb");
	// reports menu
	tbya = new KAction(i18n("&Titles By Artist"), QIconSet(BarIcon("history")), 0, this, SLOT(slotTbya()), actionCollection(), "tbya");
	trbyt = new KAction(i18n("T&racks By Title"), QIconSet(BarIcon("history")), 0, this, SLOT(slotTrbyt()), actionCollection(), "trbyt");
	collectionrep = new KAction(i18n("&Collection"), QIconSet(BarIcon("history")), 0, this, SLOT(slotCollectionRep()), actionCollection(),
		 "collectionrep");
	artistrep = new KAction(i18n("&Artists"), QIconSet(BarIcon("history")), 0, this, SLOT(slotArtistRep()), actionCollection(), "artistrep");
	daterangerep = new KAction(i18n("&Date Range"), QIconSet(BarIcon("history")), 0, this, SLOT(slotDateRangeRep()), actionCollection(),
		 "daterangerep");
	bterep = new KAction(i18n("&Bands To Explore"), QIconSet(BarIcon("history")), 0, this, SLOT(slotBteRep()), actionCollection(),
		 "bterep");
	topcdrep = new KAction(i18n("T&op Title"), QIconSet(BarIcon("history")), 0, this, SLOT(slotTopCDRep()), actionCollection(), "topcdrep");
	statistics = new KAction(i18n("&Statistics"), QIconSet(BarIcon("history")), 0, this, SLOT(slotStatistics()), actionCollection(),
		 "statistics");
	customrep = new KAction(i18n("C&ustom"), QIconSet(BarIcon("history")), 0, this, SLOT(slotCustomRep()), actionCollection(), "customrep");

	// bte menu
	btebrowse = new KAction(i18n("E&xplore List"), QIconSet(BarIcon("contents2")), CTRL + Key_X, this, SLOT(slotBteBrowse()),
		 actionCollection(), "btebrowse");

	// settings menu
	showtoolbar=new KToggleAction(i18n("Show ToolBar"), 0, actionCollection(), "tbar");
	showstatusbar = KStdAction::showStatusbar(this, SLOT(slotStatusbar()), actionCollection());
	KStdAction::preferences(this, SLOT(slotPreferences()), actionCollection());
	namesearch = new KToggleAction(i18n("Show Name Search Toolbar"), 0, actionCollection(), "nstoolbar");
	titlesearch = new KToggleAction(i18n("Show Category Search Toolbar"), 0, actionCollection(), "tstoolbar");
	// connect signals
	connect(showtoolbar, SIGNAL(toggled(bool)), this, SLOT(slotToolbar(bool)));
	connect(namesearch, SIGNAL(toggled(bool)), this, SLOT(toggleName(bool)));
	connect(titlesearch, SIGNAL(toggled(bool)), this, SLOT(toggleTitle(bool)));

}

/** Initializes all the status bar code. */
void
kmusicdb::initStatusBar()
{
	cerr << "kmusicdb::Building Status Bar" << endl;
	// ID Numbers:
	// 0 is Ready.
	// 1 is artists.
	// 2 is titles.
	// 3 is time.
	sbp = new QProgressBar(statusBar());
	if (!sbp)
		exit(1);
	sbp->setTotalSteps(100);
	sbp->setProgress(0);
	sbp->setCenterIndicator(true);
	statusBar()->addWidget(sbp, 1, FALSE);
	statusBar()->insertItem(i18n("Ready..."), 0, 150, FALSE);
	statusBar()->insertItem(i18n(" Total Artist: 0 "), 1, TRUE);
	statusBar()->insertItem(i18n(" Total Titles: 0 "), 2, TRUE);
	statusBar()->insertItem(i18n(" Total Time: 00:00:00 "), 3, TRUE);

	sbp->height();
}

/** Initializes all 3 toolbars. */
void
kmusicdb::initToolBars()
{

	cerr << "kmusicdb::Building Toolbars" << endl;
	// setup some of the variables we need.
	QStringList *namesearchlist = new QStringList();
	if (!namesearchlist)
		exit(1);
	namesearchlist->append(i18n("Artist"));
	namesearchlist->append(i18n("Title"));
	namesearchlist->append(i18n("Track"));
	namesearchlist->append(i18n("Lyric"));

	QStringList *titlesearchlist = new QStringList();
	if (!titlesearchlist)
		exit(1);
	titlesearchlist->append(i18n("Format"));
	titlesearchlist->append(i18n("Genre"));
	titlesearchlist->append(i18n("Label"));
	titlesearchlist->append(i18n("Method"));
	titlesearchlist->append(i18n("Type"));

	// title search
	// IDs: label, combo, combo, button
	QStringList *t = new QStringList();
	if (!t)
		exit(1);
	QLabel *label = new QLabel(i18n("Search From Category "), toolBar("titleSearch"));
	if (!label)
		exit(1);
	toolBar("titleSearch")->insertWidget(TITLE_LABEL1, label->sizeHint().width(), label);
	toolBar("titleSearch")->insertCombo(*titlesearchlist, TITLE_CAT_COMBO, 0,SIGNAL(activated(int)), this,SLOT(update_toolbar_combobox(int)), 1,i18n("Select category to search in."),-1);
	toolBar("titleSearch")->insertCombo(*t, TITLE_LIST_COMBO, 0,SIGNAL(activated(int)), this, SLOT(dummy_slot(int)), 1,i18n("Select category to search in."),-1);
	toolBar("titleSearch")->insertButton("filefind", TITLE_BUTTON,SIGNAL(clicked()), this, SLOT(slotCategorySearch()), 1,i18n("Search"));
	delete t;

	// name search
	// IDs: label, lined, label, combo, button
	label = new QLabel(i18n("Search For "), toolBar("nameSearch"));
	if (!label)
		exit(1);
	toolBar("nameSearch")->insertWidget(NAME_LABEL1,label->sizeHint().width(), label);
	toolBar("nameSearch")->insertLined("", NAME_LINED,SIGNAL(returnPressed()), this, SLOT(slotNameSearch()), TRUE, i18n("Insert the name here."));
	label = new QLabel(i18n(" In "), toolBar("nameSearch"));
	if (!label)
		exit(1);
	toolBar("nameSearch")->insertWidget(NAME_LABEL2,label->sizeHint().width(), label);
	toolBar("nameSearch")->insertCombo(*namesearchlist, NAME_COMBO, 0, SIGNAL(activated(int)), this, SLOT(dummy_slot(int)), TRUE, i18n("Select group to search in."),-1);
	toolBar("nameSearch")->insertButton("filefind", NAME_BUTTON,SIGNAL(clicked()), this, SLOT(slotNameSearch()), 1,i18n("Search"));
	category = toolBar("titleSearch")->getCombo(TITLE_CAT_COMBO);
	catresult = toolBar("titleSearch")->getCombo(TITLE_LIST_COMBO);
	nametypes = toolBar("nameSearch")->getCombo(NAME_COMBO);
	toolbarname = toolBar("nameSearch")->getLined(NAME_LINED);

	// add buttons to mainToolBar
	toolBar()->insertButton("dbnew.png", 0, SIGNAL(clicked()), this, SLOT(newDatabase()), true, i18n("New"));
	toolBar()->insertButton("dbopen.png", 1, SIGNAL(clicked()), this, SLOT(openDatabase()), true, i18n("Open"));
	toolBar()->insertSeparator(2);
	toolBar()->insertButton("newartist.png", 3, SIGNAL(clicked()), this, SLOT(slotNewArtist()), true, i18n("New Artist"));
	toolBar()->insertButton("editartist.png", 4, SIGNAL(clicked()), this, SLOT(slotEditArtist()), true,
					i18n("Edit Artist"));
	toolBar()->insertButton("delartist.png", 5, SIGNAL(clicked()), this, SLOT(slotDelArtist()), true, i18n("Delete Artist"));
	toolBar()->insertSeparator(6);
	toolBar()->insertButton("newtitle.png", 7, SIGNAL(clicked()), this, SLOT(slotNewTitle()), true, i18n("New Title"));
	toolBar()->insertButton("edittitle.png", 8, SIGNAL(clicked()), this, SLOT(slotEditTitle()), true, i18n("Edit Title"));
	toolBar()->insertButton("deltitle.png", 9, SIGNAL(clicked()), this, SLOT(slotDelTitle()), true, i18n("Delete Title"));
	toolBar()->insertSeparator(10);
	toolBar()->insertButton("cddb.png", 11, SIGNAL(clicked()), this, SLOT(slotCDDBAdd()), true, i18n("Add Title From CDDB"));

	delete namesearchlist;
	delete titlesearchlist;
}

/**
 * This function processes the command line args for kmusicdb. If 0 is passed the function just returns.
 *
 * @param KCmdLineArgs Command Line Arguments
 */
void
kmusicdb::processArgs(KCmdLineArgs *a)
{
	KURL kmurl;
	QString localfilename;
	KTempFile *tfile;
	cerr << "kmusicdb::Arguments were passed to kmusicdb. Processing." << endl;
	if (kapp->isRestored())
		return;
	// check for db
	if (a->isSet("d"))
		dbconnectinfo->db_name = a->getOption("d");
	// check for host
	if (a->isSet("h"))
		dbconnectinfo->db_location = a->getOption("h");
	// check for port
	if (a->isSet("p"))
		dbconnectinfo->db_port = a->getOption("p");
	// check for user
	if (a->isSet("u"))
		dbconnectinfo->db_user = a->getOption("u");
	// check for passwd
	if (a->isSet("s"))
		dbconnectinfo->db_passwd = a->getOption("s");

	bool save_auto_connect=def->auto_connect;
	// check for connect
	if (a->isSet("c"))
	{
		def->auto_connect = true;
		dbauto = dbconnectinfo;
	}

	if (a->isSet("b") || a->isSet("r") || a->isSet("t"))
	{
		hide();
		// the rest of the options are similar. They share the following code in common
		if (a->isSet("f"))
			kmurl.setPath(a->getOption("f"));
		else
		{
			cerr << "kmusicdb::Filename must be set for this option." << endl;
			exit(1);
		}
		cerr << "kmusicdb::AutoConnecting to Database" << endl;
		if (open_db(dbconnectinfo) < 0)
		{
			cerr << "kmusicdb::Could not open db. Exiting." << endl;
			exit(1);
		}
		if (kmurl.path() == NULL)
			exit(1);
		if (kmurl.isLocalFile())
			localfilename = kmurl.path();
	}

	// check for backup(automatically exists program)
	if (a->isSet("b"))
	{
		cerr << "kmusicdb::backing up" << endl;
		// check if db/server is available
		if (kmurl.isLocalFile())
		{
			stat = music->backup(localfilename);
			if (stat < 0)
			{
				cerr << "kmusicdb::Error backing up." << endl;
				exit(1);
			}
		}
		else
		{
			tfile = new KTempFile("upload", "tmp");
			if (!tfile)
				exit(1);
			tfile->setAutoDelete(TRUE);
			if (tfile->status() == 0)
			{
				cerr << "kmusicdb::Could not open file" << endl;
				goto temp_cleanup;
			}
			stat = music->backup(tfile->name());
			if (stat < 0)
			{
				cerr << "kmusicdb::Error backing up." << endl;
				goto temp_cleanup;
			}

			KIO::NetAccess::upload(tfile->name(), kmurl, this);
			tfile->close();
		}
		exit(0);
	}

	// check for restore(automatically exists program)
	if (a->isSet("r"))
	{
		cerr << "kmusicdb::restoring" << endl;
		if (kmurl.isLocalFile())
		{
			stat = music->restore(localfilename);
			if (stat < 0)
			{
				cerr << "kmusicdb::Error restoring backup." << endl;
				exit(1);
			}
		}
		else
		{
			QString tmpFile;
			if (KIO::NetAccess::download(kmurl, tmpFile, this))
			{
				stat = music->restore(tmpFile);
				if (stat < 0)
				{
				cerr << "kmusicdb::Error restoring backup." << endl;
				exit(1);
				}
				KIO::NetAccess::removeTempFile(tmpFile);
			}
		}
		exit(0);
	}

	// check for reports(automatically exists program)
	if (a->isSet("t"))
	{
		cerr << "kmusicdb::report generating" << endl;
		QString filestring;
		QString rep = a->getOption("t");
		dbReport *dbrep=new dbReport(music);
		if (!dbrep)
			exit(1);
		reportOptions->charset=KGlobal::locale()->encoding();
		if (rep == "artist")
			stat = dbrep->artistReport(filestring, reportOptions, color);
		else if (rep == "collection")
			stat = dbrep->collectionReport(filestring, reportOptions, color);
		else if (rep == "daterange")
			stat = dbrep->dateRangeReport(filestring, reportOptions, color);
		else if (rep == "statistics")
			stat = dbrep->statisticsReport(filestring, reportOptions, color);
		else if (rep == "bte")
			stat = dbrep->bteReport(filestring, reportOptions, color);
		else if (rep == "topcd")
			stat = dbrep->topTitlesReport(filestring,reportOptions,color);
		else
		{
			cerr <<
			"kmusicdb::Unknown report type. Run kmusicdb --help for available options."
			<< endl;
			exit(1);
		}
		delete dbrep;
		if (stat < 0)
		{
			cerr << "kmusicdb::Error generating report." << endl;
			exit(1);
		}

		if (kmurl.isLocalFile())
		{
			QFile f(localfilename);
			if (f.open(IO_WriteOnly))
			{
				QTextStream t(&f);
				t << filestring;
				f.close();
			}
			else
			{
				cerr << "kmusicdb::Could not write report to disk." << endl;
				exit(1);
			}
		}
		else
		{
			tfile = new KTempFile("upload", "tmp");
			if (!tfile)
				exit(1);
			tfile->setAutoDelete(TRUE);
			if (tfile->status() == 0)
			{
				cerr << "kmusicdb::Could not open file" << endl;
				goto temp_cleanup;
			}
			*tfile->textStream() << filestring;

			KIO::NetAccess::upload(tfile->name(), kmurl, this);
			tfile->close();
		}
		exit(0);
	}
	def->auto_connect=save_auto_connect;
	return;

temp_cleanup:
	tfile->close();
	exit(1);
}



/* *************************************************************************** */
/* ****************************** ARTIST FUNCTIONS *************************** */
/* *************************************************************************** */
/** Does the dirty work of adding the artist to the db.
 * @param artistInfo Artist Information
 */
void
kmusicdb::addArtist(artistInfo t)
{
	updateGUI(0, i18n("Adding Artist..."));
	// check for text formatting
	if (t.name.isEmpty())
		return;

	if (def->auto_format)
		t.name = formatText(t.name);
	else if (def->auto_capitalize)
		t.name = capitalizeWords(t.name);

	// let's see if the artist already exists
	// setartisttext, then check for match
	// two birds with one stone, nice feature plus we don't corrupt the db.
	cerr << "kmusicdb:::addArtist:setArtistText: " << endl;
	mid->setArtistText(t.name);
	if (mid->getArtistText() == t.name)
		cerr << "name match" << endl;
	else
	{
		stat = music->addArtist(&t);
		if (stat < 0)
			errBox(stat);
		updateProgress(40);
		updateArtistTitleView();
		mid->setArtistText(t.name);
	}
	updateProgress(60);
	updateTitleView();
	updateProgress(80);

	// if last values are used, then we need to update the struct now
	if (def->artistdata == LAST_VALUE)
	{
		artist->first = t.first;
		artist->follow = t.follow;
		artist->last = t.last;
		artist->name = t.name;
		artist->notes = t.notes;
		artist->homepage = t.homepage;
	}

	//now we must doublecheck the widgets and make sure everything is enabled or unenabled properly.
	enableArtist(TRUE);
	enableTitle(TRUE);
	updateGUI(100, i18n("Ready..."));
}

/** Deletes artist from the db. Dialog.*/
void
kmusicdb::slotDelArtist()
{
	updateGUI(0, i18n("Deleting Artist..."));

	QString temp = mid->getArtistText();
	if (temp.isEmpty())
		return;

	windowInc();
	int z = KMessageBox::questionYesNo(this, i18n("Do you really want to delete artist %1?").arg
						(temp), i18n("Delete Artist"));
	if (z == KMessageBox::Yes)
	{
		//delete the artist
		stat = music->deleteArtist(aid);
		if (stat < 0)
			errBox(stat);
		updateProgress(30);
		// must refresh the display
		updateArtistTitleView();
		windowDec();
		updateProgress(70);
		enableArtist(TRUE);
		enableTitle(TRUE);
		enableDeleteArtist(TRUE);
	}
	updateGUI(100, i18n("Ready..."));
}

/** Slot for editing the current artist. Dialog.*/
void
kmusicdb::slotEditArtist()
{
	QString t;
	if ( mid->getArtistText().isEmpty() )
		return;
	updateGUI(0, i18n("Getting Artist..."));

	artistInfo *art = new artistInfo();
	if (!art)
		exit(1);
	art->aid=aid;
	stat = music->getArtist(art);
	if (stat < 0)
		goto cleanup;
	updateProgress(30);
	stat = music->totalBy(t, aid, ARTIST, REPORT_ALL);
	if (stat < 0)
		goto cleanup;
	updateProgress(60);
	windowInc();
	artistdialog = new artistDialog(0, i18n("Edit Artist..."), i18n("&Update"), true);
	if (!artistdialog)
		exit(1);
	artistdialog->widget()->updateTitles(t);
	connect(artistdialog, SIGNAL(addArtist(artistInfo)), this, SLOT(updateArtist(artistInfo)));
	connect(artistdialog, SIGNAL(destroyed()), this, SLOT(windowDec()));
	artistdialog->widget()->updateInfo(art, false, false);
	delete art;
	updateGUI(100, i18n("Ready..."));
	artistdialog->show();
	return;
cleanup:
	delete art;
	errBox(stat);
	updateArtistTitleView();
}

/** Slot for adding a new artist to the current database. Dialog.*/
void
kmusicdb::slotNewArtist()
{
	windowInc();
	artistdialog = new artistDialog(0, i18n("New Artist..."), i18n("&Add"), def->artist_autofill);
	if (!artistdialog)
		exit(1);
	artistdialog->widget()->updateTitles("0");
	connect(artistdialog, SIGNAL(addArtist(artistInfo)), this, SLOT(addArtist(artistInfo)));
	connect(artistdialog, SIGNAL(destroyed()), this, SLOT(windowDec()));
	artist->follow=def->artist_follow;
	artistdialog->widget()->updateInfo(artist, def->artist_year, def->artist_autofill);
	updateGUI(100, i18n("Ready..."));
	artistdialog->show();
}

/**
 * Dirty work for updating an artist.
 * @param artistInfo Struct containing updated artist information
 */
void
kmusicdb::updateArtist(artistInfo t)
{
	updateGUI(0, i18n("Updating Artist..."));
	if (t.name.isEmpty())
		return;
	// check for text formatting
	if (def->auto_format)
		t.name = formatText(t.name);
	else if (def->auto_capitalize)
		t.name = capitalizeWords(t.name);

	stat = music->updateArtist(&t);
	if (stat < 0)
		errBox(stat);
	updateArtistView();
	updateProgress(30);
	if (stat < 0)
		return;
	updateProgress(60);
	mid->setArtistText(t.name);
	updateTitleView();
	updateGUI(100, i18n("Ready..."));
}



/* *************************************************************************** */
/* ******************************* TITLE FUNCTIONS *************************** */
/* *************************************************************************** */
/**
 * Dirty work for adding a title to the db.
 * @param titleInfo titleInfo Struct
 */
void
kmusicdb::addTitle(titleInfo t)
{
	updateGUI(0, i18n("Adding Title..."));
	if (t.name.isEmpty())
		return;
	// check for text formatting
	if (def->auto_format)
		t.name = formatText(t.name);
	else if (def->auto_capitalize)
		t.name = capitalizeWords(t.name);

	stat = music->addTitle(&t, aid);
	updateProgress(30);
	if (stat < 0)
		errBox(stat);
	updateTitleView();
	mid->listview()->ensureItemVisibleByCDID(t.cdid);
	if (stat < 0)
		return;
	updateProgress(60);

	// if last values are used, we need to update the struct now
	if (def->titledata == LAST_VALUE)
	{
		title->collected = t.collected;
		for (int i=0;i<6;i++)
			title->cat[i]=t.cat[i];
		title->name = t.name;
		title->nod = t.nod;
		title->notes = t.notes;
		title->notracks = t.notracks;
		title->yearend = t.yearend;
		title->yearstart = t.yearstart;
		title->length = "00:00:00";
		title->rating = t.rating;
		title->quality = t.quality;
	}
	updateGUI(100, i18n("Ready..."));
}

/**
 * Opens edit title dialog box.
 * @param QString CD ID
 */
void kmusicdb::openEditTitle(const QString &cdid)
{
	updateGUI(0, i18n("Edit Title..."));
	cerr << "kmusicdb::openEditTitle:CDID: " << cdid << endl;
	titleInfo *info = new titleInfo();
	if (!info)
		exit(1);
	info->cdid=cdid;
	stat = music->getTitle(info);
	if (stat < 0)
	{
		errBox(stat);
		updateTitleView();
		goto cleanup2;
	}

	QStringList *cat[6];

	windowInc();
	titledialog = new titleDialog(0, i18n("Edit Title: %1").arg(info->name), i18n("&Update"), i18n("View &Picture"), true, false,info->cdid,info->picpath);
	if (!titledialog)
		exit(1);
	connect(titledialog, SIGNAL(addTitle(titleInfo)), this, SLOT(updateTitle(titleInfo)));
	connect(titledialog, SIGNAL(cancelled(titleDialog *)), this, SLOT(deleteFromQMap(titleDialog *)));
	connect(titledialog, SIGNAL(destroyed()), this, SLOT(windowDec()));
	connect(titledialog, SIGNAL(relistTitles()),this,SLOT(updateTitleView()));
	twm->insert(titledialog, twm->count());

	for (int i = 0; i < 6; i++)
	{
		cat[i] = new QStringList();
		if (!cat[i])
			exit(1);
		stat=music->populateCategoryQStringList(cat[i], i);
		if (stat<0)
		{
			errBox(stat);
			delete titledialog;
			windowDec();
			goto cleanup;
		}
		titledialog->widget()->populateCategoryBox(i, cat[i]);
	}
	updateProgress(50);

	titledialog->widget()->updateInfo(info,false,false);
	titledialog->track()->updateCaption(info->name);

	connect(titledialog, SIGNAL(updateTracks(QListView*, QString)), this, SLOT(updateTrackInfo(QListView*, QString)));
	connect(titledialog->track(), SIGNAL(addTrack(QString, trackInfo, QListView *)),this, SLOT(addTrack(QString, trackInfo, QListView *)));
	connect(titledialog->track(), SIGNAL(updateTrack(QString, trackInfo, QListView *)), this,SLOT(updateTrack(QString, trackInfo, QListView *)));
	connect(titledialog->track(), SIGNAL(deleteTrack(QString, QString, QListView *)),this, SLOT(deleteTrack(QString, QString, QListView *)));
	connect(titledialog->track(), SIGNAL(signalTrackUp(QString, trackInfo, trackInfo, QListView *)),
		 this, SLOT(trackUp(QString, trackInfo, trackInfo, QListView *)));
	connect(titledialog->track(),SIGNAL(signalTrackDown(QString, trackInfo, trackInfo, QListView *)), this,
		 SLOT(trackDown(QString, trackInfo, trackInfo, QListView *)));
	connect(titledialog->track(),SIGNAL(updateLyric(QString,QString,bool *)),this,SLOT(updateLyric(QString,QString, bool *)));
	connect(titledialog->track(),SIGNAL(getLyric(QTextEdit *, QString, bool *)),this,SLOT(getLyric(QTextEdit *, QString, bool *)));
	connect(titledialog->track()->widget()->listview(),SIGNAL(dragged(QListView *)),this,SLOT(trackDragged(QListView *)));
	connect(titledialog->track(),SIGNAL(trackDropped(QString,trackInfo,QListView *)),this,SLOT(trackDropped(QString,trackInfo,QListView *)));
	connect(titledialog->track(),SIGNAL(relistTracks(QListView *, QString)),this,SLOT(updateTrackInfo(QListView *, QString)));

	updateProgress(100);
	titledialog->show();

	updateGUI(100, i18n("Ready..."));

cleanup:
	for (int i = 0; i < 6; i++)
		delete cat[i];
cleanup2:
	delete info;
}

/** Deletes title from db. Dialog.*/
void
kmusicdb::slotDelTitle()
{
	updateGUI(0, i18n("Deleting Title..."));
	QListViewItem *item = mid->listview()->selectedItem();
	if (item == 0)
	{
		cerr << "kmusicdb::Deleting Title: NO item selected" << endl;
		return;
	}
	if (item->childCount())
	{
		cerr << "kmusicdb::Deleting Title: Parent Item" << endl;
		errBox(BOXSET_NO_ACTION);
		return;
	}
	windowInc();
	int z = KMessageBox::questionYesNo(this, i18n("Do you really want to delete title %1?").arg(item->text(0)), i18n("Delete Title"));
	windowDec();
	if (z == KMessageBox::Yes)
	{
		//delete the artist
		stat = music->deleteTitle(item->text(mid->listview()->columns()));
		updateProgress(30);
		if (stat < 0)
			errBox(stat);
		updateTitleView();
	}
	// must update the GUI
	updateProgress(60);
	enableArtist(TRUE);
	enableTitle(TRUE);
	enableEditDeleteTitle(FALSE);

	updateGUI(100, i18n("Ready..."));
}

/** Slot for editing the selected title. Dialog.*/
void
kmusicdb::slotEditTitle()
{
	updateGUI(0, i18n("Editing Title..."));
	QListViewItem *item = mid->listview()->selectedItem();
	if (item == 0)
		return;
	if (item->childCount())
	{
		errBox(BOXSET_NO_ACTION);
		return;
	}
	openEditTitle(item->text(mid->listview()->columns()));
}

/**
 * Slot for adding a new title to the current database. Dialog.
 */
void
kmusicdb::slotNewTitle()
{
	updateGUI(0, i18n("New Title..."));
	if (mid->getArtistText().isEmpty())
	{
		cerr << "kmusicdb::New Title: Empty Artist" << endl;
		errBox(SEARCH_STRING_EMPTY);
		return;
	}

	windowInc();
	titledialog = new titleDialog(0, i18n("New Title..."), i18n("&Add"), i18n("Add &Picture"), def->title_autofill, true,"","");
	if (!titledialog)
		exit(1);
	twm->insert(titledialog, twm->count());
	connect(titledialog, SIGNAL(addTitle(titleInfo)), this, SLOT(addTitle(titleInfo)));
	connect(titledialog, SIGNAL(cancelled(titleDialog *)), this, SLOT(deleteFromQMap(titleDialog *)));
	connect(titledialog, SIGNAL(destroyed()), this, SLOT(windowDec()));

	QStringList *cat[6];
	for (int i = 0; i < 6; i++)
	{
		cat[i] = new QStringList();
		if (!cat[i])
			exit(1);
		stat=music->populateCategoryQStringList(cat[i], i);
		if (stat<0)
		{
			errBox(stat);
			goto cleanup;
		}
		titledialog->widget()->populateCategoryBox(i, cat[i]);
	}

	title->collected=def->title_collect;
	titledialog->widget()->updateInfo(title,def->title_year,def->title_autofill);

	updateGUI(100, i18n("Ready..."));
	titledialog->show();

cleanup:
	for (int i = 0; i < 6; i++)
		delete cat[i];
}

/**
 * Called when dragged(QWidget *) is emitted from mid->listview() to handle dragging.
 * @param QWidget Widget that emitted signal
 */
void kmusicdb::titleDragged(QListView *i)
{
	if (!music)
		 return;
	QListViewItem *item, *ti;
	QListView *v;
	trackInfo t;
	QDragObject *d;
	CDDBtitleInfo dd;
	updateGUI(0, i18n("Beginning Drag..."));
	item = i->selectedItem();
	if (item == 0)
		return;
	if (item->childCount())
	{
		errBox(BOXSET_NO_ACTION);
		return;
	}

	updateGUI(10, i18n("Retrieving Title..."));
	dd.title.cdid=item->text(mid->listview()->columns());
	stat = music->getTitle(&dd.title);
	if (stat < 0)
		goto cleanup;

	dd.device="memory";
	dd.disc_id="discid";
	dd.cddb_genre="cddb_genre";
	dd.totalLength="0";

	v=new QListView();
	if (!v)
		exit(1);
	updateGUI(50, i18n("Retrieving Track List..."));
	stat=music->populateTrackQListView(v, dd.title.cdid, tr_op);
	if (stat < 0)
	{
		delete v;
		goto cleanup;
	}
	ti=v->firstChild();
	if (ti)
	{
		do
		{
			t.side=ti->text(0);
			t.name=ti->text(2);
			t.length=ti->text(3);
			t.notes=ti->text(4);
			t.trackid=ti->text(5);
			dd.trackData.append(t);
			ti=ti->nextSibling();
		} while (ti);
	}
	else
		dd.trackData.clear();
	delete v;

	for (uint j=0;j<dd.trackData.count();j++)
	{
		stat=music->getLyric(&dd.trackData[j]);
		if (stat < 0)
			goto cleanup;
	}
	dd.trackSectorBegin.clear();
	dd.trackSectorLength.clear();

	updateGUI(100, i18n("Ready..."));
	d = new CDDBtitleInfoDrag( i, dd );
	d->dragCopy();
	return;

cleanup:
	errBox(stat);
	updateTitleView();
}
/**
 * Called when dropped(CDDBtitleInfo) is emitted from mid->listview() to handle dropping.
 * @param CDDBtitleInfo Struct holding from drop to enter into database
 */
void kmusicdb::titleDropped(CDDBtitleInfo i)
{
	if (!music)
		return;
	if (mid->getArtistText().isEmpty())
		return;
	QValueList<trackInfo>::Iterator it;
	updateGUI(0, i18n("Beginning Drop..."));
	// blindly add boxset and other categories.
	// very inefficient, not very reliable.
	for (int j=0;j<6;j++)
	{
		stat=music->addCategory(&(i.title.cat[j]));
		updateProgress((j+1)*10);
	}

	for (int j=0;j<6;j++)
	{
		updateLists(j);
		if (catdialog[j])
		{
			catdialog[j]->widget()->clearList();
			stat=music->populateCategoryQListView(catdialog[j]->widget()->list(), j, c_op);
			if (stat<0)
				goto cleanup;
		}
	}
	kapp->processEvents();
	// check for text formatting
	if (def->auto_format)
		i.title.name = formatText(i.title.name);
	else if (def->auto_capitalize)
		i.title.name = capitalizeWords(i.title.name);
	blanktitleInfo(i.title);

	updateGUI(60, i18n("Adding Title..."));
	stat = music->addTitle(&i.title, aid);
	if (stat < 0)
		goto cleanup;
	updateProgress(70);

	updateGUI(80, i18n("Adding Track Data..."));
	for (it=i.trackData.begin();it!=i.trackData.end();++it)
	{
		// check for text formatting
		if (def->auto_format)
			(*it).name = formatText((*it).name);
		else if (def->auto_capitalize)
			(*it).name = capitalizeWords((*it).name);
		stat = music->addTrack(&(*it), i.title.cdid);
		if (stat < 0)
			goto cleanup;
		stat=music->updateLyric(&(*it));
		if (stat < 0)
			goto cleanup;
	}
	updateProgress(90);
	updateTitleView();
	if (def->cddb_popup)
		openEditTitle(i.title.cdid);
	updateGUI(100, i18n("Ready..."));
	return;

cleanup:
	errBox(stat);
	updateTitleView();
}

/**
 * Dirty work for updating title.
 * @param titleInfo Struct containing updated title information
 */
void
kmusicdb::updateTitle(titleInfo t)
{
	updateGUI(0, i18n("Updating Title..."));
	if (t.name.isEmpty())
		return;
	// check for text formatting
	if (def->auto_format)
		t.name = formatText(t.name);
	else if (def->auto_capitalize)
		t.name = capitalizeWords(t.name);

	stat = music->updateTitle(&t);
	if (stat < 0)
		errBox(stat);
	updateProgress(70);
	updateTitleView();
	updateGUI(100, i18n("Ready..."));
}



/* *************************************************************************** */
/* ******************************* NEW FUNCTIONS ***************************** */
/* *************************************************************************** */
void kmusicdb::updateArtistTitleView() {
	updateArtistView();
	updateTitleView();
}
#include "kmusicdb.moc"
