/***************************************************************************
                          kmusicdb.h  -  description
                             -------------------
    begin                : Thu Mar 22 14:01:50 /etc/localtime 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMUSICDB_H
#define KMUSICDB_H

#include "include.h"
#include "dbconnectdialog.h"
#include "preferencesdialog.h"
#include "musicartisttitle.h"
#include "artistdialog.h"
#include "titledialog.h"
#include "kmusicdbvalidator.h"
#include "categorydialog.h"
#include "reportdialog.h"
#include "htmlreport.h"
#include "cddb.h"
#include "cddbsearchdialog.h"
#include "cddbsearchresultsdialog.h"

#include <dbmusicvalidator.h>
#include <dbinit.h>
#include <dbmusic.h>

/**
 *KmusicdB is a music information manager using a KDE3 frontend and a postgreSQL 7.2.x backend.
 *
 *On the surface, its major goals are speed, efficiency and reliability with an easy to use interface.
 *Secondly, it should be a rich, configurable program.
 *
 *Underneath the surface, design goals are to provide a robust reliable network transparency to the music database.
 *This should enable local and remote connections to the db.
 *Reusability of code through big widgets inside small dialogs.
 *Robust error handling.
 *Security conscious.
 *
 *See http://kmusicdb.sourceforge.net/ for more details.
 *
 *kmusicdb is the base class of the project, and inherits KParts::KMainWindow.
 * @short Main Class.. It starts here.
 * @author Will DeRousse
 */

class kmusicdb : public KParts::MainWindow
{
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param name Name Of Program
		 * @param KCmdLineArgs Command line arguments passed to KmusicdB
		 */
		kmusicdb(const char *name="kmusicdb", KCmdLineArgs *args=0);
		/**
		 * Destructor
		 */
		~kmusicdb();
	protected slots:
		/**
		 *Does the dirty work of adding the artist to the db.
		 * @param artistInfo Artist Information
		 */
		void addArtist(artistInfo);
		/** Add category slot.
		 * @param categoryInfo Struct containing category information
		 */
		void addCategory(categoryInfo);
		/**
		 * Dirty work for adding a title to the db.
		 * @param titleInfo titleInfo Struct
		 */
		void addTitle(titleInfo);
		/** Add track slot.
		 * @param QString CD ID
		 * @param trackInfo Track Information
		 * @param QListView Pointer to QListView for updating
		 */
		void addTrack(QString,trackInfo,QListView *);
		/**
		 * Slot wrapper for cdp errors.
		 * @param int Error Value
		 */
		void cdpError(int);
		/**
		 * Slot wrapper for cddb cleanup.
		 */
		void cddbCleanup();
		/**
		 * Slot wrapper for cddb cleanup.
		 */
		void cdpCleanup();
		/**
		 * This slot is for closing the current database. It pops up the dialog box.
		 */
		void closeDatabase();
		/**
		 * Re-implementation of this function. Using this method to save settings before the window is closed.
		 * @param QCloseEvent Event
		 */
		void closeEvent(QCloseEvent *);
		/** Delete category slot.
		 * @param categoryInfo Struct containing category information
		 */
		void deleteCategory(categoryInfo);
		/** Delete track slot.
		 * @param QString Track ID
		 * @param QString CD ID
		 * @param QListView Pointer to QListView for updating
		 */
		void deleteTrack(QString, QString, QListView *);
		/**
		 * This is the slot for File->New. Opens the new db popup.
		 */
		void newDatabase();
		/**
		 * This is the slot for File->Open. Opens an existing database.
		 */
		void openDatabase();
		/**
		 * Session Management reimplemented.
		 * @param KConfig KmusicdB Config
		 */
		void readProperties(KConfig *);
		/**
		 * Use this function to read the settings from the configuration file and set the variables in memory.
		 */
		void readSettings();
		/**
		 * Resizing to keep columns lined up.
		 * @param QResizeEvent Resize Event
		 */
		void resizeEvent(QResizeEvent *);
		/**
		 * Session Management reimplemented.
		 * @param KConfig KmusucdB Config
		 */
		void saveProperties(KConfig *);
		/**
		 * Artist report.
		 */
		void slotArtistRep();
		/**
		 * This slot is for backing up the current database to a dump file. File Dialog.
		 */
		void slotBackup();
		/**
		 * This slot will open the popup for browsing, adding, editing and removing bte names.
		 */
		void slotBteBrowse();
		/**
		 * Bands To Explore report.
		 */
		void slotBteRep();
		/**
		 * Opens dialog for generating report.
		 */
		void slotCategorySearch();
		/**
		 * Add new CD from CDDB.
		 */
		void slotCDDBAdd();
		/**
		 * Part 2 of the CDDB add process.
		 * @param CDDBtitleInfo Struct Containing CDDB Info
		 */
		void slotCDDBAdd2(CDDBtitleInfo);
		/**
		 * Finish Adding CD.
		 * @param CDDBtitleInfo Struct Containing CDDB Info
		 */
		void slotCDDBAddFinish(CDDBtitleInfo);
		/**
		 * Append to CD CDDB slot.
		 */
		void slotCDDBAppend();
		/**
		 * Part 2 of CDDB append process.
		 * @param CDDBtitleInfo Struct Containing CDDB Info
		 */
		void slotCDDBAppend2(CDDBtitleInfo);
		/**
		 * Finish Appending Data.
		 * @param CDDBtitleInfo Struct Containing CDDB Info
		 */
		void slotCDDBAppendFinish(CDDBtitleInfo);
		/**
		 * Search for title in CDDB.
		 */
		void slotCDDBSearch();
		/**
		 * Part 2 of CDDB search process.
		 * @param QString Search Text
		 * @param bool Artist Enabled Or Disabled
		 * @param bool Title Enabled Or Disabled
		 */
		void slotCDDBSearch2(QString,bool,bool);
		/**
		 * Part 3 of CDDB search process.
		 * @param QStringList Artist
		 * @param QStringList Title
		 * @param QStringList CDDB Category
		 * @param QStringList CDDB Disc ID
		 */
		void slotCDDBSearch3(QStringList, QStringList, QStringList, QStringList);
		/**
		 * Finish CDDB search process.
		 * @param QString CDDB Category
		 * @param QString CDDB Disc ID
		 */
		void slotCDDBSearch4(QString, QString);
		/**
		 * Submits a CD to CDDB.
		 */
		void slotCDDBSubmit();
		/**
		 * Part 2 of the CDDB submit process.
		 * @param CDDBtitleInfo Struct Containing CDDB Info
		 */
		void slotCDDBSubmit2(CDDBtitleInfo);
		/**
		 * Called when the submit to CDDB is finished.
		 * @param CDDBtitleInfo Struct Containing CDDB Info
		 */
		void slotCDDBSubmitFinish(CDDBtitleInfo);
		/**
		 * Update (replace tracks) CD from CDDB.
		 */
		void slotCDDBUpdate();
		/**
		 * Part 2 of CDDB update process.
		 * @param CDDBtitleInfo Struct Containing CDDB Info
		 */
		void slotCDDBUpdate2(CDDBtitleInfo);
		/**
		 * Finish CDDB update process.
		 * @param CDDBtitleInfo Struct Containing CDDB Info
		 */
		void slotCDDBUpdateFinish(CDDBtitleInfo);
		/**
		 * Collection report.
		 */
		void slotCollectionRep();
		/**
		 * Custom Report.
		 */
		void slotCustomRep();
		/**
		 * Date Range report.
		 */
		void slotDateRangeRep();
		/**
		 * Deletes artist from the db. Dialog.
		 */
		void slotDelArtist();
		/**
		 * Deletes title from db. Dialog.
		 */
		void slotDelTitle();
		/**
		 * Slot for editing the current artist. Dialog.
		 */
		void slotEditArtist();
		/**
		 * Slot for editing boxsets.
		 */
		void slotEditBoxset();
		/**
		 * Slot for editing the different formats. Dialog.
		 */
		void slotEditFormat();
		/**
		 * Slot for editing the different genres. Dialog.
		 */
		void slotEditGenre();
		/**
		 * Slot for editing the methods (music ones anyways). Dialog.
		 */
		void slotEditMethod();
		/**
		 * Slot for editing the different labels. Dialog.
		 */
		void slotEditLabel();
		/**
		 * Slot for editing the selected title. Dialog.
		 */
		void slotEditTitle();
		/**
		 * Slot for editing the different types. Dialog.
		 */
		void slotEditType();
		/**
		 * Opens dialog for generating report based on info from name search toolbar.
		 */
		void slotNameSearch();
		/**
		 * Slot for adding a new artist to the current database. Dialog.
		 */
		void slotNewArtist();
		/**
		 * Slot for adding a new title to the current database. Dialog.
		 */
		void slotNewTitle();
		/**
		 * This will perform a vacuum and an analyze on the database.
		 */
		void slotOpti();
		/**
		 * Settings dialog for the program.
		 */
		void slotPreferences();
		/**
		 * This slot is for restoring a database from a dump file. File Dialog.
		 */
		void slotRestore();
		/**
		 * Statistics report.
		 */
		void slotStatistics();
		/**
		 * Toggles the status bar on and off.
		 */
		void slotStatusbar();
		/**
		 * Toggles the mainToolbar on and off.
		 */
		void slotToolbar(bool);
		/**
		 * Top titles Report.
		 */
		void slotTopCDRep();
		/**
		 * Titles By Artist Report.
		 */
		void slotTbya();
		/**
		 * Tracks By Title Report.
		 */
		void slotTrbyt();
		/**
		 * Called when dragged(QWidget *) is emitted from mid->listview() to handle dragging.
		 * @param QWidget Widget that emitted the signal
		 */
		void titleDragged(QListView *);
		/**
		 * Called when dropped(CDDBtitleInfo) is emitted from mid->listview() to handle dropping.
		 * @param CDDBtitleInfo Struct holding from drop to enter into database
		 */
		void titleDropped(CDDBtitleInfo);
		/** Move track down slot.
		 * @param QString CDID
		 * @param trackInfo Top Track
		 * @param trackInfo Bottom Track
		 * @param QListView Pointer to QListView for updating
		 */
		void trackDown(QString, trackInfo, trackInfo, QListView *);
		/**
		 * Called when a track is dragged from a track window.
		 * @param QListView Pointer To Track QListView
		 */
		void trackDragged(QListView *);
		/**
		 * Called when a track is dropped onto a track window.
		 * @param QString Track Name
		 * @param trackInfo Track Information
		 * @param QListView Pointer to Track QListView
		 */
		void trackDropped(QString, trackInfo, QListView *);
		/** Move track up slot.
		 * @param QString CDID
		 * @param trackInfo Top Track
		 * @param trackInfo Bottom Track
		 * @param QListView Pointer to QListView for updating
		 */
		void trackUp(QString, trackInfo, trackInfo, QListView *);
		/** Transfer Artist slot.
		 * @param categoryInfo Struct containing info to xfr
		 */
		void transferArtist(categoryInfo);
		/**
		 * Dirty work for updating an artist.
		 * @param artistInfo Struct containing updated artist information
		 */
		void updateArtist(artistInfo);
		/** Updates the number of artists displayed on the statusbar.
		 * @param QString Number Of Titles
		 */
		void updateArtistText(const QString &);
		/** Update category slot.
		 * @param categoryInfo Struct containing XID and new name
		 */
		void updateCategory(categoryInfo);
		/**
		 * Dirty work for updating title.
		 * @param titleInfo Struct containing updated title information
		 */
		void updateTitle(titleInfo);
		/** Update track slot.
		 * @param QString CDID
		 * @param trackInfo trackInfo Struct
		 * @param QListView Pointer to QListView for updating
		 */
		void updateTrack(QString, trackInfo, QListView *);
		/**
		 * Repopulates the track information in a QListView.
		 * @param QListView Pointer to QListView for updating
		 * @param QString CDID
		 */
		void updateTrackInfo(QListView *, QString);
		/**
		 * Retrieves lyric and populates QTextEdit.
		 * @param QTextEdit Item To Populate
		 * @param QString Lyric
		 * @param bool Used to check if there was a successful retrieval of lyrics.
		 */
		void getLyric(QTextEdit *, QString, bool *);
		/**
		 * Updates lyrics for a track.
		 * @param QString Lyric
		 * @param QString Track ID
		 * @param bool Set TRUE on successful completion, FALSE otherwise.
		 */
		void updateLyric(QString, QString, bool *);
	private slots:
		/** Updates the Caption.
		 * @param bool TRUE if connecting, FALSE if disconnecting
		 */
		void connectGUI(bool);
		/**
		 * Delete pointer from QMap.
		 * @param titleDialog Pointer to titleDialog to remove
		 */
		void deleteFromQMap(titleDialog *) const;
		/**
		 * Delete pointer from QMap.
		 * @param preferencesDialog Pointer to preferencesDialog to remove
		 */
		void deleteFromQMap(preferencesDialog *) const;
		/**
		 * Dummy slot for disposing of run time error messages.
		 * @param int Integer
		 */
		void dummy_slot(int) const;
		/** Grays the delete option in the artist menu.
		 * @param bool TRUE enables, FALSE disables
		 */
		void enableArtist(bool);
		/**
		 * Enables or disables the CDDB menu/toolbar buttons.
		 * @param bool TRUE enables, FALSE disables
		 */
		void enableCDDB(bool);
		/**
		 * Enables or disables the new and open db buttons.
		 * @param bool TRUE enables, FALSE disables
		 */
		void enableDbGUI(bool);
		/** Sets new artist button, grays the delete option in the artist menu.
		 * @param bool TRUE enables, FALSE disables
		 */
		void enableDeleteArtist(bool);
		/** Grays the edit/delete title buttons.
		 * @param bool TRUE enables, FALSE disables
		 */
		void enableEditDeleteTitle(bool);
		/** Grays the most items in the menu.
		 * @param bool TRUE enables, FALSE disables
		 */
		void enableItems(bool);
		/** Grays some title options and toolbar buttons.
		 * @param bool TRUE Enables, FALSE Disables
		 */
		void enableTitle(bool);
		/** Displays a standard ErrorBox if a bad return value is received.
		 * @param int Pass error code (see error.h) to display the appropriate error message box
		 */
		void errBox(int);
		/**
		 * Generates and displays report.
		 * @param reportInfo Struct with report options
		 * @param colorInfo Struct with color options
		 * @param int Report Type
		 */
		void generate_report(reportInfo, colorInfo, int);
		/**
		 * Connects to the default db if that option is enabled.
		 */
		void initAutoConnect();
		/**
		 * Slot to sort open category dialogs.
		 * @param bool TRUE is ascending, FALSE is descending
		 */
		void sortCategorySlot(bool) const;
		/** Slot for toggling the name search toolbar.
		 * @param bool TRUE enables, FALSE disables
		 */
		void toggleName(bool);
		/** slot for toggling the title search toolbar.
		 * @param bool TRUE enables, FALSE disables
		 */
		void toggleTitle(bool);
		/**
		 * Code to fill up artist widgets.
		 */
		void updateArtistView();
		/**
		 * Retrieves current artist list and then updates the title list for the current artist.
		 */
		void updateArtistTitleView();
		/**
		 * Updates the GUI progress bar and text.
		 * @param int Progressbar Value
		 * @param QString Statusbar Text
		 */
		void updateGUI(int, QString);
		/** Updates total time displayed on the statusbar.
		 * @param QString Total Length
		 */
		void updateLength(const QString &);
		/** Updates the category lists in the currently open windows.
		 * This function uses the QMap's pointers to fill the combo boxes
		 * with new information.
		 * @param int Which Category
		 */
		void updateLists(int);
		/** Set the value of the progress bar on the statusbar.
		 * @param int Percentage to set progressbar
		 */
		void updateProgress(const int &) const;
		/** Updates the text on the statusbar.
		 * @param char Text to display on the statusbar
		 */
		void updateStatusBar(const char *);
		/** Slot for updating titles for current artist.
		 */
		void updateTitleView();
		/**
		 * Updates second combobox in title search when the first one is changed.
		 * @param int Sets the value of the KComboBox
		 */
		void update_toolbar_combobox(int);
		/** Updates the number of titles displayed on the statusbar.
		 * @param char Total titles not collected
		 * @param char Total titles all together
		 */
		void updateTotalTitles(const char *, const char *);
		/**
		 * Decreases window count by one. If count reaches 0, new and open database buttons are reenabled.
		 */
		void windowDec();
		/**
		 * Increments the window count by one when called. If window count is greater than 0, the new
		 * and open db buttons are disabled. This makes it impossible to create or open a database while
		 * other windows are still open.
		 */
		void windowInc();
		/** Manages the category menues to ensure category memory gets freed.
		 * @param int Which Category
		 */
		void windowManager(int);
	signals:
		/** You should manually emit this signal after (dis)connecting to the db.
		 * @param bool TRUE connected, FALSE disconnected
		 */
		void signalDbConnect(bool);
	private:
		/**
		 *Checks the version of the open database against the program . If they differ, a warning message is issued.
		 */
		void checkVersion();
		/**
		 * Closes the db by deleting the object. Emits signalDbConnect(FALSE).
		 */
		void close_db();
		/**
		 * Sets up musicArtistTitle, the browser widget.
		 */
		void initCenterWidget();
		/**
		 * Runs the initial checks on the db to insure that it's up and working properly.
		 *
		 * @param QString Connect String (see @ref returnConnectInfo)
		 */
		void initDb(const QString &);
		/**
		 * Initializes all the default values for the values. readSettings then loads the values from the config file.
		 */
		void initDefault();
		/**
		 * Initializes all the menu entries.
		 */
		void initMenu();
		/**
		 * Initializes all the status bar code.
		 */
		void initStatusBar();
		/**
		 * Initializes all 3 toolbars.
		 */
		void initToolBars();
		/**
		 * Creates a new db in music.
		 */
		void new_db();
		/**
		 * Opens a database on music.
		 * Returns negative if unsuccessful.
		 *
		 * @param dbInfo Struct with database connect info.
		 */
		int open_db(const dbInfo *const);
		/**
		 * Opens edit title dialog box.
		 * @param QString CD ID
		 */
		 void openEditTitle(const QString &);
		/**
		 * This function processes the command line args for kmusicdb. If 0 is passed the function just returns.
		 *
		 * @param KCmdLineArgs Command Line Arguments
		 */
		void processArgs(KCmdLineArgs *);

		// menu items
		KAction *restoredb, *backupdb, *opti, *eartist, *etitle, *eformat, *egenre, *etype, *emethod, *elabel;
		KAction *btebrowse, *statistics, *collectionrep, *artistrep, *daterangerep, *bterep, *tbya, *trbyt;
		KAction *newdb, *opendb, *closedb, *newtitle, *newartist, *delartist, *deltitle, *eboxset;
		KAction *addcddb, *submitcddb, *appendcddb, *updatecddb, *searchcddb, *topcdrep, *customrep, *listtitle, *listartist;
		KToggleAction *namesearch, *titlesearch, *showtoolbar, *showstatusbar;
		// statusbar items
		QProgressBar *sbp;

		// pointers to classes and structs and dbInfo
		dbInit *init;
		dbMusic *music;
		musicArtistTitle *mid;
		dbConnectDialog *opener;
		preferencesDialog *pref;
		artistDialog *artistdialog;
		titleDialog *titledialog;
		dbInfo *dbconnectinfo;
		dbInfo *dbauto;
		Defaults *def;
		titleInfo *title;
		artistInfo *artist;
		enabledTitleColumns *t_op;
		enabledTrackColumns *tr_op;
		enabledCategoryColumns *c_op;
		categoryDialog *catdialog[7];
		reportDialog *report;
		colorInfo *color;
		reportInfo *reportOptions;
		cdparanoia *cdp;
		cddb *cdnet;
		CDDBserverInfo *si;

		// QMap Window Managers
		TitleDialogManager *twm;
		PreferencesDialogManager *pwm;

		// misc stuff used by system
		KComboBox  *category, *catresult, *nametypes;
		KLineEdit *toolbarname;
		int dialog_result, c, stat, window_count;

		//aid
		QString aid;
};


#endif

