/***************************************************************************
                          kmusicdbvalidator.cpp  -  description
                             -------------------
    begin                : Sat Apr 14 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmusicdbvalidator.h"

/**
 * Returns a titleInfo struct that contains values that can be sent to the database safely.
 * @param titleInfo Struct to be filled
 */
void blanktitleInfo(titleInfo &i)
{
	i.notracks="0";
	i.length = "00:00:00";
	for (int j=0;j<6;j++)
	{
		if (i.cat[j].name.isEmpty())
			i.cat[j].name="UNKNOWN";
	}
	if (i.nod.isEmpty())
		i.nod = "1";
	if (i.yearend.isEmpty())
		i.yearend = "0";
	if (i.yearstart.isEmpty())
		i.yearstart = "0";
	if (i.rating.isEmpty())
		i.rating = "5";
	if (i.quality.isEmpty())
		i.quality = "5";
}
/**
 * Parses a QString for characters that a bash interface does not like(or could post a security
 * problem) and replaces them with '_'.
 * @param filename Name Of File
 */
QString
validateFilename(const QString &string)
{
//	cerr << "validateFilename: " << string;
	if (string.isEmpty())
	{
		cerr << "Empty." << endl;
		return string;
	}
	QString t = string;
	int z = t.length();
	QString badchars = "&`';\\\"|*?<>(){}[]$\n\r";
	int y = badchars.length();
	for (int i = 0; i < z; i++)
	{
		for (int j = 0; j < y; j++)
		{
			if (t[i] == badchars[j])
				t[i] = '_';
		}
	}
	if (t.endsWith("\n"))
		t.truncate(t.length()-1);
//	cerr << "\t" << t << endl;
	return t;
}

/**
 * Parses a QString and capitalizes the first letter of each word.
 * @param QString String
 */
QString
capitalizeWords(QString &s)
{
//	cerr << "capitalizeWords: " << s;
	if (s.isEmpty())
	{
		cerr << "Empty." << endl;
		return s;
	}
	int i = 0;
	int z = s.length() - 1;

	// capitalize first letter
	s[0]=s[0].upper();

	// capitalize after spaces
	do
	{
		i = s.find(" ", i);
		if (i == z)
			break;
		s[i + 1] = s[i + 1].upper();
		i++;
	}
	while (i != 0);

	// captitalize directly after hyphen
	i = 0;
	do
	{
		i = s.find("-", i++);
		if (i == z)
			break;
		s[i + 1] = s[i + 1].upper();
		i++;
	}
	while (i != 0);

//	cerr << "\t" << s << endl;
	return s;

}

/**
 * Parses a QString and capitalizes the first letter of each word and lowercases the rest.
 * Is intended to be a smarter version of capitalizeWords.
 * @param QString String
 */
QString
formatText(QString &s)
{
//	cerr << "formatText: " << s;
	if (s.isEmpty())
	{
		cerr << "Empty." << endl;
		return s;
	}

	s = s.lower();

	int i = 0;
	int z = s.length() - 1;
	do
	{
		i = s.find(" ", i);
		if (i == z)
			break;
		s[i + 1] = s[i + 1].upper();
		i++;
	}
	while (i != 0);


	// captitalize directly after punctuation
	for (unsigned int j = 0; j < s.length() - 1; j++)
	{
		if (s[j].isPunct())
		{
			if (s[j] == '\'')
				continue;
			s[j + 1] = s[j + 1].upper();
			j++;
		}
	}

//	cerr << "\t" << s << endl;
	return s;
}

/**
 * Decodes QByteArray into a CDDBtitleInfo(assuming it is an encoded CDDBtitleInfo in the first place).
 * @param QStringList Encoded Data
 * @param CDDBtitleInfo Struct to be filled
 */
bool processTitle(const QStringList &l, CDDBtitleInfo &i)
{
	trackInfo y;
	int nterms=0;
	QStringList::ConstIterator it;
	it=l.begin();
	cerr << "------------TOTAL NUMBER OF l ENTRIES: " << l.count() << endl;

	i.title.name=*it++;
	i.title.cdid=*it++;
	for (int j=0;j<6;j++)
	{
		i.title.cat[j].name=*it++;
		i.title.cat[j].ctype=j;
	}
	i.title.yearstart=*it++;
	i.title.yearend=*it++;
	i.title.nod=*it++;
	i.title.notracks=*it++;
	i.title.length=*it++;
	i.title.notes=*it++;
	i.title.picpath=*it++;
	i.title.rating=*it++;
	i.title.quality=*it++;
	if ((*it++)=="1")
		i.title.collected=TRUE;
	else
		i.title.collected=FALSE;
	i.title.code=*it++;
	if ((*it++)=="1")
		i.title.loan=TRUE;
	else
		i.title.loan=FALSE;
	i.device=*it++;
	i.disc_id=*it++;
	i.cddb_genre=*it++;
	i.totalLength=*it++;
	cerr << "------------TITLE RESULTS: " << i.title.name << ", " << i.title.cdid << ", " << i.title.yearstart << ", " << i.title.yearend;
	cerr << ", " << i.title.nod << ", " << i.title.notracks << ", " << i.title.length << ", " << i.title.notes << ", " << i.title.picpath;
	cerr << ", " << i.title.quality << ", " << i.title.rating << ", " << i.title.collected << ", " << i.title.code << ", " << i.title.loan;
	cerr <<	", " << i.device << ", " << i.disc_id;
	cerr << ", " << i.cddb_genre << ", " << i.totalLength << endl;
	cerr << i.title.cat[0].name << " " << i.title.cat[1].name << " " << i.title.cat[2].name << " " << i.title.cat[3].name << " " << i.title.cat[4].name << " " << i.title.cat[5].name << endl;

	nterms=(*it++).toInt();
	cerr << "------------NUMBER OF TRACKS: " << nterms << endl;
	for (int j=0;j<nterms;j++)
	{
		y.trackid=*it++;
		y.name=*it++;
		y.length=*it++;
		y.lyric=*it++;
		y.side=*it++;
		y.notes=*it++;
		cerr << "TRACKNAME: " << y.trackid << " " << y.name << " " << y.length << " " << y.lyric << " " << y.side << " " << y.notes << endl;
		i.trackData.append(y);
	}

	nterms=(*it++).toInt();
	for (int j=0;j<nterms;j++)
	{
		i.trackSectorLength.append((*it++).toInt());
	}

	nterms=(*it++).toInt();
	for (int j=0;j<nterms;j++)
	{
		i.trackSectorBegin.append((*it++).toInt());
	}
	return TRUE;
}

/**
 * In a time formatted string(xx:xx:xx), this function will return the hour segment.
 * @param QString String
 */
int
returnHour(const QString &l)
{
	QString z = l;
	z.remove(2, 6);
	return z.toInt();
}

/**
 * In a time formatted string(xx:xx:xx), this function will return the minute segment.
 * @param QString String
 */
int
returnMinute(const QString &l)
{
	QString z = l;
	z.remove(0, 3);
	z.remove(2, 3);
	return z.toInt();
}

/**
 * In a time formatted string(xx:xx:xx), this function will return the second segment.
 * @param QString String
 */
int
returnSecond(const QString &l)
{
	QString z = l;
	z.remove(0, 6);
	return z.toInt();
}

/**
 * Given the 3 ints, this function will return a formatted time segment(xx:xx:xx).
 * @param int Hours
 * @param int Minutes
 * @param int Seconds
 */
QString
returnLength(const int &h, const int &m, const int &s)
{
	QString length = "";
	if (h < 10)
		length.append("0");
	length.append(QString::number(h));
	length.append(":");
	if (m < 10)
		length.append("0");
	length.append(QString::number(m));
	length.append(":");
	if (s < 10)
		length.append("0");
	length.append(QString::number(s));

//	cerr << "Length: " << length << endl;

	return length;

}

