/***************************************************************************
                          musicartisttitle.cpp  -  description
                             -------------------
    begin                : Wed Apr 4 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "musicartisttitle.h"

/**
 * Constructor
 * @param QWidget Parent
 */
musicArtistTitle::musicArtistTitle(QWidget *parent): QWidget(parent)
{
	cerr << "musicArtistTitle::initialized" << endl;

	QGridLayout *glay = new QGridLayout(this, 2, 2, 0);
	artist = new musicComboBox(this);
	artist->setMinimumWidth(fontMetrics().width(" abcdefghijklmnopqrstuvwxyz "));

	title = new musicQListView(this, 3);
	title->setRootIsDecorated(TRUE);
	title->setVScrollBarMode(QListView::AlwaysOn);

	QLabel *label = new QLabel(title, i18n("Artist->	"), this);
	label->setMinimumWidth(fontMetrics().width("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"));
	label->setAlignment(Qt::AlignRight | Qt::AlignVCenter);

	int w = fontMetrics().width(" abcdefghijklmnopqrst ");
	title->addColumn(i18n("Title"), w);

	title->setMinimumHeight(100);

	glay->addWidget(label, 0, 0);
	glay->addWidget(artist, 0, 1);
	glay->addMultiCellWidget(title, 1, 1, 0, 1);

	QToolTip::add(artist, i18n("Click on me to drop the menu down and select an artist to view.\nRight click to edit."));
//	QToolTip::add(title, i18n("Doubleclick a title to edit."));
}

/**
 * Destructor
 */
musicArtistTitle::~musicArtistTitle()
{
}

/**
 * Returns a pointer to the musicComboBox.
 */
musicComboBox *musicArtistTitle::combobox() const
{
	return artist;
}
/** Clears out artist and title. */
void
musicArtistTitle::clearItems() const
{
	artist->clear();
	title->clear();
}

/** Returns the the current value of the KCombobox. */
QString
musicArtistTitle::getArtistText() const
{
	return artist->currentText();
}

/** Grays the items in this class.
 * @param bool TRUE enables, FALSE disables
 */
void
musicArtistTitle::enable(bool t) const
{
	if (!t)
		clearItems();
	artist->setEnabled(t);
	title->setEnabled(t);
}

/**
 * Sets up the columns in the listview. Depending on the bool values in the struct, the column is
 * either added, or not.
 * @param enabledDisplayOptions Pointer To Struct
 */
void
musicArtistTitle::initHeaders(const enabledTitleColumns *const dis)
{
	cerr << "musicArtistTitle::initHeaders" << endl;
	if (!dis)
		return;
	int columnnumber = 1;
	for (int i = 1; i < title->columns();)
		title->removeColumn(i);

	if (dis->fid)
		title->addColumn(i18n("Format"));
	if (dis->gid)
		title->addColumn(i18n("Genre"));
	if (dis->lid)
		title->addColumn(i18n("Label"));
	if (dis->mid)
		title->addColumn(i18n("Method"));
	if (dis->tid)
		title->addColumn(i18n("Type"));
	if (dis->yearstart)
		title->addColumn(i18n("Year"));
	if (dis->yearend)
		title->addColumn(i18n("Year End"));
	if (dis->nod)
		title->addColumn(i18n("Media"));
	if (dis->notracks)
		title->addColumn(i18n("Tracks"));
	if (dis->length)
		title->addColumn(i18n("Length"));
	if (dis->rating)
		title->addColumn(i18n("Rating"));
	if (dis->quality)
		title->addColumn(i18n("Quality"));
	if (dis->code)
		title->addColumn(i18n("Other"));
	if (dis->loan)
		title->addColumn(i18n("On Loan?"));
	if (dis->collected)
		title->addColumn(i18n("Collected?"));
//	title->addColumn("test1");
//	title->addColumn("test2");
//	title->addColumn("test3");
	columnnumber = title->columns();
	for (int i = 1; i < columnnumber; i++)
		title->setColumnAlignment(i, AlignHCenter);
}
/** Returns a pointer to the musicQListView. */
musicQListView *musicArtistTitle::listview() const
{
	return title;
}

/**
 * Populates the KCombobox with the artist info from QStringList.
 * @param QStringList Pointer to artist list
 */
void
musicArtistTitle::populateArtist(const QStringList *const z) const
{
	cerr << "musicArtistTitle::populateArtist" << endl;
	artist->clear();
	artist->insertStringList(*z);
}
/**
 * Searches for the value and sets the KComboBox to that value(if it exists).
 * @param QString Value to search for and then set
 */
void
musicArtistTitle::setArtistText(const QString &name) const
{
	if (name.isEmpty())
		return;
	// look	for the name and make it the current index.
	for (unsigned int i = 0; i < artist->listBox()->count(); i++)
	{
		if (name == artist->listBox()->item(i)->text())
		{
			artist->setCurrentItem(i);
			break;
		}
	}
}

/** Sorts the list depending on column int.
 * @param int Column
 * @param bool TRUE is ascending, FALSE is descending
 */
void
musicArtistTitle::sortList(int t, bool u)
{
	cerr << "musicArtistTitle::sorting list" << endl;
	QString column_search = "";
	switch(t)
	{
		case 0:
			column_search = i18n("Title");
			break;
		case 1:
			column_search = i18n("Format");
			break;
		case 2:
			column_search = i18n("Genre");
			break;
		case 3:
			column_search = i18n("Label");
			break;
		case 4:
			column_search = i18n("Method");
			break;
		case 5:
			column_search = i18n("Type");
			break;
		case 6:
			column_search = i18n("Year");
			break;
		case 7:
			column_search = i18n("Year End");
			break;
		case 8:
			column_search = i18n("Media");
			break;
		case 9:
			column_search = i18n("Tracks");
			break;
		case 10:
			column_search = i18n("Length");
			break;
		case 11:
			column_search = i18n("Rating");
			break;
		case 12:
			column_search = i18n("Quality");
			break;
		case 13:
			column_search = i18n("Other");
			break;
		case 14:
			column_search = i18n("On Loan?");
			break;
		case 15:
			column_search = i18n("Collected?");
			break;
		case 16:
			column_search = i18n("CDID");
			break;
		default:
			return;
	};

	int column_sort = 0;
	for (int i = 0; i < title->columns(); i++)
	{
		if (title->columnText(i) == column_search)
		{
			column_sort = i;
			break;
		}
	}
	if (t > 15)
		column_sort=title->columns();
	title->setSorting(column_sort, u);
}

/**
 * Resizes the columns to be aesthetically pleasing. The first column is the dynamic and the last
 * columns are set the the minimum size required to display full information. This means that the
 * first column can be cut short. */
void
musicArtistTitle::resizeColumns()
{
	cerr << "musicArtistTitle::resizing:";
	int physicalwidth = title->size().width();
	cerr << physicalwidth << endl;
	int columnwidth = title->specialWidth();
	if (physicalwidth > columnwidth)
		title->setColumnWidth(0, physicalwidth - columnwidth);
	else
		title->setColumnWidth(0, 50);
}

#include "musicartisttitle.moc"
