/***************************************************************************
                          musicartisttitle.h  -  description
                             -------------------
    begin                : Wed Apr 4 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MUSICARTISTTITLE_H
#define MUSICARTISTTITLE_H

#include "include.h"
#include "musicqlistview.h"
#include "musiccombobox.h"

/**This is the browser widget for KmusicdB. It is the center part of the program displaying the artist in
 * musicCombobox and the musicQListView with the title information.
 *
 * @author Will DeRousse
 * @short KmusicdB central browsing widget.
 *<pre>
 *musicArtistTitle *middle=new musicArtistTitle(page, sort_index, true);
 *setCenterWidget(middle);
 *</pre>
 */

class musicArtistTitle : public QWidget{
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 */
		musicArtistTitle(QWidget *);
		/**
		 * Destructor
		 */
		~musicArtistTitle();
		/**
		 * Clears out artist and title.
		 */
		void clearItems() const;
		/**
		 * Returns a pointer to the musicComboBox.
		 */
		musicComboBox *combobox() const;
		/**
		 * Returns the the current value of the KCombobox.
		 */
		QString getArtistText() const;
		/**
		 * Sets up the columns in the listview. Depending on the bool values in the struct, the column is
		 * either added, or not.
		 * @param enabledDisplayOptions Pointer To Struct
		 */
		void initHeaders(const enabledTitleColumns *const);
		/**
		 * Returns a pointer to the musicQListView.
		 */
		musicQListView *listview() const;
		/**
		 * Populates the KCombobox with the artist info from QStringList.
		 * @param QStringList Pointer to artist list
		 */
		void populateArtist(const QStringList *const) const;
		/**
		 * Resizes the columns to be aesthetically pleasing. The first column is the dynamic and the last
		 * columns are set the the minimum size required to display full information. This means that the
		 * first column can be cut short. */
		void resizeColumns();
		/**
		 * Searches for the value and sets the KComboBox to that value (if it exists).
		 * @param QString Value to search for and then set
		 */
		void setArtistText(const QString &) const;
	public slots:
		/** Grays the items in this class.
		 * @param bool TRUE enables, FALSE disables
		 */
		void enable(bool=false) const;
		/** Sorts the list depending on column int.
		 * @param int Column
		 * @param bool TRUE is ascending, FALSE is descending
		 */
		void sortList(int, bool);
	private:
		musicComboBox *artist;
		musicQListView *title;
};


#endif

