/***************************************************************************
                          musicqlistview.cpp  -  description
                             -------------------
    begin                : Wed Apr 11 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "musicqlistview.h"

//--------------------------------------CDDBTITLEINFODRAG---------------------------------------------
/**This is the CDDBtitleInfo drag and drop class. It is used for dragging and dropping titles.
 *
 * @author Will DeRousse
 * @short CDDBtitleInfo Drag and Drop
 */

/**
 * Constructor
 * @param QWidget Parent
 * @param CDDBtitleInfo Struct containing drag data
 * @param char Name
 */
CDDBtitleInfoDrag::CDDBtitleInfoDrag(QWidget *dragSource, const CDDBtitleInfo &i, const char *name)
		: QDragObject( dragSource, name )
{
	cerr << "CDDBtitleInfoDrag::initialized" << endl;
	info=i;
}
/**
 * Used to determine if this class can decode the QMimeSource or not.
 * @param QMimeSource Dropped Data
 */
bool CDDBtitleInfoDrag::canDecode(QMimeSource *e)
{
		return e->provides( "kmusicdb/cddbtitleinfo" );
}
/**
 * Decodes the dropped data.
 * @param QDropEvent Event information regarding drop
 * @param CDDBtitleInfo Struct to be filled
 */
bool CDDBtitleInfoDrag::decode(QDropEvent*e, CDDBtitleInfo &i)
{
	QByteArray data=e->encodedData("kmusicdb/cddbtitleinfo");
	cerr << "CDDBtitleInfoDrag::TOTAL DROP SIZE: " << data.size() << "\nDATA: " << data.data() << endl;
	if (!data.size())
		return FALSE;

	e->accept();

	QStringList l=QStringList::split("$@@$",data.data(),TRUE);
	processTitle(l,i);
	return TRUE;
}
/**
 * Utility function for encoding a CDDBtitleInfo struct.
 * @param CDDBtitleInfo Struct to be encoded into class
 */
void CDDBtitleInfoDrag::encode(const CDDBtitleInfo &i)
{
	info=i;
}
/**
 * Returns QByteArray encoded data.
 * @param char MIME Type
 */
QByteArray CDDBtitleInfoDrag::encodedData(const char *mime) const
{
	QByteArray d;
	QString s;
	if ( QString(mime) == "kmusicdb/cddbtitleinfo" )
	{
		// append titleInfo
		s.append(info.title.name);
		s.append("$@@$");
		s.append(info.title.cdid);
		s.append("$@@$");
		s.append(info.title.cat[KBOXSET].name);
		s.append("$@@$");
		s.append(info.title.cat[KFORMAT].name);
		s.append("$@@$");
		s.append(info.title.cat[KGENRE].name);
		s.append("$@@$");
		s.append(info.title.cat[KLABEL].name);
		s.append("$@@$");
		s.append(info.title.cat[KMETHOD].name);
		s.append("$@@$");
		s.append(info.title.cat[KTYPE].name);
		s.append("$@@$");
		s.append(info.title.yearstart);
		s.append("$@@$");
		s.append(info.title.yearend);
		s.append("$@@$");
		s.append(info.title.nod);
		s.append("$@@$");
		s.append(info.title.notracks);
		s.append("$@@$");
		s.append(info.title.length);
		s.append("$@@$");
		s.append(info.title.notes);
		s.append("$@@$");
		s.append(info.title.picpath);
		s.append("$@@$");
		s.append(info.title.rating);
		s.append("$@@$");
		s.append(info.title.quality);
		s.append("$@@$");
		if (info.title.collected)
			s.append("1$@@$");
		else
			s.append("0$@@$");
		s.append(info.title.code);
		s.append("$@@$");
		if (info.title.loan)
			s.append("1$@@$");
		else
			s.append("0$@@$");
		// rest of the QString's
		s.append(info.device);
		s.append("$@@$");
		s.append(info.disc_id);
		s.append("$@@$");
		s.append(info.cddb_genre);
		s.append("$@@$");
		s.append(info.totalLength);
		s.append("$@@$");
		// QValueList<trackInfo>
		QValueList<trackInfo>::ConstIterator it;
		s.append(QString::number(info.trackData.count()));
		s.append("$@@$");
		for (it=info.trackData.begin();it!=info.trackData.end();++it)
		{
			s.append((*it).trackid);
			s.append("$@@$");
			s.append((*it).name);
			s.append("$@@$");
			s.append((*it).length);
			s.append("$@@$");
			s.append((*it).lyric);
			s.append("$@@$");
			s.append((*it).side);
			s.append("$@@$");
			s.append((*it).notes);
			s.append("$@@$");
		}

		//QValueList<int> trackSectorLength
		QValueList<int>::ConstIterator it2;
		s.append(QString::number(info.trackSectorLength.count()));
		s.append("$@@$");
		for (it2=info.trackSectorLength.begin();it2!=info.trackSectorLength.end();++it2)
		{
			s.append(QString::number(*it2));
			s.append("$@@$");
		}

		//QValueList<int> trackSectorBegin
		s.append(QString::number(info.trackSectorBegin.count()));
		s.append("$@@$");
		for (it2=info.trackSectorBegin.begin();it2!=info.trackSectorBegin.end();++it2)
		{
			s.append(QString::number(*it2));
			s.append("$@@$");
		}

		d.resize(s.length());
		memcpy( d.data(), s.latin1(), s.length() );
	}
	return d;
}
/**
 * Returns supported formats.
 * @param int Format Number
 */
const char *CDDBtitleInfoDrag::format(int i) const
{
	if ( i == 0 )
		return "kmusicdb/cddbtitleinfo";
	else
		return 0;
}


//--------------------------------TRACKINFODRAG------------------------------------------
/**This is the trackInfo drag and drop class. It is used for dragging and dropping tracks.
 *
 * @author Will DeRousse
 * @short trackInfo Drag and Drop
 */

/**
 * Constructor
 * @param QWidget Parent
 * @param trackInfo Struct containing drag data
 * @param name Name
 */
trackInfoDrag::trackInfoDrag(QWidget *dragSource, const trackInfo &i, const char *name)
		: QDragObject( dragSource, name )
{
	cerr << "trackInfoDrag::initialized" << endl;
	info=i;
}
/**
 * Used to determine if this class can decode the QMimeSource or not.
 * @param QMimeSource Dropped Data
 */
bool trackInfoDrag::canDecode(QMimeSource *e)
{
	return e->provides( "kmusicdb/trackinfo" );
}
/**
 * Decodes the dropped data.
 * @param QDropEvent Event information regarding drop
 * @param trackInfo Struct to be filled
 */
bool trackInfoDrag::decode(QDropEvent*e, trackInfo &i)
{
	QByteArray data=e->encodedData("kmusicdb/trackinfo");
	if (!data.size())
		return FALSE;

	e->accept();

	cerr << "trackInfoDrag::TOTAL DROP SIZE: " << data.size() << " DATA:\n" << data.data() << endl;
	QStringList l=QStringList::split("$@@$",data.data(),TRUE);
	i.trackid=l[0];
	i.name=l[1];
	i.length=l[2];
	i.lyric=l[3];
	i.side=l[4];
	i.notes=l[5];
	return TRUE;
}
/**
 * Utility function for encoding a CDDBtitleInfo struct.
 * @param trackInfo Struct to be encoded into class
 */
void trackInfoDrag::encode(const trackInfo &i)
{
	info=i;
}
/**
 * Returns QByteArray encoded data.
 * @param char MIME Type
 */
QByteArray trackInfoDrag::encodedData(const char *mime) const
{
	QByteArray d;
	QString s;
	if ( QString(mime) == "kmusicdb/trackinfo" )
	{
		s.append(info.trackid);
		s.append("$@@$");
		s.append(info.name);
		s.append("$@@$");
		s.append(info.length);
		s.append("$@@$");
		s.append(info.lyric);
		s.append("$@@$");
		s.append(info.side);
		s.append("$@@$");
		s.append(info.notes);
		s.append("$@@$");
		d.resize(s.length());
		memcpy( d.data(), s.latin1(), s.length() );
	}
	return d;
}
/**
 * Returns supported formats.
 * @param int Format Number
 */
const char* trackInfoDrag::format(int i) const
{
	if ( i == 0 )
		return "kmusicdb/trackinfo";
	else
		return 0;
}



//---------------------------MUSICQLISTVIEW-------------------------------
/**
 * Constructor
 * @param QWidget Parent
 * @param int Drag 'N Drop Type: 0 none, 1 QString, 2 trackInfo, 3 CDDBtitleInfo
 */
musicQListView::musicQListView(QWidget * parent, const int &dt) : QListView(parent, "title")
{
	cerr << "musicQListView::initialized" << endl;
	connect(this,SIGNAL(selectionChanged()),this,SLOT(handleKeyEvents()));
	setAllColumnsShowFocus(TRUE);
	setAcceptDrops(TRUE);
	dndtype=dt;
}
/**
 * Destructor
 */
musicQListView::~musicQListView()
{
}
/**
 * Handles mouse and keyboard selection events for emitted imFocused.
 */
void musicQListView::handleKeyEvents()
{
	if (selectedItem())
		emit imFocused(TRUE);
	else
	{
		emit imFocused(FALSE);
		clearSelection();
	}
}
/**
 * Handles whether or not the data should be copied for dragging and dropping.
 * Encodes the category name.
 * @param QMouseEvent Mouse Movement Event Data
 */
void musicQListView::contentsMouseMoveEvent(QMouseEvent *e)
{
	QString dd;
	QDragObject *d;
	if (selectedItem()==0)
		return;
	if (!e->state())
		return;
	switch(dndtype)
	{
		case 1:
			dd=selectedItem()->text(0);
			d = new QTextDrag( dd, this );
			d->dragCopy();
			break;
		case 2:
		case 3:
			emit dragged(this);
			break;
		case 0:
		default:
			return;
	};
}
/**
 * Handles whether or not the data can be decoded or not.
 * This class decodes text.
 * @param QDragEnterEvent Event pertaining to drag enter
 */
void musicQListView::dragEnterEvent(QDragEnterEvent *e)
{
	if (e->source() == this)
		return;
	switch(dndtype)
	{
		case 1:
			if ( !QTextDrag::canDecode(e) )
				goto cleanup;
			cerr << "musicQListView::QTextDrag decode accepted" << endl;
			break;
		case 2:
			if ( !trackInfoDrag::canDecode(e) )
				goto cleanup;
			cerr << "musicQListView::trackInfoDrag decode accepted" << endl;
			break;
		case 3:
			if ( !CDDBtitleInfoDrag::canDecode(e) )
				goto cleanup;
			cerr << "musicQListView::CDDBtitleInfoDrag decode accepted" << endl;
			break;
		case 0:
		default:
			return;
	};
	e->accept();
cleanup:
	return;
}
/**
 * Emits dropped(QString) after decode.
 * @param QDropEvent Event pertaining to dropping
 */
void musicQListView::dropEvent(QDropEvent *e)
{
	QString item;
	trackInfo ti;
	CDDBtitleInfo i;
	if (e->source()==this)
		return;
	switch(dndtype)
	{
		case 1:
			if ( QTextDrag::decode(e, item) )
				emit dropped(item);
			break;
		case 2:
			if ( trackInfoDrag::decode(e, ti) )
				emit dropped(ti);
			break;
		case 3:
			if ( CDDBtitleInfoDrag::decode(e, i) )
				emit dropped(i);
			break;
		case 0:
		default:
			return;
	};
}
/**
 * Allows an item to be "ensureItemVisible" by specifying CDID rather than pointer.
 */
void musicQListView::ensureItemVisibleByCDID(const QString &id)
{
	QListViewItemIterator it(this);
	while (it.current()) {
		if (it.current()->text(columns())==id) {
			ensureItemVisible(it.current());
			break;
		}
		it++;
	}
}
/**
 * Returns width of QListView.
 */
int musicQListView::specialWidth()
{
	int number = columns();
	cerr << "musicQListView::specialWidth: " << number;
	int total_width = 0;
	for (int i = 1; i < number; i++)
		total_width += columnWidth(i);
	total_width += verticalScrollBar()->width();
	total_width += 6;
	cerr << "/" << total_width << endl;
	return total_width;
}

#include "musicqlistview.moc"
