/***************************************************************************
                          picwidget.h  -  description
                             -------------------
    begin                : Fri May 4 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PICWIDGET_H
#define PICWIDGET_H

#include "include.h"

/** This is a generic picture widget. It contains a QPixmap and a KURL to store a URL.
 *
 * It is easy to use, just instantiate the widget with a parent and a url to the path and everything will be taken care of.
 * @short Shows pic and provides methods to change and retrieve.
 * @author Will DeRousse
 *<pre>
 *KURL url;
 *url.setPath("/home/will/test.jpg");
 *picWidget *w=new picWidget(this,url);
 *w->show();
 *</pre>
 */

class picWidget : public QWidget  {
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 * @param KURL Path To Image
		 */
		picWidget(QWidget *, const KURL &);
		/**
		 * Destructor
		 */
		~picWidget();
		/**
		 * Gets image path.
		 */
		KURL getImage() const;
		/**
		 * Saves the pic to the path in the url.
		 * @param KURL Path To Image
		 */
		void saveImage(const KURL &) const;
		/**
		 * Sets image to new path.
		 * @param KURL URL to new pic location
		 */
		void setImage(const KURL &);
	protected slots:
		/**
		 * Reimplemented slot.
		 * @param paintEvent Screen Paint Event Data
		 */
		void paintEvent(QPaintEvent *);
	private:
		QPixmap *pic;
		KURL kmurl;
};


#endif

