/***************************************************************************
                          preferencesdialog.cpp  -  description
                             -------------------
    begin                : Fri Mar 30 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "preferencesdialog.h"

/**
 * Constructor
 * @param QWidget Parent
 * @param Defaults Pointer to Defaults
 * @param dbInfo Pointer to Current dbInfo
 * @param enabledTitleColumns Pointer to enabledTitleColumns (for mainview columns)
 * @param reportInfo Pointer to reportInfo
 * @param colorInfo Pointer to colorInfo(for reports)
 */
preferencesDialog::preferencesDialog(QWidget *parent, CDDBserverInfo *const fsi, Defaults *const pdefault, dbInfo *const autoc,
		enabledTitleColumns *const edopt, reportInfo *const rinfo, colorInfo *const cinfo):
KDialogBase(Tabbed, i18n("Preferences..."), Ok | Cancel | Default, Ok, parent, "name", false, false)
{
	cerr << "preferencesDialog::initialized" << endl;
	// init the pages in the preference dialog.
	title = new titleInfo();
	artist = new artistInfo();
	defs = pdefault;
	info = rinfo;
	color = cinfo;
	en_d_opt = edopt;
	autoconnect = autoc;
	si=fsi;
	initDb();
	initView();
	initArtist();
	initTitle();
	initMisc();
	initReports();
	initCDDB();
	readSettings();
}

/**
 * Destructor
 */
preferencesDialog::~preferencesDialog()
{
	delete artist;
	delete title;
}

/** Sets or unsets the Autocapitalize checkbox.
 * @param bool TRUE checks, FALSE unchecks
 */
void
preferencesDialog::checkAutoCapitalize(bool t) const
{
	if (t)
		autocapitalize->setChecked(TRUE);
	autocapitalize->setEnabled(!t);
}

/** Sets or unsets CDDBP items.
 * @param bool TRUE enables, FALSE disables
 */
void preferencesDialog::checkProxy(bool t) const
{
	if (t)
		httpbutton->setChecked(TRUE);
	cddbpbutton->setEnabled(!t);
	cddbpserver->setEnabled(!t);
	cddbpport->setEnabled(!t);
}

/** Sets up the tab for artist defaults. */
void
preferencesDialog::initArtist()
{
	QFrame *page = addPage(i18n("Artists"));
	QVBoxLayout *layout = new QVBoxLayout(page, 0, spacingHint());

	bgroup = new QButtonGroup(i18n("Default Values"), page);
	grid = new QGridLayout(bgroup, 4, 1, spacingHint());
	grid->setMargin(DIALOG_MARGIN);
	defa = new QRadioButton(i18n("&Specific"), bgroup);
	QWidget *test = new QWidget(bgroup);
	artwidget = new artistWidget(test, spacingHint(), false);
	blanka = new QRadioButton(i18n("&Blank"), bgroup);
	lasta = new QRadioButton(i18n("&Last"), bgroup);
	grid->addWidget(defa, 0, 0);
	grid->addWidget(test, 1, 0);
	grid->addWidget(blanka, 2, 0);
	grid->addWidget(lasta, 3, 0);
	QToolTip::add(defa,i18n("Choose this and fill in the default settings"));
	QToolTip::add(blanka, i18n("Choose this for blank defaults"));
	QToolTip::add(lasta,i18n("Choose this to use the last entered artist"));

	gbox = new QGroupBox(page);
	gbox->setTitle(i18n("Misc Defaults(overrides above)"));
	grid = new QGridLayout(gbox, 3, 1, spacingHint());
	grid->setMargin(DIALOG_MARGIN);
	autofillyear =new QCheckBox(i18n("&Auto Fill Conception With Current Year"), gbox);
	autofillend =new QCheckBox(i18n("A&uto Fill Demise With Conception Year"), gbox);
	defaultfollow = new QCheckBox(i18n("Default &Follow Value"), gbox);
	grid->addWidget(autofillyear, 0, 0);
	grid->addWidget(autofillend, 1, 0);
	grid->addWidget(defaultfollow, 2, 0);
	QToolTip::add(autofillyear,i18n("Fills current year into the conception year"));
	QToolTip::add(autofillend,i18n("Demise mirrors the conception year"));
	QToolTip::add(defaultfollow, i18n("Sets the default value for follow"));

	layout->addWidget(bgroup);
	layout->addWidget(gbox);

	connect(defa, SIGNAL(toggled(bool)), artwidget, SLOT(enable(bool)));
}

/**
 * Sets up the tab for the CDDB/cdparanoia preferences.
 */
void
preferencesDialog::initCDDB()
{
	QFrame *page = addPage(i18n("CDDB"));
	QVBoxLayout *layout = new QVBoxLayout(page, 0, spacingHint());

	// setup the check boxes
	bgroup = new QButtonGroup(i18n("CD Paranoia Settings"), page);
	grid = new QGridLayout(bgroup, 2, 2, spacingHint());
	grid->setMargin(DIALOG_MARGIN);
	label = new QLabel(i18n("Path To cdparanoia"), bgroup);
	grid->addWidget(label, 0, 0);
	cdparanoia = new KLineEdit("", bgroup);
	grid->addWidget(cdparanoia, 0, 1);
	label = new QLabel(i18n("Read Device"), bgroup);
	grid->addWidget(label, 1, 0);
	readdevice = new KLineEdit("", bgroup);
	grid->addWidget(readdevice, 1, 1);
	layout->addWidget(bgroup);

	bgroup = new QButtonGroup(i18n("Local CDDB Settings"), page);
	grid = new QGridLayout(bgroup, 2, 2, spacingHint());
	grid->setMargin(DIALOG_MARGIN);

	cddb_local = new QCheckBox(i18n("Check KSCD For Local CDDB First"), bgroup);
	grid->addMultiCellWidget(cddb_local, 0, 0, 1, 2);
	label=new QLabel(i18n("Local Path"), bgroup);
	grid->addWidget(label, 1, 0);
	localpath=new KLineEdit("", bgroup);
	grid->addWidget(localpath, 1, 1);
	layout->addWidget(bgroup);

	bgroup = new QButtonGroup(i18n("CDDB Server Settings"), page);
	grid = new QGridLayout(bgroup, 7, 4, spacingHint());
	grid->setMargin(DIALOG_MARGIN);

	cddbpbutton = new QRadioButton(i18n("CDDBP Server"), bgroup);
	grid->addWidget(cddbpbutton, 1, 0);
	cddbpserver = new KLineEdit("", bgroup);
	grid->addWidget(cddbpserver, 1, 1);
	label = new QLabel(i18n("Port"), bgroup);
	grid->addWidget(label, 1, 2);
	cddbpport = new KLineEdit("", bgroup);
	cddbpport->setMaxLength(4);
	cddbpport->setMinimumWidth(cddbpport->fontMetrics().width("xxxx") + 20);
	cddbpport->setMaximumWidth(cddbpport->fontMetrics().width("xxxx") + 20);
	grid->addWidget(cddbpport, 1, 3);

	httpbutton = new QRadioButton(i18n("HTTP Server"), bgroup);
	grid->addWidget(httpbutton, 2, 0);
	httpserver = new KLineEdit("", bgroup);
	grid->addWidget(httpserver, 2, 1);
	label = new QLabel(i18n("Port"), bgroup);
	grid->addWidget(label, 2, 2);
	httpport = new KLineEdit("", bgroup);
	httpport->setMaxLength(4);
	httpport->setMinimumWidth(httpport->fontMetrics().width("xxxx") + 20);
	httpport->setMaximumWidth(httpport->fontMetrics().width("xxxx") + 20);
	grid->addWidget(httpport, 2, 3);
	label = new QLabel(i18n("      HTTP CGI Path"), bgroup);
	grid->addWidget(label, 3, 0);
	httpcgi = new KLineEdit("", bgroup);
	grid->addMultiCellWidget(httpcgi, 3, 3, 1, 3);

	proxybutton = new QCheckBox(i18n("Proxy Server"), bgroup);
	grid->addWidget(proxybutton, 4, 0);
	proxyserver = new KLineEdit("", bgroup);
	grid->addWidget(proxyserver, 4, 1);
	label = new QLabel(i18n("Port"), bgroup);
	grid->addWidget(label, 4, 2);
	proxyport = new KLineEdit("", bgroup);
	proxyport->setMaxLength(4);
	proxyport->setMinimumWidth(proxyport->fontMetrics().width("xxxx") + 20);
	proxyport->setMaximumWidth(proxyport->fontMetrics().width("xxxx") + 20);
	grid->addWidget(proxyport, 4, 3);

	label = new QLabel(i18n("SMTP Server"), bgroup);
	grid->addWidget(label, 5, 0);
	smtpserver = new KLineEdit("", bgroup);
	grid->addWidget(smtpserver, 5, 1);
	label = new QLabel(i18n("Port"), bgroup);
	grid->addWidget(label, 5, 2);
	smtpport = new KLineEdit("", bgroup);
	smtpport->setMaxLength(4);
	smtpport->setMinimumWidth(smtpport->fontMetrics().width("xxxx") + 20);
	smtpport->setMaximumWidth(smtpport->fontMetrics().width("xxxx") + 20);
	grid->addWidget(smtpport, 5, 3);

	label = new QLabel(i18n("Submit Entries To "), bgroup);
	grid->addWidget(label, 6, 0);
	cddbemail = new KLineEdit("", bgroup);
	grid->addMultiCellWidget(cddbemail, 6, 6, 1, 3);
	label = new QLabel(i18n("Email Profile"), bgroup);
	grid->addWidget(label, 7, 0);
	emailProfiles = new KComboBox(0, bgroup);
	grid->addMultiCellWidget(emailProfiles, 7, 7, 1, 3);

	layout->addWidget(bgroup);

	QToolTip::add(cdparanoia, i18n("KmusicdB will search path first, then this directory."));
	QToolTip::add(readdevice,i18n("Select which device cdparanoia reads from"));
	QToolTip::add(cddb_local,i18n("Put a check here to check KSCD's local CDDB database."));
	QToolTip::add(localpath,i18n("Select your local CDDB path. Default points to KSCD on your computer."));
	QToolTip::add(cddbpbutton,i18n("Choose this item for a normal CDDB connection."));
	QToolTip::add(cddbpserver, i18n("CDDB Server"));
	QToolTip::add(cddbpport, i18n("CDDB Port"));
	QToolTip::add(httpbutton,i18n("Choose this item for HTTP CDDB (reliable)."));
	QToolTip::add(httpserver,i18n("HTTP Server"));
	QToolTip::add(httpport,i18n("HTTP Port"));
	QToolTip::add(httpcgi,i18n("Path to CGI on HTTP server"));
	QToolTip::add(proxybutton,i18n("HTTP CDDB through a proxy server."));
	QToolTip::add(proxyserver,i18n("Proxy Server"));
	QToolTip::add(proxyport,i18n("Proxy Port"));
	QToolTip::add(smtpserver, i18n("Your Email Server"));
	QToolTip::add(smtpport, i18n("Your Email Port"));
	QToolTip::add(cddbemail, i18n("The CDDB Email Address"));
	QToolTip::add(emailProfiles, i18n("Default Email Profile"));

	connect(proxybutton, SIGNAL(toggled(bool)), this, SLOT(checkProxy(bool)));
}

/** Sets up the tab for database defaults. */
void
preferencesDialog::initDb()
{
	QFrame *page = addPage(i18n("Database"));
	QVBoxLayout *layout = new QVBoxLayout(page, 0, spacingHint());

	// dbinfo
	bgroup = new QButtonGroup(i18n("Default Values"), page);
	grid = new QGridLayout(bgroup, 4, 1, spacingHint());
	grid->setMargin(DIALOG_MARGIN);
	dbvalue = new QRadioButton(i18n("&Specific"), bgroup);
	QToolTip::add(dbvalue, i18n("Use specific values for the database dialog"));
	QWidget *test = new QWidget(bgroup);
	center = new dbConnectWidget(test, spacingHint());
	center->setMargin(DIALOG_MARGIN);
	dbblank = new QRadioButton(i18n("&Blank"), bgroup);
	dbcurrent = new QRadioButton(i18n("&Last"), bgroup);
	QToolTip::add(dbblank, i18n("Use blank values for the database dialog"));
	QToolTip::add(dbcurrent, i18n("Use last entered value for the database dialog"));
	grid->addWidget(dbvalue, 0, 0);
	grid->addWidget(test, 1, 0);
	grid->addWidget(dbblank, 2, 0);
	grid->addWidget(dbcurrent, 3, 0);
	layout->addWidget(bgroup);

	// autoconnect?
	gbox = new QGroupBox(i18n("Auto Connect"), page);
	grid = new QGridLayout(gbox, 2, 1, spacingHint());
	grid->setMargin(DIALOG_MARGIN);
	autob = new QCheckBox(i18n("&Auto Connect on Startup"), gbox);
	QToolTip::add(autob,i18n("Check here if you want to automatically connect to the default database on startup"));
	test = new QWidget(gbox);
	autodb = new dbConnectWidget(test, spacingHint());
	grid->addWidget(autob, 0, 0);
	grid->addWidget(test, 1, 0);
	layout->addWidget(gbox);

	// postgres version
	gbox = new QGroupBox(i18n("Information"), page);
	grid = new QGridLayout(gbox, 1, 2, spacingHint());
	grid->setMargin(DIALOG_MARGIN);
	label = new QLabel(i18n("Version Info: "), gbox);
	label->setFixedHeight(label->sizeHint().height());
	grid->addWidget(label, 0, 0);

	vlabel = new QLabel(i18n("Not Connected"), gbox);
	grid->addWidget(vlabel, 0, 1);
	layout->addWidget(gbox);

	connect(dbvalue, SIGNAL(toggled(bool)), center, SLOT(enable(bool)));
	connect(autob, SIGNAL(toggled(bool)), autodb, SLOT(enable(bool)));
}

/** Sets up the Misc tab. */
void
preferencesDialog::initMisc()
{
	QFrame *page = addPage(i18n("Misc"));
	QVBoxLayout *layout = new QVBoxLayout(page, 0, spacingHint());

	// dbinfo
	bgroup = new QButtonGroup(i18n("Category Sort Order"), page);

	grid = new QGridLayout(bgroup, 1, 2, spacingHint());
	grid->setMargin(DIALOG_MARGIN);

	catasc = new QRadioButton(i18n("&Ascending"), bgroup);
	catdes = new QRadioButton(i18n("&Descending"), bgroup);
	grid->addWidget(catasc, 0, 0);
	grid->addWidget(catdes, 0, 1);
	QToolTip::add(catasc, i18n("Sort Ascending"));
	QToolTip::add(catdes, i18n("Sort Descending"));
	layout->add(bgroup);

	bgroup = new QButtonGroup(i18n("Text Formatting"), page);
	grid = new QGridLayout(bgroup, 2, 1, spacingHint());
	grid->setMargin(DIALOG_MARGIN);
	autocapitalize = new QCheckBox(i18n("Simple &Capitilization"), bgroup);
	autoformat = new QCheckBox(i18n("Complete &Formatting"), bgroup);
	grid->addWidget(autocapitalize, 0, 0);
	grid->addWidget(autoformat, 1, 0);
	QToolTip::add(autocapitalize,i18n("Check to automatically capitalize the first letter of every word."));
	QToolTip::add(autoformat,i18n("Check to more correctly capitilize words and lower case everything else."));
	layout->add(bgroup);

	bgroup = new QButtonGroup(i18n("Dialog Settings"), page);
	grid = new QGridLayout(bgroup, 1, 1, spacingHint());
	grid->setMargin(DIALOG_MARGIN);
	autopopup = new QCheckBox(i18n("Automatically Open Title on CDDB or Drag and Drop"), bgroup);
	grid->addWidget(autopopup, 0, 0);
	QToolTip::add(autopopup,i18n("Check if you want the title dialog to automatically open when a CDDB title is added or a title is dropped."));
	layout->addWidget(bgroup);

	connect(autoformat, SIGNAL(toggled(bool)), this, SLOT(checkAutoCapitalize(bool)));
}

/**
 * Sets up the tab for the reports preferences.
 */
void
preferencesDialog::initReports()
{
	QFrame *page = addPage(i18n("Reports"));

	// setup the check boxes
	report = new reportWidget(page, spacingHint());
}

/** Sets up the tab for title defaults. */
void
preferencesDialog::initTitle()
{
	QFrame *page = addPage(i18n("Titles"));
	QVBoxLayout *layout = new QVBoxLayout(page, 0, spacingHint());

	bgroup = new QButtonGroup(i18n("Default Values"), page);
	grid = new QGridLayout(bgroup, 4, 1, spacingHint());
	grid->setMargin(DIALOG_MARGIN);
	deft = new QRadioButton(i18n("&Specific"), bgroup);
	QWidget *test = new QWidget(bgroup);
	titlwidget = new titleWidget(test, spacingHint(), false);
	blankt = new QRadioButton(i18n("&Blank"), bgroup);
	lastt = new QRadioButton(i18n("&Last"), bgroup);
	grid->addWidget(deft, 0, 0);
	grid->addWidget(test, 1, 0);
	grid->addWidget(blankt, 2, 0);
	grid->addWidget(lastt, 3, 0);
	QToolTip::add(deft,i18n("Choose this and fill in the default settings"));
	QToolTip::add(blankt, i18n("Choose this for blank defaults"));
	QToolTip::add(lastt,i18n("Choose this to use the last entered title"));

	gbox = new QGroupBox(i18n("Misc Defaults(overrides above)"), page);
	grid = new QGridLayout(gbox, 3, 1, spacingHint());
	grid->setMargin(DIALOG_MARGIN);
	autofillyeart =new QCheckBox(i18n("&Auto Fill Year Start With Current Year"), gbox);
	autofillendt =new QCheckBox(i18n("A&uto Fill Year End With Year Start"), gbox);
	defaultcollect = new QCheckBox(i18n("Default Coll&ected Value"), gbox);
	grid->addWidget(autofillyeart, 0, 0);
	grid->addWidget(autofillendt, 1, 0);
	grid->addWidget(defaultcollect, 2, 0);
	QToolTip::add(autofillyeart,i18n("Fills current year into the begin year"));
	QToolTip::add(autofillendt, i18n("End year mirrors the begin year"));
	QToolTip::add(defaultcollect,i18n("Sets the default value for follow"));

	layout->addWidget(bgroup);
	layout->addWidget(gbox);

	connect(deft, SIGNAL(toggled(bool)), titlwidget, SLOT(enable(bool)));
}

/** Sets up the tab for column sorting and title view defaults. */
void
preferencesDialog::initView()
{
	QFrame *page = addPage(i18n("Browser"));
	QVBoxLayout *layout = new QVBoxLayout(page, 0, spacingHint());

	// setup the check boxes
	bgroup = new QButtonGroup(i18n("Display Columns"), page);
	layout->addWidget(bgroup);

	grid = new QGridLayout(bgroup, 5, 3, spacingHint());
	grid->setMargin(DIALOG_MARGIN);
	vfid = new QCheckBox(i18n("&Format"), bgroup);
	vgid = new QCheckBox(i18n("&Genre"), bgroup);
	vlid = new QCheckBox(i18n("&Label"), bgroup);
	vmid = new QCheckBox(i18n("Met&hod"), bgroup);
	vtid = new QCheckBox(i18n("Ty&pe"), bgroup);
	vyearstart = new QCheckBox(i18n("&Year"), bgroup);
	vyearend = new QCheckBox(i18n("Ye&ar End"), bgroup);
	vnod = new QCheckBox(i18n("No. of &Media"), bgroup);
	vnotracks = new QCheckBox(i18n("&No. of Tracks"), bgroup);
	vlength = new QCheckBox(i18n("Leng&th"), bgroup);
	vrating = new QCheckBox(i18n("&Rating"), bgroup);
	vquality = new QCheckBox(i18n("&Quality"), bgroup);
	vcode = new QCheckBox(i18n("Othe&r"), bgroup);
	vloan = new QCheckBox(i18n("On &Loan?"), bgroup);
	vcollected = new QCheckBox(i18n("Coll&ected"), bgroup);

	QToolTip::add(vfid, i18n("Check To Display"));
	QToolTip::add(vgid, i18n("Check To Display"));
	QToolTip::add(vlid, i18n("Check To Display"));
	QToolTip::add(vmid, i18n("Check To Display"));
	QToolTip::add(vtid, i18n("Check To Display"));
	QToolTip::add(vyearstart, i18n("Check To Display"));
	QToolTip::add(vyearend, i18n("Check To Display"));
	QToolTip::add(vnod, i18n("Check To Display"));
	QToolTip::add(vnotracks, i18n("Check To Display"));
	QToolTip::add(vlength, i18n("Check To Display"));
	QToolTip::add(vrating, i18n("Check To Display"));
	QToolTip::add(vquality, i18n("Check To Display"));
	QToolTip::add(vcollected, i18n("Check To Display"));
	QToolTip::add(vcode, i18n("Check To Display"));
	QToolTip::add(vloan, i18n("Check To Display"));

	grid->addWidget(vfid, 0, 0);
	grid->addWidget(vgid, 1, 0);
	grid->addWidget(vlid, 2, 0);
	grid->addWidget(vmid, 3, 0);
	grid->addWidget(vtid, 4, 0);
	grid->addWidget(vyearstart, 0, 1);
	grid->addWidget(vyearend, 1, 1);
	grid->addWidget(vnod, 2, 1);
	grid->addWidget(vnotracks, 3, 1);
	grid->addWidget(vlength, 4, 1);
	grid->addWidget(vrating, 0, 2);
	grid->addWidget(vquality, 1, 2);
	grid->addWidget(vcode, 2, 2);
	grid->addWidget(vloan, 3, 2);
	grid->addWidget(vcollected, 4, 2);

	// default column sort in main widget
	bgroup = new QButtonGroup(i18n("Sorting"), page);
	layout->addWidget(bgroup);

	grid = new QGridLayout(bgroup, 3, 1, spacingHint());
	grid->setMargin(DIALOG_MARGIN);
	sbox = new KComboBox(0, bgroup);
	sbox->insertItem(i18n("Title"));
	sbox->insertItem(i18n("Format"));
	sbox->insertItem(i18n("Genre"));
	sbox->insertItem(i18n("Label"));
	sbox->insertItem(i18n("Method"));
	sbox->insertItem(i18n("Type"));
	sbox->insertItem(i18n("Year"));
	sbox->insertItem(i18n("Year End"));
	sbox->insertItem(i18n("Media"));
	sbox->insertItem(i18n("Tracks"));
	sbox->insertItem(i18n("Length"));
	sbox->insertItem(i18n("Rating"));
	sbox->insertItem(i18n("Quality"));
	sbox->insertItem(i18n("Other"));
	sbox->insertItem(i18n("On Loan?"));
	sbox->insertItem(i18n("Collected"));
	sbox->insertItem(i18n("CDID"));
	grid->addWidget(sbox, 0, 0);
	QToolTip::add(sbox, i18n("Select A Column To Sort By"));

	asc = new QRadioButton(i18n("Ascending"), bgroup);
	des = new QRadioButton(i18n("Descending"), bgroup);
	grid->addWidget(asc, 1, 0);
	grid->addWidget(des, 2, 0);
	QToolTip::add(asc, i18n("Sort Ascending"));
	QToolTip::add(des, i18n("Sort Descending"));

}

/**
 *Passes the information through to titleWidget.
 * @param int Genre, Label, etc.
 * @param QStringList Pointer to list to be filled
 */
void
preferencesDialog::populateCategoryBoxPT(const int &t, QStringList *l) const
{
	titlwidget->populateCategoryBox(t, l);
}

/** Reads the settings from the config file to local variables. */
void
preferencesDialog::readSettings()
{
	cerr << "preferencesDialog::readSettings" << endl;
	KConfig *kc=kapp->config();
	center->enable(FALSE);
	autodb->enable(FALSE);

	kc->setGroup(i18n("postgres"));
	autob->setChecked(defs->auto_connect);

	int z = defs->dbdata;
	if (z == STATIC_DEFAULT)
		dbvalue->setChecked(TRUE);
	else if (z == BLANK)
		dbblank->setChecked(TRUE);
	else
		dbcurrent->setChecked(TRUE);
	dbInfo t;
	t.db_name=kc->readEntry(i18n("name"), DEFAULT_DB);
	t.db_location=kc->readEntry(i18n("location"), DEFAULT_LOCATION);
	t.db_port=kc->readEntry(i18n("port"), DEFAULT_SERVER_PORT);
	t.db_user=kc->readEntry(i18n("user"), DEFAULT_SERVER_USER);
	t.db_passwd=kc->readEntry(i18n("passwd"), DEFAULT_SERVER_PASSWD);
	center->updateInfo(&t);

	autodb->updateInfo(autoconnect);

	sbox->setCurrentItem(defs->default_column_sort);
	if (defs->default_sort_order)
		asc->setChecked(TRUE);
	else
		des->setChecked(TRUE);

	// settings for artist and titles default values
	kc->setGroup(i18n("artist"));
	// 0 for text, 1 for blank, 2 for last entry
	z = defs->artistdata;
	artwidget->enable(FALSE);
	if (z == STATIC_DEFAULT)
		defa->setChecked(TRUE);
	else if (z == BLANK)
		blanka->setChecked(TRUE);
	else
		lasta->setChecked(TRUE);
	// fill artist struct
	artist->aid = kc->readEntry(i18n("aid"), "");
	artist->notes = kc->readEntry(i18n("notes"), "");
	artist->first = kc->readEntry(i18n("first"), "");
	artist->last = kc->readEntry(i18n("last"), "");
	artist->name = kc->readEntry(i18n("name"), "");
	artist->follow = kc->readBoolEntry(i18n("follow"), FALSE);
	artist->homepage = kc->readEntry(i18n("homepage"), "http://");

	// fill artist defaults
	autofillyear->setChecked(defs->artist_year);
	autofillend->setChecked(defs->artist_autofill);
	defaultfollow->setChecked(defs->artist_follow);

	artwidget->updateInfo(artist,false,false);

	// fill title struct
	kc->setGroup(i18n("title"));
	// 0 for text, 1 for blank, 2 for last entry
	z = defs->titledata;
	titlwidget->enable(FALSE);
	if (z == STATIC_DEFAULT)
		deft->setChecked(TRUE);
	else if (z == BLANK)
		blankt->setChecked(TRUE);
	else
		lastt->setChecked(TRUE);
	// fill artist struct
	title->cdid = kc->readEntry(i18n("cdid"), "");
	title->cat[KFORMAT].name = kc->readEntry(i18n("fid"), "");
	title->cat[KGENRE].name = kc->readEntry(i18n("gid"), "");
	title->cat[KBOXSET].name = kc->readEntry(i18n("grpid"), "");
	title->cat[KLABEL].name = kc->readEntry(i18n("lid"), "");
	title->cat[KMETHOD].name = kc->readEntry(i18n("mid"), "");
	title->cat[KTYPE].name = kc->readEntry(i18n("tid"), "");
	title->length = kc->readEntry(i18n("length"), "00:00:00");
	title->name = kc->readEntry(i18n("name"), "");
	title->nod = kc->readEntry(i18n("nod"), "");
	title->notes = kc->readEntry(i18n("notes"), "");
	title->notracks = kc->readEntry(i18n("notracks"), "");
	title->yearend = kc->readEntry(i18n("yearend"), "");
	title->yearstart = kc->readEntry(i18n("yearstart"), "");
	title->collected = kc->readBoolEntry(i18n("collected"), FALSE);
	title->rating = kc->readEntry(i18n("rating"), "");
	title->quality = kc->readEntry(i18n("quality"), "");
	title->code = kc->readEntry(i18n("code"), "");
	title->loan = kc->readBoolEntry(i18n("loan"), FALSE);

	// fill title defaults
	autofillyeart->setChecked(defs->title_year);
	autofillendt->setChecked(defs->title_autofill);
	defaultcollect->setChecked(defs->title_collect);

	titlwidget->updateInfo(title,false,false);

	// read the default columns
	vfid->setChecked(en_d_opt->fid);
	vgid->setChecked(en_d_opt->gid);
	vlid->setChecked(en_d_opt->lid);
	vmid->setChecked(en_d_opt->mid);
	vtid->setChecked(en_d_opt->tid);
	vyearend->setChecked(en_d_opt->yearend);
	vnod->setChecked(en_d_opt->nod);
	vyearstart->setChecked(en_d_opt->yearstart);
	vnotracks->setChecked(en_d_opt->notracks);
	vlength->setChecked(en_d_opt->length);
	vcollected->setChecked(en_d_opt->collected);
	vrating->setChecked(en_d_opt->rating);
	vquality->setChecked(en_d_opt->quality);
	vcode->setChecked(en_d_opt->code);
	vloan->setChecked(en_d_opt->loan);

	if (defs->category_sort_order)
		catasc->setChecked(TRUE);
	else
		catdes->setChecked(TRUE);

	autocapitalize->setChecked(defs->auto_capitalize);
	autoformat->setChecked(defs->auto_format);
	if (autoformat->isChecked())
		autocapitalize->setEnabled(FALSE);

	// report settings and color settings are already set when the widget is created.
	reportProvides rp;
	rp.orderby=TRUE;
	rp.collect=TRUE;
	rp.limit=TRUE;
	rp.range=TRUE;
	rp.font=TRUE;
	rp.color=TRUE;
	report->updateInfo(info, color, &rp);

	// set the CDDB settings
	cddbpserver->setText(si->cddbp_server);
	cddbpport->setText(si->cddbp_port);
	httpserver->setText(si->http_server);
	httpport->setText(si->http_port);
	httpcgi->setText(si->http_cgi);
	proxyserver->setText(si->proxy_server);
	proxyport->setText(si->proxy_port);
	if (si->get_protocol==CDDB_PROTO_PROXY)
		proxybutton->setChecked(TRUE);
	else if (si->get_protocol==CDDB_PROTO_HTTP)
		httpbutton->setChecked(TRUE);
	else
		cddbpbutton->setChecked(TRUE);
	cdparanoia->setText(si->cdparanoia_location);
	readdevice->setText(si->read_device);
	autopopup->setChecked(defs->cddb_popup);
	cddbemail->setText(si->cddb_email);
	KEMailSettings *email = new KEMailSettings();
	if (!email)
		exit(1);
	QStringList temp2 = email->profiles();
	QStringList::Iterator it;
	int i = 0;
	for (it = temp2.begin(); it != temp2.end(); it++)
	{
		emailProfiles->insertItem(*it);
		if ((*it) == si->kde_email_profile)
		emailProfiles->setCurrentItem(i);
		i++;
	}
	delete email;
	smtpserver->setText(si->smtp_server);
	smtpport->setText(si->smtp_port);
	cddb_local->setChecked(si->check_cddb_local);
	localpath->setText(si->local_cddb_path);

}

/** Sets the Postgres version variable.
 * @param QString Version number/info
 */
void
preferencesDialog::setVersion(const QString &x)
{
	QString version = x;
	int index = 0;
	int flag = 0;
	do
	{
		index = version.find(' ', index + 1);
		flag++;
		if (flag == 3 && index != -1)
		{
			version.replace(index, 1, "\n");
			flag = 0;
		}
	}
	while (index != -1);
	vlabel->setText(version);
}

/** Closes the dialog. */
void
preferencesDialog::slotCancel()
{
	emit cancelled(this);
	delayedDestruct();
}

/** Sets current tab back to default settings. */
void
preferencesDialog::slotDefault()
{
	QColor temp;
	dbInfo t;
	switch(activePageIndex())
	{
		case 0:
			autob->setChecked(AUTO_CONNECT_DB);
			t.db_name=DEFAULT_DB;
			t.db_location=DEFAULT_LOCATION;
			t.db_port=DEFAULT_SERVER_PORT;
			t.db_user=DEFAULT_SERVER_USER;
			t.db_passwd=DEFAULT_SERVER_PASSWD;
			center->updateInfo(&t);
			break;
		case 1:
			vcollected->setChecked(TRUE);
			vlength->setChecked(TRUE);
			vnotracks->setChecked(TRUE);
			vfid->setChecked(FALSE);
			vgid->setChecked(FALSE);
			vlid->setChecked(FALSE);
			vmid->setChecked(FALSE);
			vnod->setChecked(FALSE);
			vtid->setChecked(FALSE);
			vyearend->setChecked(FALSE);
			vyearstart->setChecked(TRUE);
			vrating->setChecked(FALSE);
			vquality->setChecked(FALSE);
			vcode->setChecked(FALSE);
			vloan->setChecked(FALSE);
			sbox->setCurrentItem(DEFAULT_COLUMN);
			asc->setChecked(TRUE);
			break;
		case 2:
			blanka->setChecked(TRUE);
			defaultfollow->setChecked(DEFAULT_FOLLOW);
			autofillyear->setChecked(DEFAULT_AUTO_FILL);
			autofillend->setChecked(DEFAULT_YEAR);
			break;
		case 3:
			blankt->setChecked(TRUE);
			defaultcollect->setChecked(DEFAULT_COLLECT);
			autofillyeart->setChecked(DEFAULT_AUTO_FILL);
			autofillendt->setChecked(DEFAULT_YEAR);
			break;
		case 4:
			catasc->setChecked(DEFAULT_COLUMN_SORT_ORDER);
			autocapitalize->setChecked(DEFAULT_CAPITALIZE);
			autoformat->setChecked(DEFAULT_PROCESS_TEXT);
			cddb_local->setChecked(DEFAULT_CDDB_CHECK_LOCAL);
			break;
		case 5:
			temp.setNamedColor(DEFAULT_MISC_COLOR);
			report->setColor(COLOR_MISC, temp);
			temp.setNamedColor(DEFAULT_ARTIST_COLOR);
			report->setColor(COLOR_ARTIST, temp);
			temp.setNamedColor(DEFAULT_TITLE_COLOR);
			report->setColor(COLOR_TITLE, temp);
			temp.setNamedColor(DEFAULT_TRACK_COLOR);
			report->setColor(COLOR_TRACK, temp);
			temp.setNamedColor(DEFAULT_TOTALS_COLOR);
			report->setColor(COLOR_TOTAL, temp);
			temp.setNamedColor(DEFAULT_BG_COLOR);
			report->setColor(COLOR_BACKGROUND, temp);
			temp.setNamedColor(DEFAULT_TEXT_COLOR);
			report->setColor(COLOR_TEXT, temp);

			report->setDateRangeEnabled(FALSE);
			report->setReportType(REPORT_ALL);
			report->setSortOrder(ASC);
			report->setYearSort(TRUE);
			report->setYearStart("0");
			report->setYearEnd("9999");
			report->setLimitEnabled(FALSE);
			report->setLimit("15");
			break;
		case 6:
			autopopup->setChecked(DEFAULT_CDDB_TITLE_OPEN);
			proxybutton->setChecked(FALSE);
			cddbpbutton->setChecked(TRUE);
			cdparanoia->setText(DEFAULT_CDPARANOIA_LOCATION);
			cddbpserver->setText(DEFAULT_CDDBP_SERVER);
			cddbpport->setText(DEFAULT_CDDBP_PORT);
			httpserver->setText(DEFAULT_HTTP_SERVER);
			httpport->setText(DEFAULT_HTTP_PORT);
			httpcgi->setText(DEFAULT_HTTP_CGI);
			proxyserver->setText(DEFAULT_PROXY_SERVER);
			proxyport->setText(DEFAULT_PROXY_PORT);
			readdevice->setText(DEFAULT_READ_DEVICE);
			cddbemail->setText(DEFAULT_CDDB_EMAIL);
			smtpserver->setText(DEFAULT_SMTP_SERVER);
			smtpport->setText(DEFAULT_SMTP_PORT);
			for (int i = 0; i < emailProfiles->count(); i++)
			{
				if (emailProfiles->text(i) == DEFAULT_EMAIL_PROFILE)
				{
					emailProfiles->setCurrentItem(i);
					break;
				}
			}
			localpath->setText(KGlobal::dirs()->findDirs("data", "kscd/cddb").first());
			break;
	};

}

/**
 * Sets the settings, closes the dialog.
 * Emits okClicked and sortsignal(int, bool).
 */
void
preferencesDialog::slotOk()
{
	hide();
	writeSettings();
	emit okClicked();
	emit sortCategorySignal(catasc->isChecked());

	emit cancelled(this);
	delayedDestruct();
}

/** Sets the KComboBox values. */
void
preferencesDialog::updatetitleInfo() const
{
	titlwidget->updateInfo(title,false,false);
}

/** Writes settings to config file. */
void
preferencesDialog::writeSettings()
{
	cerr << "preferencesDialog::writeSettings" << endl;
	KConfig *kc=kapp->config();
	kc->setGroup(i18n("postgres"));
	int z = BLANK;
	if (dbvalue->isChecked())
		z = STATIC_DEFAULT;
	else if (dbblank->isChecked())
		z = BLANK;
	else
		z = LAST_VALUE;
	kc->writeEntry(i18n("data"), z);
	kc->writeEntry(i18n("auto_connect"), autob->isChecked());
	kc->writeEntry(i18n("name"), center->getName());
	kc->writeEntry(i18n("location"), center->getLocation());
	kc->writeEntry(i18n("port"), center->getPort());
	kc->writeEntry(i18n("user"), center->getUser());
	kc->writeEntry(i18n("passwd"), center->getPasswd());

	kc->setGroup(i18n("auto_connect"));
	kc->writeEntry(i18n("name"), autodb->getName());
	kc->writeEntry(i18n("location"), autodb->getLocation());
	kc->writeEntry(i18n("port"), autodb->getPort());
	kc->writeEntry(i18n("user"), autodb->getUser());
	kc->writeEntry(i18n("passwd"), autodb->getPasswd());

	kc->setGroup(i18n("mainwidget"));
	kc->writeEntry(i18n("default_column_sort"), sbox->currentItem());
	kc->writeEntry(i18n("default_sort_order"), asc->isChecked());

	// settings for artist and titles default values
	kc->setGroup(i18n("artist"));
	// 0 for text, 1 for blank, 2 for last entry
	if (defa->isChecked())
		z = STATIC_DEFAULT;
	else if (blanka->isChecked())
		z = BLANK;
	else
		z = LAST_VALUE;

	kc->writeEntry(i18n("data"), z);
	// fill artist struct
	kc->writeEntry(i18n("aid"), artwidget->getAid());
	kc->writeEntry(i18n("notes"), artwidget->getNotes());
	kc->writeEntry(i18n("first"), artwidget->getFirst());
	kc->writeEntry(i18n("last"), artwidget->getLast());
	kc->writeEntry(i18n("name"), artwidget->getName());
	kc->writeEntry(i18n("follow"), artwidget->getFollow());
	kc->writeEntry(i18n("homepage"), artwidget->getHomepage());

	// fill artist defaults
	kc->writeEntry(i18n("year"), autofillyear->isChecked());
	kc->writeEntry(i18n("autofill"), autofillend->isChecked());
	kc->writeEntry(i18n("follow2"), defaultfollow->isChecked());

	// fill title struct
	kc->setGroup(i18n("title"));
	// 0 for text, 1 for blank, 2 for last entry
	if (deft->isChecked())
		z = STATIC_DEFAULT;
	else if (blankt->isChecked())
		z = BLANK;
	else
		z = LAST_VALUE;
	kc->writeEntry(i18n("data"), z);
	// fill artist struct
	kc->writeEntry(i18n("cdid"), titlwidget->getCDID());
	kc->writeEntry(i18n("fid"), titlwidget->getFormat());
	kc->writeEntry(i18n("gid"), titlwidget->getGenre());
	kc->writeEntry(i18n("grpid"), titlwidget->getBoxset());
	kc->writeEntry(i18n("lid"), titlwidget->getLabel());
	kc->writeEntry(i18n("mid"), titlwidget->getMethod());
	kc->writeEntry(i18n("tid"), titlwidget->getType());
	kc->writeEntry(i18n("length"), titlwidget->getLength());
	kc->writeEntry(i18n("name"), titlwidget->getName());
	kc->writeEntry(i18n("nod"), titlwidget->getNod());
	kc->writeEntry(i18n("notes"), titlwidget->getNotes());
	kc->writeEntry(i18n("notracks"), titlwidget->getNotracks());
	kc->writeEntry(i18n("yearend"), titlwidget->getYearend());
	kc->writeEntry(i18n("yearstart"), titlwidget->getYearstart());
	kc->writeEntry(i18n("collected"), titlwidget->getCollected());
	kc->writeEntry(i18n("rating"), titlwidget->getRating());
	kc->writeEntry(i18n("quality"), titlwidget->getQuality());
	kc->writeEntry(i18n("code"), titlwidget->getCode());
	kc->writeEntry(i18n("loan"), titlwidget->getLoan());

	// fill title defaults
	kc->writeEntry(i18n("year"), autofillyeart->isChecked());
	kc->writeEntry(i18n("autofill"), autofillendt->isChecked());
	kc->writeEntry(i18n("collected2"), defaultcollect->isChecked());

	// read the default columns
	kc->setGroup(i18n("columns"));
	kc->writeEntry(i18n("fid"), vfid->isChecked());
	kc->writeEntry(i18n("gid"), vgid->isChecked());
	kc->writeEntry(i18n("lid"), vlid->isChecked());
	kc->writeEntry(i18n("mid"), vmid->isChecked());
	kc->writeEntry(i18n("tid"), vtid->isChecked());
	kc->writeEntry(i18n("yearend"), vyearend->isChecked());
	kc->writeEntry(i18n("nod"), vnod->isChecked());
	kc->writeEntry(i18n("yearstart"), vyearstart->isChecked());
	kc->writeEntry(i18n("notracks"), vnotracks->isChecked());
	kc->writeEntry(i18n("length"), vlength->isChecked());
	kc->writeEntry(i18n("collected"), vcollected->isChecked());
	kc->writeEntry(i18n("rating"), vrating->isChecked());
	kc->writeEntry(i18n("quality"), vquality->isChecked());
	kc->writeEntry(i18n("code"), vcode->isChecked());
	kc->writeEntry(i18n("loan"), vloan->isChecked());

	kc->setGroup(i18n("category"));
	kc->writeEntry(i18n("category_sort_order"), catasc->isChecked());

	kc->setGroup(i18n("text_processing"));
	kc->writeEntry(i18n("auto_capitalize"), autocapitalize->isChecked());
	kc->writeEntry(i18n("auto_format"), autoformat->isChecked());

	// report options and report colors
	kc->setGroup(i18n("reports"));
	kc->writeEntry(i18n("misc"), report->getColor(COLOR_MISC).name());
	kc->writeEntry(i18n("artist"), report->getColor(COLOR_ARTIST).name());
	kc->writeEntry(i18n("title"), report->getColor(COLOR_TITLE).name());
	kc->writeEntry(i18n("track"), report->getColor(COLOR_TRACK).name());
	kc->writeEntry(i18n("totals"), report->getColor(COLOR_TOTAL).name());
	kc->writeEntry(i18n("background"), report->getColor(COLOR_BACKGROUND).name());
	kc->writeEntry(i18n("text"), report->getColor(COLOR_TEXT).name());

	kc->writeEntry(i18n("sort_order"), report->getSortOrder());
	kc->writeEntry(i18n("sort_time"), report->getYearSort());
	kc->writeEntry(i18n("date_range_enabled"), report->getDateRangeEnabled());
	kc->writeEntry(i18n("limit_enabled"), report->getLimitEnabled());
	kc->writeEntry(i18n("report_type"), report->getReportType());
	kc->writeEntry(i18n("start"), report->getYearStart());
	kc->writeEntry(i18n("end"), report->getYearEnd());
	kc->writeEntry(i18n("limit"), report->getLimit());

	kc->writeEntry(i18n("rfont_family"), report->getFont().family());
	kc->writeEntry(i18n("rfont_weight"), report->getFont().bold());
	kc->writeEntry(i18n("rfont_style"), report->getFont().italic());
	kc->writeEntry(i18n("rfont_size"), report->getFont().pointSize());

	kc->setGroup(i18n("cddb"));
	kc->writeEntry(i18n("cddbp_server"), cddbpserver->text());
	kc->writeEntry(i18n("cddbp_port"), cddbpport->text());
	kc->writeEntry(i18n("http_server"), httpserver->text());
	kc->writeEntry(i18n("http_port"), httpport->text());
	kc->writeEntry(i18n("proxy_server"), proxyserver->text());
	kc->writeEntry(i18n("proxy_port"), proxyport->text());
	kc->writeEntry(i18n("http_cgi"), httpcgi->text());
	kc->writeEntry(i18n("cdparanoia_location"), cdparanoia->text());
	kc->writeEntry(i18n("read_device"), readdevice->text());
	kc->writeEntry(i18n("cddb_popup"), autopopup->isChecked());
	kc->writeEntry(i18n("cddb_email"), cddbemail->text());
	kc->writeEntry(i18n("kde_email_profile"), emailProfiles->currentText());
	kc->writeEntry(i18n("smtp_server"), smtpserver->text());
	kc->writeEntry(i18n("smtp_port"), smtpport->text());
	kc->writeEntry(i18n("check_cddb_local"), cddb_local->isChecked());
	kc->writeEntry(i18n("local_cddb_path"), localpath->text());
	if (proxybutton->isChecked())
		kc->writeEntry(i18n("get_protocol"), CDDB_PROTO_PROXY);
	else if (httpbutton->isChecked())
		kc->writeEntry(i18n("get_protocol"), CDDB_PROTO_HTTP);
	else
		kc->writeEntry(i18n("get_protocol"), CDDB_PROTO_CDDBP);
	kc->writeEntry(i18n("put_protocol"), CDDB_PROTO_EMAIL);

	kc->sync();
}

#include "preferencesdialog.moc"
