/***************************************************************************
                          preferencesdialog.h  -  description
                             -------------------
    begin                : Fri Mar 30 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PREFERENCESDIALOG_H
#define PREFERENCESDIALOG_H

#include "include.h"
#include "dbconnectwidget.h"
#include "artistwidget.h"
#include "titlewidget.h"
#include "reportwidget.h"

/**This is the preferences dialog.
 * This is the primary interface for changing the default settings/behaviour of the program.
 *
 * @short Preferences dialog
 * @author Will DeRousse
 *<pre>
 *preferencesDialog *pref=new preferencesDialog(parent, def, dbinfo, edo, rinfo, cinfo);
 *pref->show();
 *</pre>
 */

class preferencesDialog : public KDialogBase  {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 * @param CDDBserverInfo Pointer to CDDBserverInfo
		 * @param Defaults Pointer To Defaults
		 * @param dbInfo Pointer To Current dbInfo
		 * @param enabledTitleColumns Pointer To enabledTitleColumns (for mainview columns)
		 * @param reportInfo Pointer To reportInfo
		 * @param colorInfo Pointer To colorInfo (for reports)
		 */
		preferencesDialog(QWidget *, CDDBserverInfo *const, Defaults *const, dbInfo *const, enabledTitleColumns *const, reportInfo *const, colorInfo *const);
		/**
		 * Destructor
		 */
		~preferencesDialog();
		/**
		 *Passes the information through to titleWidget.
		 * @param int Genre, Label, etc.
		 * @param QStringList Pointer to QStringList to be filled
		 */
		void populateCategoryBoxPT(const int &, QStringList *) const;
		/** Sets the Postgres version variable.
		 * @param QString Version Number/Info
		 */
		void setVersion(const QString &);
		/**
		 * Sets the KComboBox values.
		 */
		void updatetitleInfo() const;
	protected slots:
		/**
		 * Closes the dialog.
		 */
		void slotCancel();
		/**
		 * Sets current tab back to default settings.
		 */
		void slotDefault();
		/**
		 * Sets the settings, closes the dialog.
		 * Emits okClicked and sortsignal(int, bool).
		 */
		void slotOk();
	private slots:
		/** Sets or unsets the Autocapitalize checkbox.
		 * @param bool TRUE checks, FALSE unchecks
		 */
		void checkAutoCapitalize(bool) const;
		/** Sets or unsets CDDBP items.
		 * @param bool TRUE enables, FALSE disables
		 */
		void checkProxy(bool) const;
	private:
		/**
		 * Sets up the tab for artist defaults.
		 */
		void initArtist();
		/**
		 * Sets up the tab for the CDDB/cdparanoia preferences.
		 */
		void initCDDB();
		/**
		 * Sets up the tab for database defaults.
		 */
		void initDb();
		/**
		 * Sets up the Misc tab.
		 */
		void initMisc();
		/**
		 * Sets up the tab for the reports preferences.
		 */
		void initReports();
		/**
		 * Sets up the tab for title defaults.
		 */
		void initTitle();
		/**
		 * Sets up the tab for column sorting and title view defaults.
		 */
		void initView();
		/**
		 * Reads the settings from the config file to local variables.
		 */
		void readSettings();
		/**
		 * Writes settings to config file.
		 */
		void writeSettings();
	signals:
		/**
		 * Signal emitted when cancel is clicked for memory management purposes.
		 * @param preferencesDialog Pointer to this dialog
		 */
		void cancelled(preferencesDialog*);
		/**
		 * Emitted on slotOk() so the open category boxes will be sorted.
		 * @param bool TRUE is ascending, FALSE is descending
		 */
		void sortCategorySignal(bool);
	private:
		// "global"
		int pageIndex;
		QLabel *label;
		QGroupBox *gbox;
		QButtonGroup *bgroup;
		QGridLayout *grid;
		artistInfo *artist;
		titleInfo *title;
		Defaults *defs;
		dbInfo *autoconnect;
		enabledTitleColumns *en_d_opt;
		CDDBserverInfo *si;
		// initDb()
		dbConnectWidget *center;
		dbConnectWidget *autodb;
		QCheckBox *autob;
		QLabel *vlabel;
		QRadioButton *dbvalue, *dbblank, *dbcurrent;
		// initView()
		QCheckBox *vcollected, *vfid, *vgid, *vlength, *vlid;
		QCheckBox *vmid, *vnod, *vnotracks, *vtid, *vyearend, *vyearstart, *vrating, *vquality, *vcode, *vloan;
		KComboBox *sbox;
		QRadioButton *asc, *des;
		// initArtist()
		artistWidget *artwidget;
		QRadioButton *defa, *blanka, *lasta;
		QCheckBox *autofillyear, *autofillend, *defaultfollow;
		// initTitle()
		titleWidget *titlwidget;
		QRadioButton *deft, *blankt, *lastt;
		QCheckBox *autofillyeart, *autofillendt, *defaultcollect;
		// initMisc()
		QRadioButton *catasc, *catdes;
		QCheckBox *autocapitalize, *autoformat;
		// initReports()
		reportWidget *report;
		reportInfo *info;
		colorInfo *color;
		// initCDDB()
		QCheckBox *autopopup;
		KLineEdit *cdparanoia, *cddbpserver, *cddbpport, *httpserver, *httpport, *httpcgi, *proxyserver, *proxyport;
		KLineEdit *readdevice, *cddbemail, *smtpserver, *smtpport, *localpath;
		KComboBox *emailProfiles;
		QCheckBox *cddb_local;
		QRadioButton *cddbpbutton, *httpbutton;
		QCheckBox *proxybutton;
};

/**
 * Used to keep track of pointers to preferencesDialog's.
 */
typedef QMap<preferencesDialog*,int> PreferencesDialogManager;


#endif

