/***************************************************************************
                          reportdialog.cpp  -  description
                             -------------------
    begin                : Thu Jul 5 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "reportdialog.h"

/**
 * Constructor
 * @param QWidget Parent
 * @param QString Caption
 * @param int Report Type
 */
reportDialog::reportDialog(QWidget *parent, const QString &caption, const int &rt):
KDialogBase(Plain, i18n("%1 Report Generator").arg(caption), Ok | Close,
			 Ok, parent, "report", false, false)
{
	cerr << "reportDialog::initialized" << endl;
	QFrame *page = plainPage();
	setButtonText(Ok, i18n("&Generate"));
	report_type=rt;

	rw = new reportWidget(page, spacingHint());
}

/**
 * Destructor
 */
reportDialog::~reportDialog()
{
}

/**
 * Reimplement close button.
 */
void reportDialog::slotClose()
{
	delayedDestruct();
}

/**
 * Emits generateReport signal.
 */
void
reportDialog::slotOk()
{
	report.sort_order = rw->getSortOrder();
	report.sort_time = rw->getYearSort();
	report.date_range_enabled = rw->getDateRangeEnabled();
	report.limit_enabled=rw->getLimitEnabled();
	report.report_type = rw->getReportType();
	report.start = rw->getYearStart();
	report.end = rw->getYearEnd();
	report.limit=rw->getLimit();
	report.rfont = rw->getFont();
	report.charset=KGlobal::locale()->encoding();

	color.misc = rw->getColor(COLOR_MISC).name();
	color.artist = rw->getColor(COLOR_ARTIST).name();
	color.title = rw->getColor(COLOR_TITLE).name();
	color.track = rw->getColor(COLOR_TRACK).name();
	color.totals = rw->getColor(COLOR_TOTAL).name();
	color.background = rw->getColor(COLOR_BACKGROUND).name();
	color.text = rw->getColor(COLOR_TEXT).name();
	cerr << "reportDialog::report.sort_order: " << report.sort_order << endl;
	cerr << "reportDialog::report.date_range_enabled: " << report.date_range_enabled << endl;
	cerr << "reportDialog::report.report_type: " << report.report_type << endl;
	cerr << "reportDialog::report.start: " << report.start << endl;
	cerr << "reportDialog::report.end: " << report.end << endl;
	cerr << "reportDialog::report.rfont: " << report.rfont.family() << endl;
	cerr << "reportDialog::color.misc: " << color.misc << endl;
	cerr << "reportDialog::color.artist: " << color.artist << endl;
	cerr << "reportDialog::color.title: " << color.title << endl;
	cerr << "reportDialog::color.track: " << color.track << endl;
	cerr << "reportDialog::color.totals: " << color.totals << endl;
	cerr << "reportDialog::color.background: " << color.background << endl;

	// check for blank values. if no go, then return and make the user cancel or change.
	if (report.date_range_enabled)
	{
		if (report.start.isEmpty())
		{
			KMessageBox::information(this, i18n("Year Start cannot be empty."), i18n("Year Start"));
			return;
		}
		if (report.end.isEmpty())
		{
			KMessageBox::information(this, i18n("Year End cannot be empty."), i18n("Year End"));
			return;
		}
	}
	if (report.limit_enabled == 0)
	{
		if (report.limit.isEmpty())
		{
			KMessageBox::information(this,i18n("Limit cannot be empty."),i18n("Limit"));
			return;
		}
	}
	hide();
	emit generateReport(report, color,report_type);
	delayedDestruct();
}

reportWidget *reportDialog::widget()
{
	return rw;
}

#include "reportdialog.moc"
