/***************************************************************************
                          reportdialog.h  -  description
                             -------------------
    begin                : Thu Jul 5 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef REPORTDIALOG_H
#define REPORTDIALOG_H

#include "include.h"
#include "reportwidget.h"

/**
 * Common Report Interface. Opens a dialog with the default report properties, allows you to change them,
 * and when you click Generate, emits a signal which hopefully, is connected to a slot that generates
 * the report.
 * @short Report generating dialog
 * @author Will DeRousse
 */

class reportDialog : public KDialogBase  {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 * @param QString Caption
		 * @param int Report Type
		 */
		reportDialog(QWidget *, const QString &, const int &);
		/**
		 * Destructor
		 */
		~reportDialog();
		/**
		 * Returns reportWidget
		 */
		reportWidget *widget();
	protected slots:
		/**
		 * Reimplement close button.
		 */
		void slotClose();
		/**
		 * Emits generateReport signal.
		 */
		void slotOk();
	signals:
		/**
		 * Emitted when the Ok (Generate) button is clicked.
		 * @param reportInfo reportInfo struct containing values set in the dialog
		 * @param colorInfo colorInfo struct containing values set in the dialog
		 * @param int Report Type
		 */
		void generateReport(reportInfo, colorInfo, int);
	private:
		int report_type;
		colorInfo color;
		reportInfo report;
		reportWidget *rw;
};


#endif

