/***************************************************************************
                          reportwidget.cpp  -  description
                             -------------------
    begin                : Thu Jul 5 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "reportwidget.h"

/**
 * Constructor
 * @param QWidget Parent
 * @param int Spacing
 */
reportWidget::reportWidget(QWidget *parent, const int &spacing):
QGridLayout(parent, 4, 2, 0, spacing)
{
	cerr << "reportWidget::initialized" << endl;
	bgroup = new QButtonGroup(i18n("Order"), parent);
	grid = new QGridLayout(bgroup, 3, 1, spacing);
	grid->setMargin(DIALOG_MARGIN);
	asc = new QRadioButton(i18n("Asc&ending"), bgroup);
	des = new QRadioButton(i18n("&Descending"), bgroup);
	yearsort = new QCheckBox(i18n("By &Year Not Name"), bgroup);
	grid->addWidget(asc, 0, 0);
	grid->addWidget(des, 1, 0);
	grid->addWidget(yearsort, 2, 0);
	QToolTip::add(asc, i18n("In order, from A to Z or 0-9"));
	QToolTip::add(des, i18n("In order, from Z to A or 9-0"));
	QToolTip::add(yearsort, i18n("Order By Year, Rather Than Name"));
	addWidget(bgroup, 0, 0);

	bgroup = new QButtonGroup(i18n("Selection"), parent);
	grid = new QGridLayout(bgroup, 3, 1, spacing);
	grid->setMargin(DIALOG_MARGIN);
	all_c = new QRadioButton(i18n("&All"), bgroup);
	yes_c = new QRadioButton(i18n("&Only Collected"), bgroup);
	no_c = new QRadioButton(i18n("&Not Collected"), bgroup);
	grid->addWidget(all_c, 0, 0);
	grid->addWidget(no_c, 1, 0);
	grid->addWidget(yes_c, 2, 0);
	QToolTip::add(all_c, i18n("All Items Listed"));
	QToolTip::add(no_c, i18n("Only items not in collection listed"));
	QToolTip::add(yes_c, i18n("Only items in collection listed"));
	addWidget(bgroup, 0, 1);

	// add date range variables and check box
	bgroup = new QButtonGroup(i18n("Misc Options"), parent);
	grid = new QGridLayout(bgroup, 3, 4, spacing);
	grid->setMargin(DIALOG_MARGIN);
	dr_enable = new QCheckBox(i18n("Da&te Range"), bgroup);
	label = new QLabel(i18n("Year Starting"), bgroup);
	dr_first = new KLineEdit("", bgroup);
	dr_first->setMinimumWidth(dr_first->fontMetrics().width("xxxx") + 20);
	dr_first->setMaximumWidth(dr_first->fontMetrics().width("xxxx") + 20);
	grid->addMultiCellWidget(dr_enable, 0, 0, 0, 1);
	grid->addWidget(label, 1, 0);
	grid->addWidget(dr_first, 1, 1);
	label = new QLabel(i18n("Year Ending"), bgroup);
	dr_last = new KLineEdit("", bgroup);
	dr_last->setMinimumWidth(dr_last->fontMetrics().width("xxxx") + 20);
	dr_last->setMaximumWidth(dr_last->fontMetrics().width("xxxx") + 20);
	grid->addWidget(label, 2, 0);
	grid->addWidget(dr_last, 2, 1);
	QToolTip::add(dr_enable, i18n("Enable to Limit Results by Year"));
	QToolTip::add(dr_first, i18n("Starting Date"));
	QToolTip::add(dr_last, i18n("Ending Date"));
	connect(dr_enable, SIGNAL(toggled(bool)), this, SLOT(enableDates(bool)));

	// add limit
	grid->setMargin(DIALOG_MARGIN);
	limit_enable = new QCheckBox(i18n("L&imit Results"), bgroup);
	label = new QLabel(i18n("Number"), bgroup);
	limit_text = new KLineEdit("", bgroup);
	limit_text->setMaximumWidth(limit_text->fontMetrics().width("xxxxxxx") + 20);
	limit_text->setMinimumWidth(limit_text->fontMetrics().width("xxxxxxx") + 20);
	grid->addMultiCellWidget(limit_enable, 0, 0, 2, 3);
	grid->addWidget(label, 1, 2);
	grid->addWidget(limit_text, 1, 3);
	QToolTip::add(limit_enable, i18n("Enable this to limit results to a finite amount"));
	QToolTip::add(limit_text, i18n("Number of Results to Display"));
	addMultiCellWidget(bgroup, 1, 1, 0, 1);
	connect(limit_enable,SIGNAL(toggled(bool)),this,SLOT(enableLimit(bool)));


	// set color options
	bgroup = new QButtonGroup(i18n("Colors"), parent);
	grid = new QGridLayout(bgroup, 8, 2, spacing);
	grid->setMargin(DIALOG_MARGIN);
	label = new QLabel(i18n("Artist"), bgroup);
	grid->addWidget(label, 0, 0);
	label = new QLabel(i18n("Background"), bgroup);
	grid->addWidget(label, 1, 0);
	label = new QLabel(i18n("Miscellaneous"), bgroup);
	grid->addWidget(label, 2, 0);
	label = new QLabel(i18n("Title"), bgroup);
	grid->addWidget(label, 3, 0);
	label = new QLabel(i18n("Totals"), bgroup);
	grid->addWidget(label, 4, 0);
	label = new QLabel(i18n("Track"), bgroup);
	grid->addWidget(label, 5, 0);
	label = new QLabel(i18n("Text"), bgroup);
	grid->addWidget(label, 6, 0);

	pmap = new QPixmap();
	if (!pmap)
		exit(1);
	for (int i = 0; i < 7; i++)
	{
		colorButton[i] = new QPushButton(bgroup);
		grid->addWidget(colorButton[i], i, 1);
		QToolTip::add(colorButton[i], i18n("Choose A Color"));
	}
	connect(bgroup, SIGNAL(clicked(int)), this, SLOT(slotChangeColor(int)));
	addMultiCellWidget(bgroup, 2, 2, 0, 1);

	bgroup = new QButtonGroup(i18n("Font"), parent);
	sfont = new QPushButton(bgroup);
	sfont->setText(i18n("Change Font"));
	grid->addMultiCellWidget(sfont, 0, 0, 0, 1);
	connect(sfont, SIGNAL(clicked()), this, SLOT(slotChangeFont()));
	addMultiCellWidget(bgroup, 3, 3, 0, 1);
	QToolTip::add(sfont,i18n("Change the report font"));

}

/**
 * Destructor
 */
reportWidget::~reportWidget()
{
}

/**
 * Used for enabling or disabling the date range section.
 * @param bool TRUE enabled, FALSE disabled
 */
void
reportWidget::enableDates(bool t) const
{
	dr_first->setEnabled(t);
	dr_last->setEnabled(t);
}

/**
 * Used for enabling or disabling the limit section.
 * @param bool TRUE enabled, FALSE disabled
 */
void reportWidget::enableLimit(bool t) const
{
	limit_text->setEnabled(t);
}

/**
 * Retrieves color based on color type.
 * @param int Color type defined in include.h
 */
QColor
reportWidget::getColor(const int &c)
{
	int d=c;
	if (d < -1 && d >7)
		d=0;
	return colorswatch[c];
}

/**
 * Retrieves date range enabled.
 */
bool
reportWidget::getDateRangeEnabled() const
{
	return dr_enable->isChecked();
}

/**
 * Retrieves the font.
 */
QFont
reportWidget::getFont() const
{
	return myfont;
}

/**
 * Retrieves the limit.
 */
QString reportWidget::getLimit() const
{
	return limit_text->text();
}

/**
 * Retrieves whether or not limit is enabled.
 */
bool reportWidget::getLimitEnabled() const
{
	return limit_enable->isChecked();
}

/**
 * Retrieves report type.
 */
int
reportWidget::getReportType() const
{
	if (all_c->isChecked())
		return REPORT_ALL;
	else if (yes_c->isChecked())
		return REPORT_YES;
	else
		return REPORT_NO;
}

/**
 * Retrieves sort order.
 */
int
reportWidget::getSortOrder() const
{
	if (asc->isChecked())
		return ASC;
	else
		return DES;
}

/**
 * Retrieves sort order by year.
 */
bool
reportWidget::getYearSort() const
{
	return yearsort->isChecked();
}

/**
 * Retrieves year end.
 */
QString
reportWidget::getYearEnd() const
{
	return dr_last->text();
}

/**
 * Retrieves year start.
 */
QString
reportWidget::getYearStart() const
{
	return dr_first->text();
}

/**
 * Sets color.
 * @param int Color type defined in include.h
 * @param QColor Color
 */
void
reportWidget::setColor(const int &t, const QColor &c)
{
	if (t > -1 && t < 7)
	{
		colorswatch[t] = c;
		pmap->fill(colorswatch[t]);
		colorButton[t]->setPixmap(*pmap);
	}
}

/**
 * Slot used for selecting the proper font from the font dialog.
 * @param int KType
 */
void reportWidget::slotChangeColor(int kwhat)
{
	int result = KColorDialog::getColor(colorswatch[kwhat]);
	if (result == KColorDialog::Accepted)
	{
		pmap->fill(colorswatch[kwhat]);
		colorButton[kwhat]->setPixmap(*pmap);
	}
}

/**
 * Sets which report options are enabled and disabled.
 * @param reportProvides Report Options
 */
void reportWidget::setEnable(const reportProvides *rp)
{
	if (!rp->orderby)
	{
		asc->setEnabled(FALSE);
		des->setEnabled(FALSE);
		yearsort->setEnabled(FALSE);
	}
	if (!rp->collect)
	{
		all_c->setEnabled(FALSE);
		no_c->setEnabled(FALSE);
		yes_c->setEnabled(FALSE);
	}
	if (!rp->range)
	{
		dr_first->setEnabled(FALSE);
		dr_last->setEnabled(FALSE);
		dr_enable->setEnabled(FALSE);
	}
	if (!rp->font)
	{
		 sfont ->setEnabled(FALSE);
	}
	if (!rp->color)
	{
		for (int i = 0; i < 7; i++)
			colorButton[i]->setEnabled(FALSE);
	}
	if (!rp->limit)
	{
		limit_enable->setEnabled(FALSE);
		limit_text->setEnabled(FALSE);
	}
}

/**
 * Sets the limit.
 * @param QString Limit Value
 */
void reportWidget::setLimit(const QString &t) const
{
	limit_text->setText(t);
}

/**
 * Setes whether or not limit is enabled.
 * @param bool TRUE enables, FALSE disables
 */
void reportWidget::setLimitEnabled(const bool &t) const
{
	limit_enable->setChecked(t);
}

/**
 * Slot used for selecting the proper font from the font dialog.
 */
void
reportWidget::slotChangeFont()
{
	KFontDialog::getFont(myfont, FALSE);
}

/**
 * Sets date range.
 * @param bool TRUE enables, FALSE disables
 */
void
reportWidget::setDateRangeEnabled(const bool & t) const
{
	dr_enable->setChecked(t);
}

/**
 * Sets the font.
 * @param QFont Font
 */
void
reportWidget::setKMFont(const QFont &f)
{
	myfont = f;
}

/**
 * Sets report type.
 * @param int Report Type
 */
void
reportWidget::setReportType(const int &t) const
{
	switch(t)
	{
		case REPORT_ALL:
			all_c->setChecked(TRUE);
			break;
		case REPORT_YES:
			yes_c->setChecked(TRUE);
			break;
		case REPORT_NO:
		default:
			no_c->setChecked(TRUE);
	};
}

/**
 * Sets sort order.
 * @param int Sort Order
 */
void
reportWidget::setSortOrder(const int &t) const
{
	switch(t)
	{
		case ASC:
			asc->setChecked(TRUE);
			break;
		case DES:
		default:
			des->setChecked(TRUE);
			break;
	};
}

/**
 * Sets sort order for year.
 * @param bool On/Off
 */
void
reportWidget::setYearSort(const bool &status) const
{
	yearsort->setChecked(status);
}

/**
 * Sets year end.
 * @param QString Year End
 */
void
reportWidget::setYearEnd(const QString &t) const
{
	dr_last->setText(t);
}

/**
 * Sets year start.
 * @param QString Year Start
 */
void
reportWidget::setYearStart(const QString &t) const
{
	dr_first->setText(t);
}

/**
 * Populates this widget with customer information regarding enabled features, colors, fonts, etc.
 * @param reportInfo reportInfo struct containing default report values
 * @param colorInfo colorInfo struct containing default color values
 * @param reportProvides reportProvides struct holding options report supports
 */
void reportWidget::updateInfo(const reportInfo *options, const colorInfo *color, const reportProvides *rp)
{
	colorswatch[COLOR_MISC].setNamedColor(color->misc);
	colorswatch[COLOR_ARTIST].setNamedColor(color->artist);
	colorswatch[COLOR_TITLE].setNamedColor(color->title);
	colorswatch[COLOR_TRACK].setNamedColor(color->track);
	colorswatch[COLOR_TOTAL].setNamedColor(color->totals);
	colorswatch[COLOR_BACKGROUND].setNamedColor(color->background);
	colorswatch[COLOR_TEXT].setNamedColor(color->text);
	pmap->resize(120, 20);
	for (int i = 0; i < 7; i++)
	{
		pmap->fill(colorswatch[i]);
		colorButton[i]->setPixmap(*pmap);
	}

	setSortOrder(options->sort_order);
	setYearSort(options->sort_time);
	setReportType(options->report_type);
	setDateRangeEnabled(options->date_range_enabled);
	setLimitEnabled(options->limit_enabled);
	setYearStart(options->start);
	setYearEnd(options->end);
	setLimit(options->limit);
	setKMFont(options->rfont);

	enableDates(options->date_range_enabled);
	enableLimit(options->limit_enabled);

	setEnable(rp);
}

#include "reportwidget.moc"
