/***************************************************************************
                          reportwidget.h  -  description
                             -------------------
    begin                : Thu Jul 5 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef REPORTWIDGET_H
#define REPORTWIDGET_H

#include "include.h"

/**
 *reportWidget is a widget which holds all the widgets for setting report generating preferences.
 *
 *It contains set and get functions.
 * @short Layout with report options.
 * @author Will DeRousse
 */

class reportWidget : public QGridLayout {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 * @param int Spacing
		 */
		reportWidget(QWidget *, const int &);
		/**
		 * Destructor
		 */
		~reportWidget();
		/**
		 * Retrieves color based on color type.
		 * @param int Color type defined in include.h
		 */
		QColor getColor(const int &);
		/*
		 * Retrieves whether or not the date range has been enabled.
		 */
		bool getDateRangeEnabled() const;
		/**
		 * Retrieves the font.
		 */
		QFont getFont() const;
		/**
		 * Retrieves the limit.
		 */
		QString getLimit() const;
		/**
		 * Retrieves whether or not limit is enabled.
		 */
		bool getLimitEnabled() const;
		/**
		 * Retrieves report type.
		 */
		int getReportType() const;
		/**
		 * Retrieves sort order.
		 */
		int getSortOrder() const;
		/**
		 * Retrieves sort order by year.
		 */
		bool getYearSort() const;
		/**
		 * Retrieves year end.
		 */
		QString getYearEnd() const;
		/**
		 * Retrieves year start.
		 */
		QString getYearStart() const;
		/**
		 * Sets color.
		 * @param int Color type defined in include.h
		 * @param QColor Color
		 */
		void setColor(const int &, const QColor &);
		/**
		 * Sets date range.
		 * @param bool TRUE enables, FALSE disables
		 */
		void setDateRangeEnabled(const bool &) const;
		/**
		 * Sets which report options are enabled and disabled.
		 * @param reportProvides Report Options
		 */
		void setEnable(const reportProvides *);
		/**
		 * Sets the limit.
		 * @param QString Limit Value
		 */
		void setLimit(const QString &) const;
		/**
		 * Setes whether or not limit is enabled.
		 * @param bool TRUE enables, FALSE disables
		 */
		void setLimitEnabled(const bool &) const;
		/**
		 * Sets the font.
		 * @param QFont Font
		 */
		void setKMFont(const QFont &);
		/**
		 * Sets report type.
		 * @param int Report Type
		 */
		void setReportType(const int &) const;
		/**
		 * Sets sort order.
		 * @param int Sort Order
		 */
		void setSortOrder(const int &) const;
		/**
		 * Sets sort order for year.
		 * @param bool On/Off
		 */
		void setYearSort(const bool &) const;
		/**
		 * Sets year end.
		 * @param QString Year End
		 */
		void setYearEnd(const QString &) const;
		/**
		 * Sets year start.
		 * @param QString Year Start
		 */
		void setYearStart(const QString &) const;
		/**
		 * Populates this widget with customer information regarding enabled features, colors, fonts, etc.
		 * @param reportInfo reportInfo struct containing default report values
		 * @param colorInfo colorInfo struct containing default color values
		 * @param reportProvides reportProvides struct holding options report supports
		 */
		void updateInfo(const reportInfo *, const colorInfo *, const reportProvides *);
	private slots:
		/**
		 * Used for enabling or disabling the date range section.
		 * @param bool TRUE Enabled, FALSE Disabled
		 */
		void enableDates(bool) const;
		/**
		 * Used for enabling or disabling the limit section.
		 * @param bool TRUE Enabled, FALSE Disabled
		 */
		void enableLimit(bool) const;
		/**
		 * Slot used for changing the color on the color buttons.
		 * @param int KType
		 */
		void slotChangeColor(int);
		/**
		 * Slot used for selecting the proper font from the font dialog.
		 */
		void slotChangeFont();
	private:
		QButtonGroup *bgroup;
		QGridLayout *grid;
		QCheckBox *yearsort;
		QRadioButton *asc, *des, *all_c, *no_c, *yes_c;
		QLabel *label;
		QPushButton *colorButton[7], *sfont;
		QColor colorswatch[7];
		QPixmap *pmap;
		QFont myfont;

		QCheckBox *limit_enable, *dr_enable;
		QLineEdit *limit_text, *dr_first, *dr_last;
};


#endif

