/***************************************************************************
                          sm.cpp  -  description
                             -------------------
    begin                : Thu Jan 17 2002
    copyright            : (C) 2002 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmusicdb.h"

/* *************************************************************************** */
/* *********************** SESSION MANAGEMENT *************************** */
/* ************************************************************************** */
/**
 * Re-implementation of this function. Using this method to save settings before the window is closed.
 * @param QCloseEvent Event
 */
void
kmusicdb::closeEvent(QCloseEvent *e)
{
	cerr << "kmusicdb::closeEvent" << endl;
	KConfig *kc=kapp->config();
	// load toolbar settings
	kc->setGroup(i18n("ToolBars"));
	kc->writeEntry(i18n("main"), showtoolbar->isChecked());
	kc->writeEntry(i18n("name"), namesearch->isChecked());
	kc->writeEntry(i18n("title"), titlesearch->isChecked());
	kc->writeEntry(i18n("status"), showstatusbar->isChecked());

	// save default values for dbinfo
	if (def->dbdata == LAST_VALUE)
	{
		kc->setGroup(i18n("lastdb"));
		kc->writeEntry(i18n("location"), dbconnectinfo->db_location);
		kc->writeEntry(i18n("name"), dbconnectinfo->db_name);
		kc->writeEntry(i18n("port"), dbconnectinfo->db_port);
		kc->writeEntry(i18n("user"), dbconnectinfo->db_user);
		kc->writeEntry(i18n("passwd"), dbconnectinfo->db_passwd);
	}

	// settings for artist
	if (def->artistdata == LAST_VALUE)
	{
		kc->setGroup(i18n("lastartist"));
		kc->writeEntry(i18n("aid"), artist->aid);
		kc->writeEntry(i18n("notes"), artist->notes);
		kc->writeEntry(i18n("first"), artist->first);
		kc->writeEntry(i18n("last"), artist->last);
		kc->writeEntry(i18n("name"), artist->name);
		kc->writeEntry(i18n("follow"), artist->follow);
		kc->writeEntry(i18n("homepage"), artist->homepage);
	}

	// settings for title
	if (def->titledata == LAST_VALUE)
	{
		kc->setGroup(i18n("lasttitle"));
		kc->writeEntry(i18n("cdid"), title->cdid);
		kc->writeEntry(i18n("fid"), title->cat[KFORMAT].name);
		kc->writeEntry(i18n("gid"), title->cat[KGENRE].name);
		kc->writeEntry(i18n("grpid"), title->cat[KBOXSET].name);
		kc->writeEntry(i18n("lid"), title->cat[KLABEL].name);
		kc->writeEntry(i18n("mid"), title->cat[KMETHOD].name);
		kc->writeEntry(i18n("tid"), title->cat[KTYPE].name);
		kc->writeEntry(i18n("length"), title->length);
		kc->writeEntry(i18n("name"), title->name);
		kc->writeEntry(i18n("nod"), title->nod);
		kc->writeEntry(i18n("notes"), title->notes);
		kc->writeEntry(i18n("notracks"), title->notracks);
		kc->writeEntry(i18n("yearend"), title->yearend);
		kc->writeEntry(i18n("yearstart"), title->yearstart);
		kc->writeEntry(i18n("collected"), title->collected);
		kc->writeEntry(i18n("rating"), title->rating);
		kc->writeEntry(i18n("quality"), title->quality);
		kc->writeEntry(i18n("code"), title->code);
		kc->writeEntry(i18n("loan"), title->loan);
	}

	kc->sync();
	e->accept();
}

/**
 * Session Management reimplemented.
 * @param KConfig KmusicdB Config
 */
void
kmusicdb::readProperties(KConfig *session)
{
	cerr << "kmusicdb::readProperties" << endl;
	KConfig *kc=kapp->config();
	// toolbar settings
	session->setGroup(i18n("interface"));
	category->setCurrentItem(session->readNumEntry(i18n("category")));
	catresult->setCurrentItem(session->readNumEntry(i18n("categoryresult")));
	nametypes->setCurrentItem(session->readNumEntry(i18n("namecategory")));
	toolbarname->setText(session->readEntry(i18n("nametext")));

	// load db if last session was connected. reset auto_connect to
	// stored value.
	session->setGroup(i18n("postgres"));
	c = session->readNumEntry(i18n("reconnect"));
	if (c)
	{
		// set our variables straight for reconnecting.
		dbconnectinfo->db_name = session->readEntry(i18n("name"));
		dbconnectinfo->db_location = session->readEntry(i18n("location"));
		dbconnectinfo->db_port = session->readEntry(i18n("port"));
		dbconnectinfo->db_user = session->readEntry(i18n("user"));
		dbconnectinfo->db_passwd = session->readEntry(i18n("passwd"));
		open_db(dbconnectinfo);
		// load the current artist
		emit signalDbConnect(TRUE);
		if (music)
		{
			session->setGroup(i18n("artist"));
			updateArtistView();
			mid->setArtistText(session->readEntry(i18n("current"), ""));
			updateTitleView();
		}
	}
	// we have to read that backup copy into memory and then WRITE it to our settings to keep it current
	session->setGroup(i18n("postgres"));
	def->auto_connect = session->readBoolEntry(i18n("auto_connect"));
	kc->setGroup(i18n("postgres"));
	kc->writeEntry(i18n("auto_connect"), def->auto_connect);
	kc->sync();

	// toolbar info
	session->setGroup(i18n("searchbars"));
	//title search
	toolBar("titleSearch")->getCombo(TITLE_CAT_COMBO)-> setCurrentItem(session->readNumEntry(i18n("kombo1"), 0));
	toolBar("titleSearch")->getCombo(TITLE_LIST_COMBO)-> setCurrentItem(session->readNumEntry(i18n("kombo2"), 0));
	//name search
	toolBar("nameSearch")->getCombo(NAME_COMBO)->setCurrentItem(session-> readNumEntry(i18n("kombo3"), 0));
	toolBar("nameSearch")->getLined(NAME_LINED)->setText(session-> readEntry(i18n("name1"), ""));

	updateGUI(100, i18n("Ready..."));
}

/** Use this function to read the settings from the configuration file and set the variables in memory. */
void
kmusicdb::readSettings()
{
	cerr << "Kmusicdb::readSettings" << endl;
	KConfig *kc=kapp->config();
	// tool bar settings.
	kc->setGroup(i18n("ToolBars"));
	showtoolbar->setChecked(kc-> readBoolEntry(i18n("main"), TOOLBAR_DEFAULT));
	namesearch->setChecked(kc-> readBoolEntry(i18n("name"), TOOLBAR_NAME));
	titlesearch->setChecked(kc-> readBoolEntry(i18n("title"), TOOLBAR_TITLE));
	showstatusbar->setChecked(kc-> readBoolEntry(i18n("status"), STATUSBAR_DEFAULT));
	slotToolbar(1);
	slotStatusbar();
	toggleTitle(titlesearch->isChecked());
	toggleName(namesearch->isChecked());

	// server defaults settings.
	kc->setGroup(i18n("postgres"));
	def->auto_connect = kc->readBoolEntry(i18n("auto_connect"), AUTO_CONNECT_DB);
	def->dbdata = kc->readNumEntry(i18n("data"), STATIC_DEFAULT);
	if (def->dbdata == BLANK)
	{
		dbconnectinfo->db_name = "";
		dbconnectinfo->db_location = "";
		dbconnectinfo->db_port = "";
		dbconnectinfo->db_user = "";
		dbconnectinfo->db_passwd = "";
	}
	else
	{
		if (def->dbdata == LAST_VALUE)
			kc->setGroup(i18n("lastdb"));
		dbconnectinfo->db_name = kc->readEntry(i18n("name"), DEFAULT_DB);
		dbconnectinfo->db_location = kc->readEntry(i18n("location"), DEFAULT_LOCATION);
		dbconnectinfo->db_port = kc->readEntry(i18n("port"), DEFAULT_SERVER_PORT);
		dbconnectinfo->db_user = kc->readEntry(i18n("user"), DEFAULT_SERVER_USER);
		dbconnectinfo->db_passwd = kc->readEntry(i18n("passwd"), DEFAULT_SERVER_PASSWD);
	}

	// autoconnect settings.
	kc->setGroup(i18n("auto_connect"));
	dbauto->db_name = kc->readEntry(i18n("name"), DEFAULT_DB);
	dbauto->db_location = kc->readEntry(i18n("location"), DEFAULT_LOCATION);
	dbauto->db_port = kc->readEntry(i18n("port"), DEFAULT_SERVER_PORT);
	dbauto->db_user = kc->readEntry(i18n("user"), DEFAULT_SERVER_USER);
	dbauto->db_passwd = kc->readEntry(i18n("passwd"), DEFAULT_SERVER_PASSWD);

	// the main widget settings
	kc->setGroup(i18n("mainwidget"));
	def->default_column_sort = kc->readNumEntry(i18n("default_column_sort"), DEFAULT_COLUMN);
	// 0 is ascending, 1 is descending
	def->default_sort_order = kc->readBoolEntry(i18n("default_sort_order"), DEFAULT_SORT_ORDER);

	// settings for artist and titles default values
	kc->setGroup(i18n("artist"));
	// 0 for text, 1 for blank, 2 for last entry
	def->artistdata = kc->readNumEntry(i18n("data"), BLANK);

	// fill artist defaults
	def->artist_year = kc->readBoolEntry(i18n("year"), DEFAULT_YEAR);
	def->artist_autofill = kc->readBoolEntry(i18n("autofill"), DEFAULT_AUTO_FILL);
	def->artist_follow = kc->readBoolEntry(i18n("follow2"), DEFAULT_FOLLOW);

	// fill artist struct
	if (def->artistdata == BLANK)
	{
		artist->aid = "";
		artist->notes = "";
		artist->first = "";
		artist->last = "";
		artist->name = "";
		artist->follow = FALSE;
		artist->homepage = "";
	}
	else
	{
		if (def->artistdata == LAST_VALUE)
			kc->setGroup(i18n("lastartist"));
		artist->aid = kc->readEntry(i18n("aid"), "");
		artist->notes = kc->readEntry(i18n("notes"), "");
		artist->first = kc->readEntry(i18n("first"), "");
		artist->last = kc->readEntry(i18n("last"), "");
		artist->name = kc->readEntry(i18n("name"), "");
		artist->follow = kc->readBoolEntry(i18n("follow"), FALSE);
		artist->homepage = kc->readEntry(i18n("homepage"), "http://");
	}
	// fill title struct
	kc->setGroup(i18n("title"));
	// 0 for text, 1 for blank, 2 for last entry
	def->titledata = kc->readNumEntry(i18n("data"), BLANK);

	// fill title defaults
	def->title_year = kc->readBoolEntry(i18n("year"), DEFAULT_YEAR);
	def->title_autofill = kc->readBoolEntry(i18n("autofill"), DEFAULT_AUTO_FILL);
	def->title_collect = kc->readBoolEntry(i18n("collected2"), DEFAULT_FOLLOW);

	// fill artist struct
	if (def->titledata == BLANK)
	{
		title->cdid = "";
		title->cat[KFORMAT].name = i18n("UNKNOWN");
		title->cat[KGENRE].name = i18n("UNKNOWN");
		title->cat[KBOXSET].name = i18n("UNKNOWN");
		title->cat[KLABEL].name = i18n("UNKNOWN");
		title->cat[KMETHOD].name = i18n("UNKNOWN");
		title->cat[KTYPE].name = i18n("UNKNOWN");
		title->length = "00:00:00";
		title->name = "";
		title->nod = "";
		title->notes = "";
		title->notracks = "";
		title->yearend = "";
		title->yearstart = "";
		title->collected = FALSE;
		title->rating = "";
		title->quality = "";
		title->code="";
		title->loan=FALSE;
	}
	else
	{
		if (def->titledata == LAST_VALUE)
			kc->setGroup(i18n("lasttitle"));
		title->cdid = kc->readEntry(i18n("cdid"), "");
		title->cat[KFORMAT].name = kc->readEntry(i18n("fid"), "");
		title->cat[KGENRE].name = kc->readEntry(i18n("gid"), "");
		title->cat[KBOXSET].name = kc->readEntry(i18n("grpid"), "");
		title->cat[KLABEL].name = kc->readEntry(i18n("lid"), "");
		title->cat[KMETHOD].name = kc->readEntry(i18n("mid"), "");
		title->cat[KTYPE].name = kc->readEntry(i18n("tid"), "");
		title->length = "00:00:00";
		title->name = kc->readEntry(i18n("name"), "");
		title->nod = kc->readEntry(i18n("nod"), "");
		title->notes = kc->readEntry(i18n("notes"), "");
		title->notracks = kc->readEntry(i18n("notracks"), "");
		title->yearend = kc->readEntry(i18n("yearend"), "");
		title->yearstart = kc->readEntry(i18n("yearstart"), "");
		title->collected = kc->readBoolEntry(i18n("collected"), FALSE);
		title->rating = kc->readEntry(i18n("rating"), "");
		title->quality = kc->readEntry(i18n("quality"), "");
		title->code = kc->readEntry(i18n("code"), "");
		title->loan = kc->readBoolEntry(i18n("loan"), FALSE);
	}

	kc->setGroup(i18n("cddb"));
	def->cddb_popup = kc->readBoolEntry(i18n("cddb_popup"), DEFAULT_CDDB_TITLE_OPEN);
	si->cddbp_server = kc->readEntry(i18n("cddbp_server"), DEFAULT_CDDBP_SERVER);
	si->cddbp_port = kc->readEntry(i18n("cddbp_port"), DEFAULT_CDDBP_PORT);
	si->http_server = kc->readEntry(i18n("http_server"), DEFAULT_HTTP_SERVER);
	si->http_port = kc->readEntry(i18n("http_port"), DEFAULT_HTTP_PORT);
	si->http_cgi = kc->readEntry(i18n("http_cgi"), DEFAULT_HTTP_CGI);
	si->proxy_server = kc->readEntry(i18n("proxy_server"), DEFAULT_PROXY_SERVER);
	si->proxy_port = kc->readEntry(i18n("proxy_port"), DEFAULT_PROXY_PORT);
	si->smtp_server = kc->readEntry(i18n("smtp_server"), DEFAULT_SMTP_SERVER);
	si->smtp_port = kc->readEntry(i18n("smtp_port"), DEFAULT_SMTP_PORT);
	si->cdparanoia_location = kc->readEntry(i18n("cdparanoia_location"), DEFAULT_CDPARANOIA_LOCATION);
	si->read_device = kc->readEntry(i18n("read_device"), DEFAULT_READ_DEVICE);
	si->cddb_email = kc->readEntry(i18n("cddb_email"), DEFAULT_CDDB_EMAIL);
	si->kde_email_profile = kc->readEntry(i18n("kde_email_profile"), DEFAULT_EMAIL_PROFILE);
	si->check_cddb_local = kc->readBoolEntry(i18n("check_cddb_local"), DEFAULT_CDDB_CHECK_LOCAL);
	si->local_cddb_path = kc->readEntry(i18n("local_cddb_path"), "");
	si->get_protocol = kc->readNumEntry(i18n("get_protocol"), CDDB_PROTO_CDDBP);
	si->put_protocol = kc->readNumEntry(i18n("put_protocol"), CDDB_PROTO_EMAIL);
	// find kscd cddb files
	if (si->local_cddb_path.isEmpty())
		si->local_cddb_path=KGlobal::dirs()->findDirs("data", "kscd/cddb").first();

	// read the default columns
	kc->setGroup(i18n("columns"));
	t_op->fid = kc->readBoolEntry(i18n("fid"), FALSE);
	t_op->gid = kc->readBoolEntry(i18n("gid"), FALSE);
	t_op->lid = kc->readBoolEntry(i18n("lid"), FALSE);
	t_op->mid = kc->readBoolEntry(i18n("mid"), FALSE);
	t_op->tid = kc->readBoolEntry(i18n("tid"), FALSE);
	t_op->yearend = kc->readBoolEntry(i18n("yearend"), FALSE);
	t_op->nod = kc->readBoolEntry(i18n("nod"), FALSE);
	t_op->yearstart = kc->readBoolEntry(i18n("yearstart"), TRUE);
	t_op->notracks = kc->readBoolEntry(i18n("notracks"), TRUE);
	t_op->length = kc->readBoolEntry(i18n("length"), TRUE);
	t_op->collected = kc->readBoolEntry(i18n("collected"), TRUE);
	t_op->rating = kc->readBoolEntry(i18n("rating"), FALSE);
	t_op->quality = kc->readBoolEntry(i18n("quality"), FALSE);
	t_op->code = kc->readBoolEntry(i18n("code"), FALSE);
	t_op->loan = kc->readBoolEntry(i18n("loan"), FALSE);

	mid->initHeaders(t_op);

	kc->setGroup(i18n("category"));
	def->category_sort_order = kc->readBoolEntry(i18n("category_sort_order"), DEFAULT_COLUMN_SORT_ORDER);

	kc->setGroup(i18n("text_processing"));
	def->auto_capitalize = kc->readBoolEntry(i18n("auto_capitalize"), DEFAULT_CAPITALIZE);
	def->auto_format = kc->readBoolEntry(i18n("auto_format"), DEFAULT_PROCESS_TEXT);

	// report options and report colors
	kc->setGroup(i18n("reports"));
	color->misc = kc->readEntry(i18n("misc"), DEFAULT_MISC_COLOR);
	color->artist = kc->readEntry(i18n("artist"), DEFAULT_ARTIST_COLOR);
	color->title = kc->readEntry(i18n("title"), DEFAULT_TITLE_COLOR);
	color->track = kc->readEntry(i18n("track"), DEFAULT_TRACK_COLOR);
	color->totals = kc->readEntry(i18n("totals"), DEFAULT_TOTALS_COLOR);
	color->background = kc->readEntry(i18n("background"), DEFAULT_BG_COLOR);
	color->text = kc->readEntry(i18n("text"), DEFAULT_TEXT_COLOR);

	reportOptions->sort_order = kc->readNumEntry(i18n("sort_order"), ASC);
	reportOptions->sort_time = kc->readBoolEntry(i18n("sort_time"), TRUE);
	reportOptions->date_range_enabled = kc->readBoolEntry(i18n("date_range_enabled"), FALSE);
	reportOptions->limit_enabled=kc->readBoolEntry(i18n("limit_enabled"),FALSE);
	reportOptions->report_type = kc->readNumEntry(i18n("report_type"), REPORT_ALL);
	reportOptions->start = kc->readEntry(i18n("start"), "0");
	reportOptions->end = kc->readEntry(i18n("end"), "9999");
	reportOptions->limit=kc->readEntry(i18n("limit"),"15");

	reportOptions->rfont.setFamily(kc-> readEntry(i18n("rfont_family"), DEFAULT_FAMILY));
	reportOptions->rfont.setBold(kc-> readBoolEntry(i18n("rfont_weight"), DEFAULT_WEIGHT));
	reportOptions->rfont.setItalic(kc-> readBoolEntry(i18n("rfont_style"), DEFAULT_STYLE));
	reportOptions->rfont.setPointSize(kc-> readNumEntry(i18n("rfont_size"), DEFAULT_SIZE));
}

/**
 * Session Management reimplemented.
 * @param KConfig KmusucdB Config
 */
void
kmusicdb::saveProperties(KConfig *session)
{
	cerr << "kmusicdb::saveProperties" << endl;
	KConfig *kc=kapp->config();
	// toolbar settings
	session->setGroup(i18n("interface"));
	session->writeEntry(i18n("category"), category->currentItem());
	session->writeEntry(i18n("categoryresult"), catresult->currentItem());
	session->writeEntry(i18n("namecategory"), nametypes->currentItem());
	session->writeEntry(i18n("nametext"), toolbarname->text());

	// add code for restoring/not restoring a database connection.
	// must disable autoload if enabled.
	session->setGroup(i18n("postgres"));
	if (music)
	{
		session->writeEntry(i18n("reconnect"), 1);
		session->writeEntry(i18n("name"), dbconnectinfo->db_name);
		session->writeEntry(i18n("location"), dbconnectinfo->db_location);
		session->writeEntry(i18n("port"), dbconnectinfo->db_port);
		session->writeEntry(i18n("user"), dbconnectinfo->db_user);
		session->writeEntry(i18n("passwd"), dbconnectinfo->db_passwd);
		session->writeEntry(i18n("auto_connect"), def->auto_connect);
		// we must tell the prog not to auto connect on startup
		kc->setGroup(i18n("postgres"));
		kc->writeEntry(i18n("auto_connect"), FALSE);
		kc->sync();
		// save the artist position.
		session->setGroup(i18n("artist"));
		session->writeEntry(i18n("current"), mid->getArtistText());
		session->sync();
	}
	else
		session->writeEntry(i18n("reconnect"), 0);

	// toolbar info
	session->setGroup(i18n("searchbars"));
	//title search
	session->writeEntry(i18n("kombo1"), toolBar("titleSearch")->getCombo(TITLE_CAT_COMBO)-> currentItem());
	session->writeEntry(i18n("kombo2"), toolBar("titleSearch")->getCombo(TITLE_LIST_COMBO)-> currentItem());
	//name search
	session->writeEntry(i18n("kombo3"), toolBar("nameSearch")->getCombo(NAME_COMBO)-> currentItem());
	session->writeEntry(i18n("name1"), toolBar("nameSearch")->getLined(NAME_LINED)-> text());

}

