/***************************************************************************
                          titledialog.cpp  -  description
                             -------------------
    begin                : Tue Apr 10 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "titledialog.h"

/**
 * Constructor
 * @param QWidget Parent
 * @param QString Caption
 * @param QString OK Button Text
 * @param QString Picture Button Text
 * @param bool Last Mirrors First
 * @param bool New/Edit title- grays track button
 * @param QString CDID
 * @param QString Path To Picture
 */
titleDialog::titleDialog(QWidget *parent, const QString &caption, const QString &okbuttontext, const QString &pictext,
		const bool &matchyears, const bool &newused, const QString &cd,const QString &picpath):
KDialogBase(Plain, caption, Ok | Cancel | User1 | User2, Ok, parent, "title", false, false, pictext, i18n("Edit &Tracks"))
{
	cerr << "titleDialog::initialized" << endl;
	picD = 0;
	test.picpath=picpath;
	test.cdid=cd;
	QFrame *page = plainPage();
	setButtonText(Ok, okbuttontext);
	setButtonTip(Ok, i18n("Click here to add/update this title."));
	setButtonTip(User1,i18n("Click here to add/view the picture (album cover)."));
	setButtonTip(User2,i18n("Click here to open the track editor."));
	title = new titleWidget(page, spacingHint(), matchyears);
	__track = new trackDialog(this, "",cd);

	connect(__track, SIGNAL(closeClicked()), this, SLOT(updateTracks()));

	if (newused)
		enableButton(User2, FALSE);
}

/**
 * Destructor
 */
titleDialog::~titleDialog()
{
	delete __track;
}

/** Closes dialog. No settings applied. */
void
titleDialog::slotCancel()
{
	emit relistTitles();
	emit cancelled(this);
	delayedDestruct();
}

/** Checks values and then emits addTitle(titleInfo).
 */
void
titleDialog::slotOk()
{
	test.name = title->getName();
	test.cat[KFORMAT].name = title->getFormat();
	test.cat[KGENRE].name = title->getGenre();
	test.cat[KMETHOD].name = title->getMethod();
	test.cat[KLABEL].name = title->getLabel();
	test.cat[KTYPE].name = title->getType();
	test.cat[KBOXSET].name = title->getBoxset();
	test.yearstart = title->getYearstart();
	test.yearend = title->getYearend();
	test.nod = title->getNod();
	test.notracks = title->getNotracks();
	test.notes = title->getNotes();
	test.collected = title->getCollected();
	test.rating.setNum(title->getRating());
	test.quality.setNum(title->getQuality());
	test.code=title->getCode();
	test.loan=title->getLoan();

	// check for blank values. if no go, then return and make the user cancel or change.
	if (test.name.isEmpty())
	{
		KMessageBox::information(this, i18n("Title name can not be empty."),i18n("Title Name"));
		return;
	}
	if (test.yearstart.isEmpty())
	{
		KMessageBox::information(this, i18n("Year can not be empty."),i18n("Year"));
		return;
	}
	if (test.yearend.isEmpty())
	{
		KMessageBox::information(this, i18n("Year End can not be empty."),i18n("Year End"));
		return;
	}
	if (test.nod.isEmpty())
	{
		KMessageBox::information(this, i18n("No. Media can not be empty."),i18n("No. Discs"));
		return;
	}
	if (test.cat[KFORMAT].name.isEmpty())
	{
		KMessageBox::information(this, i18n("Format cannot be blank."),i18n("Format"));
		return;
	}
	if (test.cat[KBOXSET].name.isEmpty())
	{
		KMessageBox::information(this, i18n("Boxset cannot be blank."),i18n("Format"));
		return;
	}
	if (test.cat[KGENRE].name.isEmpty())
	{
		KMessageBox::information(this, i18n("Genre cannot be blank."),i18n("Boxset"));
		return;
	}
	if (test.cat[KLABEL].name.isEmpty())
	{
		KMessageBox::information(this, i18n("Label cannot be blank."),i18n("Genre"));
		return;
	}
	if (test.cat[KMETHOD].name.isEmpty())
	{
		KMessageBox::information(this, i18n("Method cannot be blank."),i18n("Label"));
		return;
	}
	if (test.cat[KTYPE].name.isEmpty())
	{
		KMessageBox::information(this, i18n("Type cannot be blank."),i18n("Method"));
		return;
	}
	// check if pic has been changed and update.
	if (picD)
	{
		cerr << "titleDialog::Pic exists, getting new location." << endl;
		test.picpath =(picD->widget()->getImage()).path();
	}

	cerr << "titleDialog::Emitting addTitle() and cancelled(this)" << endl;
	hide();
	emit addTitle(test);
	emit cancelled(this);
	delayedDestruct();
}

/** Re-implemented for viewing picture dialog. */
void
titleDialog::slotUser1()
{
	KURL temp;
	cerr << "titleDialog::Creating picDialog" << endl;
	if (picD)
	{
		cerr << "titleDialog::Already exists... showing" << endl;
		picD->show();
		return;
	}
	temp.setPath(test.picpath);
	picD = new picDialog(this, temp, title->getName());
	if (!picD)
		exit(1);
	picD->show();
}

/** Re-implemented for editing tracks. */
void
titleDialog::slotUser2()
{
	emit updateTracks(__track->widget()->listview(), test.cdid);
	__track->show();
	// for some reason we must resize here, the resize event does not work(I guess qt3 isn't PERFECT).
	__track->widget()->resizeColumns();
}
/**
 * Returns a pointer to the trackDialog.
 */
trackDialog *titleDialog::track() const
{
	return __track;
}
/** Updates the number of tracks in the titleDialog. */
void
titleDialog::updateTracks() const
{
	title->setNumTracks(__track->widget()->listview()->childCount());
}
/**
 * Returns a pointer to titleWidget.
 */
titleWidget *titleDialog::widget() const
{
	return title;
}

#include "titledialog.moc"
