/***************************************************************************
                          titledialog.h  -  description
                             -------------------
    begin                : Tue Apr 10 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TITLEDIALOG_H
#define TITLEDIALOG_H

#include "include.h"
#include "titlewidget.h"
#include "kmusicdbvalidator.h"
#include "picdialog.h"
#include "trackdialog.h"

/**This is the new/edit dialog for titles.
 * Use it whenever you need to get information from the user in a user friendly, safe way.
 *
 * This dialog IS non modal. Be careful!
 * @short New/Edit title dialog
 * @author Will DeRousse
 *<pre>
 *titleDialog *ad=new titleDialog(parent, caption, okbuttontext, picturebuttontext, true, true, "455", "/home/test.jpg");
 *ad->show();
 *</pre>
 */

class titleDialog : public KDialogBase  {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 * @param QString Caption
		 * @param QString OK Button Text
		 * @param QString Picture Button Text
		 * @param bool Last Mirrors First
		 * @param bool New/Edit title- grays track button
		 * @param QString CDID
		 * @param QString Path To Picture
		 */
		titleDialog(QWidget *, const QString &, const QString &, const QString &, const bool &, const bool &, const QString &, const QString &);
		/**
		 * Destructor
		 */
		~titleDialog();
		/**
		 * Returns a pointer to the trackDialog.
		 */
		trackDialog *track() const;
		/**
		 * Returns a pointer to titleWidget.
		 */
		titleWidget *widget() const;
	protected slots:
		/**
		 * Closes dialog. No settings applied.
		 */
		void slotCancel();
		/** Checks values and then emits addTitle(titleInfo).
		 */
		void slotOk();
		/**
		 * Re-implemented for viewing picture dialog.
		 */
		void slotUser1();
		/**
		 * Re-implemented for editing tracks.
		 */
		void slotUser2();
	private slots:
		/**
		 * Updates the number of tracks in the titleDialog.
		 */
		void updateTracks() const;
	signals:
		/** Emitted when you need to return current title info.
		 * @param titleInfo titleInfo Struct
		 */
		void addTitle(titleInfo);
		/**
		 * Emitted when the "Cancel" is clicked. This helps keep track of the titleDialog pointers.
		 * @param titleDialog Pointer To Self
		 */
		void cancelled(titleDialog *);
		/**
		 * Emitted when "Cancel" is clicked.
		 */
		void relistTitles();
		/**
		 * Emitted from track dialog is opened so it can be populated with track information.
		 * @param QListView Pointer To trackQListView
		 * @param QString CDID
		 */
		void updateTracks(QListView *, QString);
	private:
		titleWidget *title;
		picDialog *picD;
		titleInfo test;
		trackDialog *__track;
};

/**
 * Used to keep track of pointers to titleDialog's.
 */
typedef QMap<titleDialog*,int> TitleDialogManager;


#endif

