/***************************************************************************
                          titlewidget.cpp  -  description
                             -------------------
    begin                : Tue Apr 10 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "titlewidget.h"

/**
 * Constructor
 *
 * @param QWidget Parent
 * @param int Spacing
 * @param bool Last Mirrors First
 */
titleWidget::titleWidget(QWidget *parent, const int &spacing,
				const bool &z): QGridLayout(parent, 12, 4, 0, spacing)
{
	cerr << "titleWidget::initialized" << endl;

	label = new QLabel(i18n("Title"), parent);
	addWidget(label, 0, 0);
	name = new KLineEdit("", parent);
	name->setMaxLength(55);
	QToolTip::add(name, i18n("Enter the name of the title"));
	addMultiCellWidget(name, 0, 0, 1, 3);

	label = new QLabel(i18n("Year"), parent);
	addWidget(label, 1, 0);
	start = new KLineEdit("", parent);
	start->setMaxLength(4);
	start->setMinimumWidth(start->fontMetrics().width("xxxx") + 20);
	start->setMaximumWidth(start->fontMetrics().width("xxxx") + 20);
	QToolTip::add(start, i18n("Enter the starting year"));
	addWidget(start, 1, 1);

	label = new QLabel(i18n("Year End"), parent);
	addWidget(label, 2, 0);
	end = new KLineEdit("", parent);
	end->setMaxLength(4);
	end->setMinimumWidth(end->fontMetrics().width("xxxx") + 20);
	end->setMaximumWidth(end->fontMetrics().width("xxxx") + 20);
	QToolTip::add(end, i18n("Put the ending year"));
	addWidget(end, 2, 1);

	label = new QLabel(i18n("No. Media"), parent);
	addWidget(label, 3, 0);
	nodiscs = new KLineEdit("", parent);
	nodiscs->setMaxLength(2);
	nodiscs->setMinimumWidth(nodiscs->fontMetrics().width("xx") + 20);
	nodiscs->setMaximumWidth(nodiscs->fontMetrics().width("xx") + 20);
	QToolTip::add(nodiscs, i18n("Enter the total number of media"));
	addWidget(nodiscs, 3, 1);

	label = new QLabel(i18n("Collected?"), parent);
	addWidget(label, 6, 0);
	collected = new QCheckBox("", parent);
	QToolTip::add(collected,i18n("If this title is already a part of your collection put a check here"));
	addWidget(collected, 6, 1);

	label = new QLabel(i18n("On Loan?"), parent);
	addWidget(label, 7, 0);
	loan = new QCheckBox("",parent);
	QToolTip::add(loan,i18n("Is this on loan?"));
	addWidget(loan, 7, 1);

	label = new QLabel(i18n("Format"), parent);
	addWidget(label, 1, 2);
	format = new KComboBox(0, parent, "format");
	QToolTip::add(format, i18n("Select A Format"));
	addWidget(format, 1, 3);


	label = new QLabel(i18n("Box Set"), parent);
	addWidget(label, 2, 2);
	boxset = new KComboBox(0, parent, "boxset");
	QToolTip::add(boxset, i18n("If this title is part of a box set, select which one."));
	addWidget(boxset, 2, 3);


	label = new QLabel(i18n("Genre"), parent);
	addWidget(label, 3, 2);
	genre = new KComboBox(0, parent, "genre");
	QToolTip::add(genre, i18n("Select A Genre"));
	addWidget(genre, 3, 3);

	label = new QLabel(i18n("Label"), parent);
	addWidget(label, 4, 2);
	rlabel = new KComboBox(0, parent, "tlabel");
	QToolTip::add(rlabel, i18n("Select A Label"));
	addWidget(rlabel, 4, 3);

	label = new QLabel(i18n("Method"), parent);
	addWidget(label, 5, 2);
	method = new KComboBox(0, parent, "method");
	QToolTip::add(method, i18n("Select A Method"));
	addWidget(method, 5, 3);

	label = new QLabel(i18n("Type"), parent);
	addWidget(label, 6, 2);
	type = new KComboBox(0, parent, "type");
	QToolTip::add(type, i18n("Select A Type"));
	addWidget(type, 6, 3);

	label = new QLabel(i18n("No. Tracks:"), parent);
	addWidget(label, 4, 0);
	notracks = new QLabel("", parent);
	QToolTip::add(notracks, i18n("The total number of tracks"));
	addWidget(notracks, 4, 1);

	label = new QLabel(i18n("Length:"), parent);
	addWidget(label, 5, 0);
	length = new QLabel("", parent);
	QToolTip::add(length, i18n("The total length of this title"));
	addWidget(length, 5, 1);

	label = new QLabel(i18n("Other"), parent);
	addWidget(label, 7, 2);
	code = new KLineEdit("", parent);
	QToolTip::add(code,i18n("Customer catalogue or lookup code."));
	addWidget(code, 7, 3);

	label = new QLabel(i18n("Rating:"), parent);
	addWidget(label, 8, 0);
	ratinglabel = new QLabel("0", parent);
	addWidget(ratinglabel, 8, 1);
	rating = new QSlider(1, 10, 1, 5, Qt::Horizontal, parent);
	rating->setTickmarks(QSlider::Above);
	QToolTip::add(rating,i18n("From 1 to 10, how much do you like this title?"));
	addMultiCellWidget(rating, 8, 8, 2, 3);
	connect(rating, SIGNAL(valueChanged(int)), ratinglabel, SLOT(setNum(int)));

	label = new QLabel(i18n("Quality:"), parent);
	addWidget(label, 9, 0);
	qualitylabel = new QLabel("0", parent);
	addWidget(qualitylabel, 9, 1);
	quality = new QSlider(1, 10, 1, 5, Qt::Horizontal, parent);
	quality->setTickmarks(QSlider::Above);
	QToolTip::add(quality,i18n("From 1 to 10, how well was this title recorded/produced?"));
	addMultiCellWidget(quality, 9, 9, 2, 3);
	connect(quality, SIGNAL(valueChanged(int)), qualitylabel, SLOT(setNum(int)));

	notes = new QTextEdit(parent);
	label = new QLabel(i18n("Notes"), parent);
	addWidget(label, 10, 0);
	notes->setWordWrap(QTextEdit::WidgetWidth);
	addMultiCellWidget(notes, 11, 11, 0, 3);

	name->setFocus();

	if (z)
		connect(start, SIGNAL(textChanged(const QString &)), end, SLOT(setText(const QString &)));
}

/**
 * Destructor
 */
titleWidget::~titleWidget()
{
}

/** Returns value of boxset name.
 */
QString
titleWidget::getBoxset() const
{
	return boxset->currentText();
}

/** Returns value of CDID.
 */
QString
titleWidget::getCDID() const
{
	return localcdid;
}

/** Returns value of code.
 */
QString titleWidget::getCode() const
{
	return code->text();
}

/** Returns value of collected.
 */
bool
titleWidget::getCollected() const
{
	return collected->isChecked();
}

/** Returns value of format name.
 */
QString
titleWidget::getFormat() const
{
	return format->currentText();
}

/** Returns value of genre name.
 */
QString
titleWidget::getGenre() const
{
	return genre->currentText();
}

/** Returns value of label name.
 */
QString
titleWidget::getLabel() const
{
	return rlabel->currentText();
}

/** Returns value of length.
 */
QString
titleWidget::getLength() const
{
	return length->text();
}

/** Returns whether loan is checked or not.
 */
bool titleWidget::getLoan() const
{
	return loan->isChecked();
}

/** Returns value of method name.
 */
QString
titleWidget::getMethod() const
{
	return method->currentText();
}

/** Returns value of name.
 */
QString
titleWidget::getName() const
{
	return name->text();
}

/** Returns number of media.
 */
QString
titleWidget::getNod() const
{
	return nodiscs->text();
}

/** Returns value of notes.
 */
QString
titleWidget::getNotes() const
{
	return notes->text();
}

/** Returns number of tracks.
 */
QString
titleWidget::getNotracks() const
{
	return notracks->text();
}

/** Retrieves the quality value from 1-10. */
int
titleWidget::getQuality() const
{
	return quality->value();
}

/** Retrieve the rating value from 1-10. */
int
titleWidget::getRating() const
{
	return rating->value();
}

/** Returns value of type name.
 */
QString
titleWidget::getType() const
{
	return type->currentText();
}

/** Returns value of year end.
 */
QString
titleWidget::getYearend() const
{
	return end->text();
}

/** Returns value of year start.
 */
QString
titleWidget::getYearstart() const
{
	return start->text();
}

/** Grays or ungrays the entire widget.
 * @param bool TRUE enables, FALSE disables
 */
void
titleWidget::enable(bool t) const
{
	boxset->setEnabled(t);
	collected->setEnabled(t);
	end->setEnabled(t);
	format->setEnabled(t);
	genre->setEnabled(t);
	method->setEnabled(t);
	rlabel->setEnabled(t);
	name->setEnabled(t);
	nodiscs->setEnabled(t);
	notes->setEnabled(t);
	notracks->setEnabled(t);
	start->setEnabled(t);
	type->setEnabled(t);
	rating->setEnabled(t);
	quality->setEnabled(t);
	code->setEnabled(t);
	loan->setEnabled(t);
}

/** Populates the KComboBox's with the QStringList.
 * @param int Category Type defined in include.h
 * @param QStringList Pointer To QStringList
 */
void
titleWidget::populateCategoryBox(const int &t, QStringList *z) const
{
	QString savedname = "";
	int savedindex = 0;
	int found = 0;
	KComboBox * temp = 0;
	switch(t)
	{
		case KBOXSET:
			temp = boxset;
			break;
		case KFORMAT:
			temp = format;
			break;
		case KGENRE:
			temp = genre;
			break;
		case KLABEL:
			temp = rlabel;
			break;
		case KMETHOD:
			temp = method;
			break;
		case KTYPE:
			temp = type;
			break;
		default:
			return;
	};

	savedname = temp->currentText();
	savedindex = temp->currentItem();
	temp->clear();
	temp->insertStringList(*z);
	for (int i = 0; i < temp->count(); i++)
	{
		if (savedname == temp->text(i))
		{
			temp->setCurrentItem(i);
			found = 1;
			break;
		}
	}
	if (!found)
		temp->setCurrentItem(savedindex);

}

/** Sets the number of tracks to the passed value.
 * @param int Number Of Tracks
 */
void
titleWidget::setNumTracks(int t) const
{
	notracks->setText(QString::number(t));
}

/** Updates the information.
 * @param titleInfo Pointer To Struct
 * @param bool Fill in current year for first
 * @param bool Last Mirrors First
 */
void titleWidget::updateInfo(titleInfo *b, const bool &x, const bool &z)
{
	format->setCurrentItem(format->listBox()-> index(format->listBox()->findItem(b->cat[KFORMAT].name)));
	genre->setCurrentItem(genre->listBox()-> index(genre->listBox()->findItem(b->cat[KGENRE].name)));
	rlabel->setCurrentItem(rlabel->listBox()-> index(rlabel->listBox()->findItem(b->cat[KLABEL].name)));
	method->setCurrentItem(method->listBox()-> index(method->listBox()->findItem(b->cat[KMETHOD].name)));
	type->setCurrentItem(type->listBox()-> index(type->listBox()->findItem(b->cat[KTYPE].name)));
	boxset->setCurrentItem(boxset->listBox()-> index(boxset->listBox()->findItem(b->cat[KBOXSET].name)));

	localcdid = b->cdid;
	name->setText(b->name);

	if (x)
		start->setText(b->yearstart.setNum(QDate::currentDate().year()));
	else
		start->setText(b->yearstart);

	if (z)
		end->setText(b->yearstart);
	else
		end->setText(b->yearend);

	nodiscs->setText(b->nod);

	if (b->notracks.isEmpty())
		notracks->setText("0");
	else
		notracks->setText(b->notracks);

	collected->setChecked(b->collected);

	notes->setText(b->notes);
	length->setText(b->length);
	rating->setValue((b->rating).toInt());
	ratinglabel->setText(b->rating);
	quality->setValue((b->quality).toInt());
	qualitylabel->setText(b->quality);
	code->setText(b->code);
	loan->setChecked(b->loan);
}

#include "titlewidget.moc"
