/***************************************************************************
                          titlewidget.h  -  description
                             -------------------
    begin                : Tue Apr 10 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TITLEWIDGET_H
#define TITLEWIDGET_H

#include "include.h"

/**This is the main widget for titles. It is a QGridLayout containing the titleInfo values.
 *
 *Embed it anywhere you need it (dialog, preferences box).
 * @author Will DeRousse
 * @short Add/Edit Widget for title information.
 *<pre>
 *QFrame *page=new QFrame();
 *titleWidget *title=new titleWidget(page, spacing(), true, true);
 *</pre>
 */

class titleWidget : public QGridLayout  {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 *
		 * @param QWidget Parent
		 * @param int Spacing
		 * @param bool Last Mirrors First
		 */
		titleWidget(QWidget *, const int &, const bool &);
		/**
		 * Destructor
		 */
		~titleWidget();
		/** Returns value of boxset name.
		 */
		QString getBoxset() const;
		/** Returns value of CDID.
		 */
		QString getCDID() const;
		/** Returns value of code.
		 */
		QString getCode() const;
		/** Returns value of collected.
		 */
		bool getCollected() const;
		/** Returns value of format name.
		 */
		QString getFormat() const;
		/** Returns value of genre name.
		 */
		QString getGenre() const;
		/** Returns value of label name.
		 */
		QString getLabel() const;
		/** Returns value of length.
		 */
		QString getLength() const;
		/** Returns whether loan is checked or not.
		 */
		bool getLoan() const;
		/** Returns value of method name.
		 */
		QString getMethod() const;
		/** Returns value of name.
		 */
		QString getName() const;
		/** Returns number of media.
		 */
		QString getNod() const;
		/** Returns value of notes.
		 */
		QString getNotes() const;
		/** Returns number of tracks.
		 */
		QString getNotracks() const;
		/**
		 * Retrieves the quality value from 1-10.
		 */
		int getQuality() const;
		/**
		 * Retrieve the rating value from 1-10.
		 */
		int getRating() const;
		/** Returns value of type name.
		 */
		QString getType() const;
		/** Returns value of year end.
		 */
		QString getYearend() const;
		/** Returns value of year start.
		 */
		QString getYearstart() const;
		/** Populates the KComboBox's with the QStringList.
		 * @param int Category Type defined in include.h
		 * @param QStringList Pointer To QStringList
		 */
		void populateCategoryBox(const int &, QStringList *) const;
		/** Sets the number of tracks to the passed value.
		 * @param int Number Of Tracks
		 */
		void setNumTracks(int) const;
		/** Updates the information.
		 * @param titleInfo Pointer To Struct
		 * @param bool Fill in current year for first
		 * @param bool Last Mirrors First
		 */
		void updateInfo(titleInfo *, const bool &, const bool &);
	public slots:
		/** Grays or ungrays the entire widget.
		 * @param bool TRUE enables, FALSE disables
		 */
		void enable(bool=false) const;
	private:
		QString localcdid;
		QLabel *label,*length,*notracks,*ratinglabel,*qualitylabel;
		KLineEdit *name,*start,*end,*nodiscs, *code;
		QCheckBox *collected, *loan;
		KComboBox *genre,*method,*format,*rlabel,*type,*boxset;
		QTextEdit *notes;
		QSlider *quality, *rating;
};


#endif

