/***************************************************************************
                          trackdialog.cpp  -  description
                             -------------------
    begin                : Thu May 31 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "trackdialog.h"

/**
 * Constructor
 * @param QWidget Parent
 * @param QString Caption Name
 * @param QString CDID
 */
trackDialog::trackDialog(QWidget *parent, const QString &caption, const QString &c):
KDialogBase(Plain, i18n("Edit Tracks for %1").arg(caption), Ok | Close | User1 | User2 | User3, Ok, parent, "pic", false, false)
{
	cerr << "trackDialog::initialized" << endl;
	CDID=c;
	QFrame *page = plainPage();
	setButtonBoxOrientation(Vertical);

	setButtonText(Ok, i18n("&Add"));
	setButtonText(User1, i18n("&Update"));
	setButtonText(User2, i18n("&Delete"));
	setButtonText(User3, i18n("&Lyrics"));
	setButtonTip(Ok,i18n("Click here to add track"));
	setButtonTip(User1,i18n("Click here to update track"));
	setButtonTip(User2,i18n("Click here to delete track"));
	setButtonTip(User3,i18n("Click here to view or change lyrics"));

	track = new trackWidget(page, spacingHint());

	enableButton(Ok, FALSE);
	enableButtons(FALSE);

	connect(track, SIGNAL(nameTextChanged(const QString &)), this, SLOT(enableAdd(const QString &)));
	connect(track->listview(), SIGNAL(imFocused(bool)),this,SLOT(enableButtons(bool)));
	connect(track->listview(),SIGNAL(doubleClicked(QListViewItem *)),this,SLOT(slotUser3()));

	connect(track, SIGNAL(signalTrackDown(trackInfo, trackInfo,QListView*)), this, SLOT(slotTrackDown(trackInfo, trackInfo,QListView*)));
	connect(track, SIGNAL(signalTrackUp(trackInfo, trackInfo,QListView*)), this, SLOT(slotTrackUp(trackInfo, trackInfo,QListView*)));

	connect(track->listview(), SIGNAL(dropped(trackInfo)), this, SLOT(droppedtrackInfo(trackInfo)));

}

/**
 * Destructor
 */
trackDialog::~trackDialog()
{
}

/**
 * Slot for DnD data.
 * @param trackInfo Struct that has been dropped
 */
void trackDialog::droppedtrackInfo(trackInfo t)
{
	cerr << "trackDialog::dnd track info dropped" << endl;
	t.trackid="";
	emit trackDropped(CDID,t,track->listview());
}

/** For graying and ungraying of the "Add" button, depending on what text is found in the KLineEdit.
 * @param QString KLineEdit text
 */
void
trackDialog::enableAdd(const QString &t)
{
	enableButton(Ok, !t.isEmpty());
}

/**
 * Used to enable buttons that are not "Add".
 * @param bool TRUE enables, FALSE disables
 */
void trackDialog::enableButtons(bool z)
{
	enableButton(User1, z);
	enableButton(User2, z);
	enableButton(User3, z);
	track->enableButtons(z);
}

/**
 * Re-implemented resize event.
 * @param QResizeEvent Resize Event
 */
void
trackDialog::resizeEvent(QResizeEvent *)
{
	cerr << "trackDialog::resizeEvent" << endl;
	track->resizeColumns();
}

/**
 * Re-implemented as Add.
 */
void
trackDialog::slotOk()
{
	trackInfo t;
	t.name = track->getName();
	t.side = track->getSide();
	t.notes = track->getNotes();
	t.length = returnLength(track->getHour(), track->getMinute(), track->getSecond());
	t.trackid = "";
	emit addTrack(CDID,t,track->listview());
	track->clearName();
}

/**
 * Emits signalTrackDown.
 * @param trackInfo Top Track
 * @param trackInfo Bottom Track
 * @param QListView Returns QListView
 */
void trackDialog::slotTrackDown(trackInfo a, trackInfo b, QListView *c)
{
	emit signalTrackDown(CDID,a,b,c);
}

/**
 * Emits signalTrackUp.
 * @param trackInfo Top Track
 * @param trackInfo Bottom Track
 * @param QListView Returns QListView
 */
void trackDialog::slotTrackUp(trackInfo a, trackInfo b, QListView *c)
{
	emit signalTrackUp(CDID,a,b,c);
}
/** Used to implement error handling for updating lyrics.
 * @param QString Lyric String
 * @param QString Track ID
 */
void trackDialog::slotUpdateLyric(QString l, QString id)
{
	bool s=FALSE;
	emit updateLyric(l,id,&s);
	if (!s)
		emit relistTracks(track->listview(),CDID);
}

/** Re-implemented as Update. */
void
trackDialog::slotUser1()
{
	if (track->listview()->selectedItem() == 0)
		return;
	trackInfo t;
	t.name = track->getName();
	t.side = track->getSide();
	t.notes = track->getNotes();
	t.length = returnLength(track->getHour(), track->getMinute(), track->getSecond());
	t.trackid = track->getTrackID();
	emit updateTrack(CDID,t,track->listview());
}

/** Re-implemented as Delete. */
void
trackDialog::slotUser2()
{
	if (track->listview()->selectedItem() == 0)
		return;
	emit deleteTrack(track->getTrackID(),CDID,track->listview());
}

void trackDialog::slotUser3()
{
	bool goodreturn;
	lyricDialog *lyric=new lyricDialog(this, i18n("Lyrics for %1").arg(track->getName()),track->getTrackID());
	if (!lyric)
		exit(1);
	connect(lyric,SIGNAL(updateLyric(QString,QString)),this,SLOT(slotUpdateLyric(QString,QString)));
	emit getLyric(lyric->textEdit(),track->getTrackID(), &goodreturn);
	if (goodreturn) {
		lyric->show();
	} else {
		delete lyric;
		emit relistTracks(track->listview(), CDID);
	}
}
/**
 * Updates the title name in the track dialog caption.
 *
 * updateCaption("rockalbum"); would result in a caption that reads:
 *
 * Edit Tracks for rockalbum
 */
void
trackDialog::updateCaption(const QString &c)
{
	setCaption(i18n("Edit Tracks for %1").arg(c));
}

/** Returns a pointer to the trackWidget's listview. */
trackWidget *trackDialog::widget() const
{
	return track;
}

#include "trackdialog.moc"
