/***************************************************************************
                          trackdialog.h  -  description
                             -------------------
    begin                : Thu May 31 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TRACKDIALOG_H
#define TRACKDIALOG_H

#include "include.h"
#include "trackwidget.h"
#include "lyricdialog.h"

/** This is the track dialog. It contains the trackWidget and its functionality along with the "Close", "Add", "Delete" and "Update" buttons.
 * @short Track Dialog
 * @author Will DeRousse
 *<pre>
 *trackDialog *w=new trackDialog(this,"album_name","512");
 *w->show();
 *</pre>
 */

class trackDialog : public KDialogBase  {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 * @param QString Caption Name
		 * @param QString CDID
		 */
		trackDialog(QWidget *, const QString &, const QString &);
		/**
		 * Destructor
		 */
		~trackDialog();
		/**
		 * Updates the title name in the track dialog caption.
		 *
		 * updateCaption("rockalbum"); would result in a caption that reads:
		 *
		 * Edit Tracks for rockalbum
		 */
		void updateCaption(const QString &);
		/**
		 * Returns a pointer to trackWidget.
		 */
		trackWidget *widget() const;
	protected slots:
		/**
		 * Slot for DnD data.
		 * @param trackInfo Struct that has been dropped
		 */
		void droppedtrackInfo(trackInfo);
		/** For graying and ungraying of the Add button, depending on what text is found in the KLineEdit.
		 * @param QString KLineEdit text
		 */
		void enableAdd(const QString &);
		/**
		 * Re-implemented resize event.
		 * @param QResizeEvent Resize Event
		 */
		void resizeEvent(QResizeEvent*);
		/**
		 * Re-implemented as Add.
		 */
		void slotOk();
		/**
		 * Emits signalTrackDown.
		 * @param trackInfo Top Track
		 * @param trackInfo Bottom Track
		 * @param QListView Returns QListView
		 */
		void slotTrackDown(trackInfo, trackInfo, QListView *);
		/**
		 * Emits signalTrackUp.
		 * @param trackInfo Top Track
		 * @param trackInfo Bottom Track
		 * @param QListView Returns QListView
		 */
		void slotTrackUp(trackInfo, trackInfo, QListView *);
		/** Used to implement error handling for updating lyrics.
		 * @param QString Lyric String
		 * @param QString Track ID
		 */
		void slotUpdateLyric(QString, QString);
		/**
		 * Re-implemented as Update.
		 */
		void slotUser1();
		/**
		 * Re-implemented as Delete.
		 */
		void slotUser2();
		/**
		 * Re-implemented to open Lyric Dialog.
		 */
		void slotUser3();
	private slots:
		/**
		 * Used to enable buttons that are not "Add".
		 * @param bool TRUE enables, FALSE disables
		 */
		void enableButtons(bool);
	signals:
		/**
		 * Add track signal.
		 * @param QString CDID
		 * @param trackInfo trackInfo Struct
		 * @param QListView returns QListView
		 */
		void addTrack(QString, trackInfo, QListView *);
		/** Delete track signal.
		 * @param QString Track ID
		 * @param QString CDID
		 * @param QListView Returns QListView
		 */
		void deleteTrack(QString, QString, QListView *);
		/**
		 * Emitted in order to retrieve and populate lyrics into QTextEdit.
		 * @param QTextEdit Pointer To QTextEdit
		 * @param QString Track ID
		 * @param bool Used to check if there was a successful retrieval of lyrics.
		 */
		void getLyric(QTextEdit *, QString, bool *);
		/** Relists tracks.
		 * @param QListView Pointer To QListView
		 * @param QString CDID
		 */
		void relistTracks(QListView *, QString);
		/** Track down signal.
		 * @param QString CDID
		 * @param trackInfo Top Track
		 * @param trackInfo Bottom Track
		 * @param QListView Returns QListView
		 */
		void signalTrackDown(QString, trackInfo, trackInfo, QListView *);
		/** Track up signal.
		 * @param QString CDID
		 * @param trackInfo Top Track
		 * @param trackInfo Bottom Track
		 * @param QListView Returns Q	ListView
		 */
		void signalTrackUp(QString, trackInfo, trackInfo, QListView *);
		/**
		 * Emittted when a track is dropped.
		 * @param QString CDID
		 * @param trackInfo Struct With Track Information
		 * @param QListView Pointer To QListView
		 */
		void trackDropped(QString, trackInfo, QListView *);
		/**
		 * Emits updated lyrics.
		 * @param QString Lyric
		 * @param QString Track ID
		 * @param bool Success
		 */
		void updateLyric(QString, QString, bool *);
		/**
		 * Update track signal.
		 * @param QString CDID
		 * @param trackInfo trackInfo Struct
		 * @param QListView Returns QListView
		 */
		void updateTrack(QString, trackInfo, QListView *);
	private:
		trackWidget *track;
		QString CDID;
};


#endif

