/***************************************************************************
                          trackwidget.cpp  -  description
                             -------------------
    begin                : Thu May 31 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "trackwidget.h"

/**
 * Constructor
 * @param QWidget Parent
 * @param int Spacing
 */
trackWidget::trackWidget(QWidget *parent, const int &spacing):
QGridLayout(parent, 5, 6, 0, spacing)
{
	cerr << "trackWidget::initialized" << endl;
	label = new QLabel(i18n("Side"), parent);
	label->setAlignment(Qt::AlignCenter);
	addWidget(label, 0, 1);
	label = new QLabel(i18n("Name"), parent);
	label->setAlignment(Qt::AlignCenter);
	addWidget(label, 0, 2);
	label = new QLabel(i18n("Hour"), parent);
	label->setAlignment(Qt::AlignCenter);
	addWidget(label, 0, 3);
	label = new QLabel(i18n("Minute"), parent);
	label->setAlignment(Qt::AlignCenter);
	addWidget(label, 0, 4);
	label = new QLabel(i18n("Second"), parent);
	label->setAlignment(Qt::AlignCenter);
	addWidget(label, 0, 5);

	side = new KLineEdit("", parent);
	name = new KLineEdit("", parent);
	name->setMaxLength(60);
	hour = new QSpinBox(parent);
	minute = new QSpinBox(parent);
	second = new QSpinBox(parent);
	hour->setMaxValue(23);
	minute->setMaxValue(59);
	second->setMaxValue(59);
	side->setMinimumWidth(20);
	int w = hour->fontMetrics().width("Side") + 20;
	side->setMaximumWidth(w);
	name->setMinimumWidth(120);
	w = hour->fontMetrics().width("xxx") + 20;
	hour->setMinimumWidth(w);
	hour->setMaximumWidth(w);
	minute->setMinimumWidth(w);
	minute->setMaximumWidth(w);
	second->setMinimumWidth(w);
	second->setMaximumWidth(w);
	hour->setWrapping(TRUE);
	minute->setWrapping(TRUE);
	second->setWrapping(TRUE);
	addWidget(side, 1, 1);
	addWidget(name, 1, 2);
	addWidget(hour, 1, 3);
	addWidget(minute, 1, 4);
	addWidget(second, 1, 5);

	notes = new KLineEdit("", parent);
	addMultiCellWidget(notes,2,2,1,5);

	QVBox *bgroup = new QVBox(parent);
	up = new QPushButton(bgroup);
	down = new QPushButton(bgroup);
	up->setPixmap(BarIcon("up"));
	down->setPixmap(BarIcon("down"));
	addMultiCellWidget(bgroup, 3, 4, 0, 0);

	list = new musicQListView(parent, 2);
	list->addColumn(i18n("Side"));
	list->addColumn(i18n("No."));
	list->addColumn(i18n("Name"));
	list->addColumn(i18n(" Length "));
//	list->addColumn(i18n(" Notes "));
//	list->addColumn(i18n(" TrackID "));
	list->setColumnWidth(0, list->fontMetrics().width(" Side "));
	list->setColumnWidth(1, list->fontMetrics().width(" No. "));
	list->setColumnWidth(3, list->fontMetrics().width(" 00:00:00 "));
	list->setMinimumHeight(280);
//	list->setSorting(0, TRUE);
	list->setVScrollBarMode(QScrollView::AlwaysOn);
	list->setColumnWidthMode(2,QListView::Manual);
	list->setColumnWidth(4, 0);
	list->setColumnWidth(5, 0);
	addMultiCellWidget(list, 3, 4, 1, 5);
	QToolTip::add(down, i18n("Move Track Down"));
	QToolTip::add(hour, i18n("Change Hours"));
	QToolTip::add(list, i18n("Track List"));
	QToolTip::add(minute, i18n("Change Minutes"));
	QToolTip::add(second, i18n("Change Seconds"));
	QToolTip::add(side, i18n("Enter Side"));
	QToolTip::add(name, i18n("Enter or edit name"));
	QToolTip::add(up, i18n("Move Track Up"));

	// get stretching to work correctly
	setRowStretch(4, 10);

	// set focus on the name, convenient
	name->setFocus();
	side->setText("1");

	connect(list, SIGNAL(selectionChanged(QListViewItem *)), this, SLOT(updateName(QListViewItem *)));
	connect(name, SIGNAL(textChanged(const QString &)), this, SIGNAL(nameTextChanged(const QString &)));

	connect(up, SIGNAL(clicked()), this, SLOT(trackUp()));
	connect(down, SIGNAL(clicked()), this, SLOT(trackDown()));
}

/**
 * Destructor
 */
trackWidget::~trackWidget()
{
}

/** Clears the name and sets the spin box's to zero. */
void
trackWidget::clearName() const
{
	name->setText("");
	notes->setText("");
	hour->setValue(0);
	minute->setValue(0);
	second->setValue(0);
	name->setFocus();
}

/** Grays the buttons.
 * @param bool TRUE enables, FALSE disables
 */
void
trackWidget::enableButtons(const bool &t) const
{
	up->setEnabled(t);
	down->setEnabled(t);
}

/** Returns the hour value. */
int
trackWidget::getHour() const
{
	return hour->value();
}

/** Returns the minute value. */
int
trackWidget::getMinute() const
{
	return minute->value();
}

/** Returns the track name. */
QString
trackWidget::getName() const
{
	return name->text();
}

/**
 * Returns the track notes.
 */
QString trackWidget::getNotes() const
{
	return notes->text();
}

/** Returns the second. */
int
trackWidget::getSecond() const
{
	return second->value();
}

/** Returns the side. */
QString trackWidget::getSide() const
{
	return side->text();
}

/** Returns the Track ID. */
QString
trackWidget::getTrackID() const
{
	return list->currentItem()->text(5);
}

/** Returns a pointer to the listview. */
musicQListView *trackWidget::listview() const
{
	return list;
}

/**
 * Resizes columns to be more aesthetic.
 * Needs to be manually called from time to time.
 */
void trackWidget::resizeColumns()
{
	int physicalwidth = list->size().width();
	int columnwidth = list->columnWidth(0);
	columnwidth += list->columnWidth(1);
	columnwidth += list->columnWidth(3);
	columnwidth += list->verticalScrollBar()->width();
	columnwidth += 6;
	cerr << "trackWidget::resizeColumns(): " << physicalwidth << "/" << columnwidth << endl;
	if (physicalwidth > columnwidth)
		list->setColumnWidth(2, physicalwidth - columnwidth);
	else
		list->setColumnWidth(2, 50);
}

/** This slot emits the track down signal. */
void
trackWidget::trackDown()
{
	QListViewItem *topitem, *bottomitem;
	topitem = list->selectedItem();
	if (topitem == 0)
		return;

	bottomitem = list->selectedItem()->itemBelow();
	if (bottomitem == 0)
		return;

	trackInfo top, bottom;
	top.side = topitem->text(0);
	top.name = topitem->text(2);
	top.length = topitem->text(3);
	top.notes = topitem->text(4);
	top.trackid = topitem->text(5);
//	cerr << "top: side: " << topitem->text(0) << " name: " << topitem->text(2) << " length: " << topitem->text(3) << " notes: " << topitem->text(4) << " trackid: " << topitem->text(5) << endl;

	bottom.side = bottomitem->text(0);
	bottom.name = bottomitem->text(2);
	bottom.length = bottomitem->text(3);
	bottom.notes = bottomitem->text(4);
	bottom.trackid = bottomitem->text(5);
//	cerr << "btm: side: " << bottomitem->text(0) << " name: " << bottomitem->text(2) << " length: " << bottomitem->text(3) << " notes: " << bottomitem->text(4) << " trackid: " << bottomitem->text(5) << endl;

	emit signalTrackDown(bottom, top,list);
}

/** This slot emits the track up signal. */
void
trackWidget::trackUp()
{
	QListViewItem *topitem, *bottomitem;
	bottomitem = list->selectedItem();
	if (bottomitem == 0)
		return;

	topitem = list->selectedItem()->itemAbove();
	if (topitem == 0)
		return;

	trackInfo top, bottom;
	top.side = topitem->text(0);
	top.name = topitem->text(2);
	top.length = topitem->text(3);
	top.notes = topitem->text(4);
	top.trackid = topitem->text(5);
//	cerr << "top: side: " << topitem->text(0) << " name: " << topitem->text(2) << " length: " << topitem->text(3) << " notes: " << topitem->text(4) << " trackid: " << topitem->text(5) << endl;

	bottom.side = bottomitem->text(0);
	bottom.name = bottomitem->text(2);
	bottom.length = bottomitem->text(3);
	bottom.notes = bottomitem->text(4);
	bottom.trackid = bottomitem->text(5);
//	cerr << "btm: side: " << bottomitem->text(0) << " name: " << bottomitem->text(2) << " length: " << bottomitem->text(3) << " notes: " << bottomitem->text(4) << " trackid: " << bottomitem->text(5) << endl;

	emit signalTrackUp(bottom, top,list);
}

/** This tracks is used to update the name and spinbox values to match the just selected item.
 *
 * This slot also emits the listTextChanged signal.
 * @param QListViewItem Item that has changed
 */
void
trackWidget::updateName(QListViewItem *t)
{
	if (t == NULL)
	{
		// instead of setText, need to run it through some processing and set
		// all four widgets.
		side->setText("1");
		name->setText("");
		hour->setValue(0);
		minute->setValue(0);
		second->setValue(0);
	}
	else
	{
		// need to run it through some processing and set widgets.
		QString time = t->text(3);
		name->setText(t->text(2));
		notes->setText(t->text(4));
		side->setText(t->text(0));
		hour->setValue(returnHour(time));
		minute->setValue(returnMinute(time));
		second->setValue(returnSecond(time));
	}
}

#include "trackwidget.moc"
