/***************************************************************************
                          trackwidget.h  -  description
                             -------------------
    begin                : Thu May 31 2001
    copyright            : (C) 2001 by Will DeRousse
    email                : badhack@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TRACKWIDGET_H
#define TRACKWIDGET_H

#include "include.h"
#include "kmusicdbvalidator.h"
#include "musicqlistview.h"

/** This is the track widget. It contains the listview, up and down buttons, the KLineEdit and the 3 spin boxes.
 * @short Track Widget
 * @author Will DeRousse
 *<pre>
 *trackWidget *w=new trackWidget(this,spacing);
 *w->show();
 *</pre>
 */

class trackWidget : public QGridLayout  {
	Q_OBJECT
	public:
		/**
		 * Constructor
		 * @param QWidget Parent
		 * @param int Spacing
		 */
		trackWidget(QWidget *, const int &);
		/**
		 * Destructor
		 */
		~trackWidget();
		/**
		 * Clears the name and sets the spin box's to zero, sets focus to name.
		 */
		void clearName() const;
		/** Grays the buttons.
		 * @param bool TRUE enables, FALSE disables
		 */
		void enableButtons(const bool &) const;
		/**
		 * Returns the hour value.
		 */
		int getHour() const;
		/**
		 * Returns the minute value.
		 */
		int getMinute() const;
		/**
		 * Returns the track name.
		 */
		QString getName() const;
		/**
		 * Returns the track notes.
		 */
		QString getNotes() const;
		/**
		 * Returns the second.
		 */
		int getSecond() const;
		/**
		 * Returns the side.
		 */
		QString getSide() const;
		/**
		 * Returns the Track ID.
		 */
		QString getTrackID() const;
		/**
		 * Returns a pointer to the listview.
		 */
		musicQListView *listview() const;
		/**
		 * Resizes columns to be more aesthetic.
		 * Needs to be manually called from time to time.
		 */
		void resizeColumns();
	private slots:
		/**
		 * This slot emits the track down signal.
		 */
		void trackDown();
		/**
		 * This slot emits the track up signal.
		 */
		void trackUp();
		/** This tracks is used to update the name and spinbox values to match the just selected item.
		 *
		 * This slot also emits the listTextChanged signal.
		 * @param QListViewItem Pointer to item that has changed
		 */
		void updateName(QListViewItem *);
	signals:
		/** Name Text Changed signal.
		 * @param QString String value for name
		 */
		void nameTextChanged(const QString &);
		/** Track down button clicked.
		 * @param trackInfo Top Track
		 * @param trackInfo Bottom Track
		 * @param QListView Returns QListView
		 */
		void signalTrackDown(trackInfo, trackInfo, QListView *);
		/** Track up button clicked.
		 * @param trackInfo Top Track
		 * @param trackInfo Bottom Track
		 * @param QListView Returns QListView
		 */
		void signalTrackUp(trackInfo, trackInfo, QListView *);
	private:
		KLineEdit *side, *name, *notes;
		QSpinBox *hour, *minute, *second;
		musicQListView *list;
		QLabel *label;
		QPushButton *up, *down;
};


#endif

